##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Interfaces for apelib.sql.

$Id: interfaces.py,v 1.1 2004/06/28 18:08:30 johanc Exp $
"""

from Interface import Interface
from Interface.Attribute import Attribute

class IRDBMSConnection (Interface):
    """Interface of basic RDBMS connections.

    This interface provides only relatively basic operations.  Create
    subinterfaces for complex and vendor-specific extensions.
    """

    module = Attribute("module", "The DB-API module")

    connector = Attribute("connector", "The shared DB-API connection")

    def select(table, result_columns, **filter):
        """Selects rows from a table and returns column values for those rows.
        """

    def insert(table, columns, row):
        """Inserts one row in a table.
        """

    def set_one(table, oid, columns, row, is_new):
        """Sets one row in a table.

        Executes either an update or insert operation, depending
        on the is_new argument and configured policies.
        """

    def set_many(table, oid, key_columns, other_columns, rows):
        """Sets multiple rows in a table.

        'rows' is a sequence of tuples containing values for the
        key_columns as well as the other_columns.

        Either deletes all rows for an oid and inserts new rows, or
        examines the current state of the database and modifies it in
        pieces.
        """

    def delete_from(table, **filter):
        """Deletes rows from a table.
        """

    def exists(name, type_name):
        """Returns true if the specified database object exists.

        'name' is the name of the object.  'type_name' is 'table' or
        'sequence'.
        """

    def list_table_names():
        """Returns a list of existing table names.
        """

    def create_table(name, column_defs):
        """Creates a table.

        column_defs is [(name, type, is_unique)].
        """

    def drop_table(table):
        """Drops a table.
        """

    def create_sequence(name, start=1):
        """Creates a sequence.
        """

    def reset_sequence(name, start=1):
        """Resets a sequence to a starting value.
        """

    def increment(name):
        """Increments a sequence and returns the value.

        Whether the value is before or after the increment is not specified.
        """
