##############################################################################
#
# Copyright (c) 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Abstract gateways

$Id: sqlbase.py,v 1.1 2004/06/28 18:08:30 johanc Exp $
"""

from apelib.core.interfaces import IGateway, IDatabaseInitializer
from interfaces import IRDBMSConnection


class SQLGatewayBase:
    """SQL gateway base class"""

    __implements__ = IGateway, IDatabaseInitializer

    table = '(override this)'
    schema = None       # override
    column_defs = None  # optional override
    oid_column_def = (('oid', 'int', 1),)

    def __init__(self, conn_name='db'):
        self.conn_name = conn_name
        if self.column_defs is None and self.schema is not None:
            self.column_defs = tuple(self.schema.get_column_defs())
        self.columns = tuple([name for name, t, u in self.column_defs])

    def get_connection(self, event):
        return event.connections[self.conn_name]

    def create(self, event):
        conn = self.get_connection(event)
        defs = self.oid_column_def + self.column_defs
        conn.create_table(self.table, defs)

    def init(self, event):
        conn = self.get_connection(event)
        assert IRDBMSConnection.isImplementedBy(conn)
        if conn.exists(self.table, 'table'):
            if event.clear_all:
                conn.delete_from(self.table)
        else:
            self.create(event)

    def load(self, event):
        raise NotImplementedError, "abstract method"

    def store(self, event, obj):
        raise NotImplementedError, "abstract method"

    def get_sources(self, event):
        return None
