# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyApplication/EasyAppWorkflowManagement.py,v 1.1 2004/06/28 10:04:07 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""Overrides of the Workflow management interface for EasyApplications"""
__version__='$Revision: 1.1 $'[11:-2]


import Globals
from Globals import DTMLFile
from AccessControl import getSecurityManager, ClassSecurityInfo
from Acquisition import aq_inner, aq_parent


from Products.EasyWorkflow.EasyWorkflowBase import *
from Products.EasyCore.EasyCorePermissions import *
from Products.EasyWorkflow.EasyWorkflow import WorkflowManagement
from Products.EasyCore.BodyTabs import InitializeBodyTabs, UpdateTab, BodyTabs
from Products.EasyApplication.EasyApplicationTypeRegistry import EasyApplicationTypeRegistry


from Products.EasyCore import MLOG_TAB


def patchTrackedFields(tfs, **kw):
    new_tfs=[]
    keys=kw.keys()
    used_keys=[]
    for tf in tfs:
        id=tf['id']
        if id in keys:
            tf=kw[id]
            keys.remove(id)
        new_tfs.append(tf)
    for id in keys:
        new_tfs.append(kw[id])
    return tuple(new_tfs)



class EasyAppWorkflowManagement(WorkflowManagement):
    """\
    Overrides of the Workflow management interface for EasyApplications.
    """

    security = ClassSecurityInfo()

    ####
    # Workflow support
    ####


    track_fields = patchTrackedFields(WorkflowManagement.track_fields
            , show_types={'id':'show_types',             'type':'lines',     'default':EasyApplicationTypeRegistry.getWorkflowableTypes(), }
            , filtercolumns={'id':'filtercolumns',          'type':'lines',     'default':('type','published','state','title','author','effective_date','expiration_date')})


    security.declarePrivate('getWorkflowCatalog')
    def getWorkflowCatalog(self):
        return self.getApplicationObject()

    security.declarePublic('getMetaTypeKeys')
    def getMetaTypeKeys(self):
        return EasyApplicationTypeRegistry.getWorkflowableTypes()

    security.declarePublic('getMetaTypeItems')
    def getMetaTypeItems(self):
        return map(lambda x: (x, x) ,EasyApplicationTypeRegistry.getWorkflowableTypes())

    security.declarePrivate('getWorkflowScoop')
    def getWorkflowScoop(self, obj):
        return obj.getPhysicalPath()

    security.declarePublic('getPossibleColumn')
    def getPossibleColumn(self):
        return (('type','Type'),
                ('published','Published'),
                ('state','State'),
                ('title','Title'),
                ('virtualURL','URL'),
                ('author','Author'),
                ('published_by','Published By'),
                ('owner','Owner'),
                ('stateby','State By'),
                ('statedate','State Date'),
                ('creation_date','Created Date'),
                ('effective_date','Effective Date'),
                ('expiration_date','Expiration Date'),
            )


    security.declarePrivate('getValidInvalidBrains')
    def getValidInvalidBrains(self, urls, new_state):
        paths = self.urls2paths(urls)
        valid, invalid = self.getValidFromStates(new_state)
        searchquery = {}
        searchquery['physicalPath'] = {'query': paths }
        searchquery['getWorkflowState'] = {'query': valid}
        valid_res = self.searchResults(searchquery)
        searchquery['getWorkflowState'] = {'query': invalid}
        invalid_res = self.searchResults(searchquery)
        return valid_res, invalid_res

    #Was security.declareProtected(EasyWorkflowView,'getCatalogBrainsForUrls')
    security.declarePrivate('getCatalogBrainsForUrls')
    def getCatalogBrainsForUrls(self, urls):
        paths = self.urls2paths(urls)
        searchquery = {}
        searchquery['physicalPath'] = {'query': paths}
        res = self.searchResults(searchquery)
        return res


Globals.InitializeClass(EasyAppWorkflowManagement)
InitializeBodyTabs(EasyAppWorkflowManagement)
