# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyApplication/EasyApplication.py,v 1.1 2004/06/28 10:04:07 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__='''EasyApplication'''
__version__='$Revision: 1.1 $'[11:-2]

import Globals, Products
from Acquisition import aq_base, aq_parent, aq_inner
from Globals import DTMLFile
from AccessControl import ClassSecurityInfo, getSecurityManager
from ComputedAttribute import ComputedAttribute
from OFS.Folder import Folder
####from App.ProductContext import ProductContext
####from App.FactoryDispatcher import FactoryDispatcher
import urllib, time

#EasyWorkflow
from Products.EasyWorkflow.EasyPublishable import Publishable
from Products.EasyWorkflow.EasyWorkflow import WorkflowManagement
from Products.EasyWorkflow.EasyWorkflowBase import *

from Products.EasyPresentationHelpers.EasyPresentationHelpers import EasyPresentationHelpers
from Products.EasyLanguageService.EasyLanguageHelpers import getLanguageByRFC3066

#EasyNotification
from Products.EasyNotification.EasyNotifiable import EasyNotifiable
from Products.EasyNotification.EasyNotification import EASYNOTIFICATIONID, EasyNotification, getDefaultNotificationItems
from Products.EasyNotification.EasyNotificationInfo import EasyNotificationInfo

#
from Products.EasyCore.Utils.PersistentList import PersistentList
from Products.EasyCore.GUID import GUIDGenerator
from Products.EasyCore.BodyTabs import InitializeBodyTabs, UpdateTab, BodyTabs
from Products.EasyCore.Utils.PCounter import PCounter
from Products.EasyCore.Utils import Tutils
from Products.EasyCore.UpgradeSupport.Utils import upgrade_indexes
from Products.EasyCore.EasyCorePermissions import *
from Products.EasyCore.VirtualPath import VirtualPathBase


#EasyApplication.EasySecurityConf
from Products.EasyApplication.EasySecurityConf import EasyApplicationObjectView
from Products.EasyApplication.EasySecurityConf import EasyApplicationObjectAdd
from Products.EasyApplication.EasySecurityConf import EasyApplicationObjectChange
from Products.EasyApplication.EasySecurityConf import EasyApplicationViewManagementViews

#EasyApplication
from Products.EasyApplication.EasyApplicationGlobals import EasyAppConfigId, EasyObjectTemplateType, EasyPropertySheetType
from Products.EasyApplication.MultiCatalog import MultiCatalog
from Products.EasyApplication.EasyObject import EasyObject
from Products.EasyApplication.EasySecurityConf import EasyApplicationAdd, EasyApplicationClassManage
from Products.EasyApplication.EasyApplicationConf import EasyApplicationConf
from Products.EasyApplication.EasyApplicationTypeRegistry import EasyApplicationTypeRegistry
from Products.EasyApplication.EasyAppWorkflowManagement import EasyAppWorkflowManagement
from Products.EasyApplication.EasyCSVSupport import EasyCSVSupport

#interfaces
from Products.EasyApplication.interfaces.EasyObject import EasyObjectInterface
from Products.EasyCore.interfaces.EasyNotification import EasyNotificationInterface
from Products.EasyApplication.interfaces.EasyApplication import EasyApplicationInterface

from Products.EasyCore import MLOG_TAB

_marker=[]


DEFAULT_INDEXES = {}
REQUIRED_INDEXES = {
          'id':'FieldIndex'
        , 'title':'TextIndex'
        , 'meta_type':'FieldIndex'
        , 'bobobase_modification_time':'FieldIndex'
        , 'getWorkflowState':'FieldIndex'
        , 'getWorkflowStateLastChangedDate':'FieldIndex'
        , 'getWorkflowStateLastChangedBy':'FieldIndex'
        , 'creation_date':'FieldIndex'
        , 'effective_date':'FieldIndex'
        , 'expiration_date':'FieldIndex'
        , 'last_modified_by':'FieldIndex'
        , 'getCurrentLanguage':'FieldIndex'
        , 'subclass':'FieldIndex'
        , 'virtualURL':'FieldIndex'
        , 'virtualPath':'FieldIndex'
        , 'physicalURL':'FieldIndex'
        , 'physicalPath':'FieldIndex'
        , 'physicalPathIndex':'PathIndex'
}
DEPRECATED_INDEXES=('PrincipiaSearchSource','path')

DEFAULT_COLUMNS = ()
DEPRECATED_COLUMNS=('summary', )
REQUIRED_COLUMNS = (
          'id'
        , 'title'
        , 'meta_type'
        , 'bobobase_modification_time'
        , 'getWorkflowState'
        , 'getWorkflowStateLastChangedDate'
        , 'getWorkflowStateLastChangedBy'
        , 'creation_date'
        , 'effective_date'
        , 'expiration_date'
        , 'last_modified_by'
        , 'getCurrentLanguage'
        , 'subclass'
        , 'virtualURL'
        , 'virtualPath'
        , 'physicalURL'
        , 'physicalPath'
)


try:
    from Products.EasyCore.Utils.Tutils import distinct
except:
    def distinct(alist):    # Fastest order preserving
        set = {}
        return [set.setdefault(e,e) for e in alist if e not in set.keys()]



class Diff:
    def __init__(self, brain, obj, catalog):
        self.id=brain.id
        self.path=brain.physicalPath
        self.language=brain.getCurrentLanguage

        self.diffs={}
        for key, b_value in catalog.getMetadataForRID(brain.data_record_id_).items():
            if key not in ('sequence_item','getCurrentLanguage'):
                o_value=getattr(obj, key, _marker)
                if o_value is _marker:
                    self.diffs[key]='Only in brain.'
                else:
                    if callable(o_value): o_value=o_value()
                    if b_value!=o_value:
                        self.diffs[key]='Brain value (%s) differs from object value (%s).' % (b_value, o_value)

    def isDifferent(self):
        return len(self.diffs)>0


class SearchResult:
    def __init__(self, catalog):
        self.result={}
        self.catalog=catalog

    def diffObject(self, obj, path):
        brains=self.catalog.searchResults(physicalURL=obj.physicalURL, subclass='product')
        if brains:
            for brain in brains:
                if brain.getCurrentLanguage=='en':
                    self.result[brain.data_record_id_]=Diff(brain, obj, self.catalog)


class EasyApplication(    EasyNotifiable, MultiCatalog
                        , EasyAppWorkflowManagement, EasyPresentationHelpers
                        , EasyCSVSupport
                        , BodyTabs, VirtualPathBase):
    """Web application main class"""

    security = ClassSecurityInfo()

    meta_type = 'EasyApplication'

    # This is here as a "private" meta type. It will show up in "all_meta_types",
    # which is necessary for proper copy support, but it will not appear in
    # filtered_meta_types, and hence not in the add box.
    # This is all pretty much an undocemented and magical part of Zope.

    __interfaces__=(EasyApplicationInterface,)
    _product_meta_types = ({'name': 'EasyObject', 'meta_type':'EasyObject', 'action': ''}, )
    _product_interfaces = (EasyNotificationInterface, EasyObjectInterface,)


    #track_fields = patchTrackedFields(EasyAppWorkflowManagement.track_fields
    #        , show_types={'id':'show_types',             'type':'lines',     'default':EasyApplicationTypeRegistry.getWorkflowableTypes(), }
    #        , filtercolumns={'id':'filtercolumns',          'type':'lines',     'default':('type','published','state','title','author','effective_date','expiration_date')})

    def manage_options(self):
        content=MultiCatalog.manage_options[0].copy()
        #content['help']=('EasyWorkflow','Icons_Description.stx')
        options = ( content, ) +\
                  (
                      {'label': 'Workflow', 'action': 'manage_workflow',}
                   ,)

        if hasattr(self, 'EasyNotification'):
            options = options + ({'label': 'Notification', 'action': 'manage_notifications_form',},)

        options = options + (
                      {'label': 'Change Data', 'action': 'manage_changeData',}
                    , {'label': 'Languages', 'action': 'manage_language_mapping',}

                  )  + EasyCSVSupport.manage_options + MultiCatalog.manage_options[1:]
        return options


    _properties = (
                {'id':'default_language', 'type': 'string', 'mode': 'w'},
            )


    def __init__(self, id, title='', vocab_id=None, container=None):
        MultiCatalog.__init__(self, id, title, vocab_id, container)
        self._id_serial = PCounter()
        self._guid_generator=GUIDGenerator()
        self.default_language='en'
        self.language_order = PersistentList()
        # Standard indexes:
        # getCurrentLanguage
        # subclass
        #


        for id in DEPRECATED_INDEXES:
            self.delIndex(id, force=1, dont_raise=1)
        for id, type in REQUIRED_INDEXES.items():
            self.addIndex(id, type, undeletable=1)
        for id, type in DEFAULT_INDEXES.items():
            self.addIndex(id, type)

        for id in DEPRECATED_COLUMNS:
            self.delColumn(id, force=1, dont_raise=1)
        for id in REQUIRED_COLUMNS:
            self.addColumn(id, undeletable=1)
        for id in DEFAULT_COLUMNS:
            self.addColumn(id)


    #Redefine: permissions<->methods mapping
    security.declareProtected(EasyApplicationViewManagementViews,'manage_main')
    security.declareProtected(EasyApplicationClassManage, 'manage_addProperty')
    security.declareProtected(EasyApplicationClassManage, 'manage_delProperties')
    security.declareProtected(EasyApplicationClassManage, 'manage_propertiesForm')
    security.declareProtected(EasyApplicationClassManage, 'manage_editProperties')
    security.declareProtected(EasyApplicationClassManage, 'manage_changeProperties')
    security.declareProtected(EasyApplicationClassManage, 'manage_propertyTypeForm')
    security.declareProtected(EasyApplicationClassManage, 'manage_changePropertyTypes')


    #ZCatalog security redefinition: EasyApplicationClassManage
    ## 'Manage ZCatalog Entries':['Manager']
    security.declareProtected(EasyApplicationObjectChange, 'manage_catalogObject')
    security.declareProtected(EasyApplicationObjectChange, 'manage_uncatalogObject')
    security.declareProtected(EasyApplicationObjectChange, 'catalog_object')
    security.declareProtected(EasyApplicationObjectChange, 'uncatalog_object')
    security.declareProtected(EasyApplicationObjectChange, 'refreshCatalog')
    security.declareProtected(EasyApplicationObjectChange, 'manage_catalogView')
    #security.declareProtected(EasyApplicationObjectChange, 'manage_catalogFind')
    security.declareProtected(EasyApplicationObjectChange, 'manage_catalogSchema')
    security.declareProtected(EasyApplicationObjectChange, 'manage_catalogIndexes')
    security.declareProtected(EasyApplicationObjectChange, 'manage_catalogAdvanced')
    security.declareProtected(EasyApplicationObjectChange, 'manage_objectInformation')
    security.declareProtected(EasyApplicationObjectChange, 'manage_catalogReindex')
    #security.declareProtected(EasyApplicationObjectChange, 'manage_catalogFoundItems')
    security.declareProtected(EasyApplicationObjectChange, 'manage_catalogClear')
    security.declareProtected(EasyApplicationObjectChange, 'manage_addColumn')
    security.declareProtected(EasyApplicationObjectChange, 'manage_delColumn')
    security.declareProtected(EasyApplicationObjectChange, 'manage_addIndex')
    security.declareProtected(EasyApplicationObjectChange, 'manage_delIndex')
    security.declareProtected(EasyApplicationObjectChange, 'manage_clearIndex')
    security.declareProtected(EasyApplicationObjectChange, 'manage_reindexIndex')
    security.declareProtected(EasyApplicationObjectChange, 'availableSplitters')

    ## 'Search ZCatalog':['Anonymous', 'Manager']
    security.declareProtected('Search ZCatalog', 'searchResults')
    security.declareProtected('Search ZCatalog', '__call__')
    security.declareProtected('Search ZCatalog', 'uniqueValuesFor')
    security.declareProtected('Search ZCatalog', 'getpath')
    security.declareProtected('Search ZCatalog', 'schema')
    security.declareProtected('Search ZCatalog', 'indexes')
    security.declareProtected('Search ZCatalog', 'index_objects')
    security.declareProtected('Search ZCatalog', 'all_meta_types')
    security.declareProtected('Search ZCatalog', 'valid_roles')
    security.declareProtected('Search ZCatalog', 'resolve_url')
    security.declareProtected('Search ZCatalog', 'getobject')


    security.declareProtected(EasyApplicationObjectAdd,'setObject')
    def setObject(self,id,object,roles=None,user=None, set_owner=1):
        return self._setObject(id,object,roles,user, set_owner)

    def filtered_meta_types(self,user=None):
        gmt = []
        entry = None

        # Fetch the single EasyObject product instance and use as a base for the product list
        for each in Products.meta_types:
            if each['name'] == 'EasyObject':
                entry = each.copy()
                break

        if not entry: return []
        action = entry['action']
        for each in self.getAllEasyProducts(parent_template='[Root]'):
            if hasattr(aq_base(each), 'id'):
                entry['action'] = 'manage_addProduct/EasyApplication/' + each['id']
                entry['name'] = each['title']
                gmt.append(entry.copy())
        return gmt



    security.declareProtected(EasyApplicationObjectView,'getApplicationObject')
    def getApplicationObject(self):
        return self

    #Search Interface


    security.declareProtected(EasyApplicationObjectView,'getAllEasyProducts')
    def getAllEasyProducts(self, parent_template=None):
        return self.getSkinsFolder().getAllTemplates(parent_template)


    def getSubObjectsByClass(self, class_name, language=None, revision=None, tags=None, query=None, **kw):
        if query is None: query={}
        if language is None : language=self.default_language
        query['physicalPathIndex']={'query': aq_inner(self).physicalURL}
        query.update(kw)
        results=self.getObjectsByClass('product', language=language, revision=revision, tags=tags, query=query)
        return results

    def getObjectsByClass(self, class_name, language=None, revision=None, tags=None, query=None, **kw):
        if query is None: query={}
        query['subclass']={'query':class_name}
        if language is None: language=self.default_language
        query['getCurrentLanguage']={'query':language}

        #Set Brain:
        return self.searchResults(query)
        #physicalPathIndex=aq_inner(self).physicalURL)




    #index_html=None

    security.declareProtected(EasyApplicationObjectAdd, 'addObject')
    def addObject(self, subclass, id=None):
        """Add an object."""
        subclass_ob = self.getSkinsFolder().getTemplateByName(subclass)
        if subclass_ob is None:
            raise AttributeError, "Can't add an object of the template %s" % subclass
        subclass_title = subclass_ob.title

        if id is None:
            id = str(self.getUniqueId())
            while len(id) < 4: id = '0' + id
        id=str(id)
        id=id.strip()
        id = subclass_ob.id_prefix + id
        id=Tutils.xlate2url(id)

        ob=EasyObject(id, self.id, subclass, subclass_title)
        self._setObject(id, ob)
        ob=self._getOb(id)
        #ob.index_object()

        return ob


    security.declareProtected(EasyApplicationClassManage,'manage_changeData')
    manage_changeData = DTMLFile('dtml/manage_changeDataForm', globals())


    security.declareProtected(EasyApplicationClassManage,'manage_changePropertyData')
    def manage_changePropertyData(self, property_name, from_value, to_value, REQUEST=None):
        """Changes all EasyObjects property property_name from value from_value to value to_value"""
        changed = self.changePropertyData(self, property_name, from_value, to_value, property_is_list=0, recursive=1)

        if REQUEST is not None:
            manage_tabs_message = str(changed) + ' objects changed'
            return self.manage_changeData(self, management_view = 'Change Data', manage_tabs_message=manage_tabs_message, REQUEST=REQUEST)

    security.declareProtected(EasyApplicationClassManage,'changePropertyData')
    def changePropertyData(self, object, property_name, from_value, to_value, property_is_list=0, recursive=1):

        changed = 0
        property_name=property_name.strip()
        if object.meta_type == 'EasyObject': #Only change EasyObjects.
            # Accessing _language_data directly is a vialation to
            # the objects integrity. There should be a interface
            # on object for changing PropertyData (changePropertyData)
            for key, langobj in object._language_data.items():
                if hasattr(langobj, property_name):
                    old_value = getattr(langobj, property_name)
                    if property_is_list:
                        if from_value in old_value:
                            old_value.remove(from_value)
                            old_value.append(to_value)
                            setattr(langobj, property_name, old_value)
                            changed = 1
                    else:
                        if from_value == old_value:
                            setattr(langobj, property_name, to_value)
                            changed = 1

            if changed: object.index_object()

        if recursive:
            if hasattr(object, 'objectItems'): #It's a container!
                for each in object.objectItems():
                    changed = changed + self.changePropertyData(each[1], property_name, from_value, to_value, property_is_list=property_is_list)

        return changed


    #Manage Languages

    _manage_language_mapping = DTMLFile('dtml/manage_language_mapping', globals())
    security.declareProtected(EasyLanguageServiceManage, 'manage_language_mapping')
    def manage_language_mapping(self
                    , selected_language=[]
                    , add_language=None
                    , add_folder_url=None
                    , action='View'
                    , REQUEST=None):
        """Manage Language Folders"""
        management_view='LanguageFolders'
        manage_tabs_message=""
        deleted_languages=[]
        if action=='Delete' and selected_language:
            for key in selected_language:
                if key:
                    while key in self.language_order:
                        self.language_order.remove(key)
                    deleted_languages.append(key)
            self._p_changed=1
            manage_tabs_message="Language(s) %s [%s] was deleted." % (getLanguageByRFC3066(selected_language), selected_language)

        elif action=='Move Up' and selected_language:
            try:
                pos = self.language_order.index(selected_language[0])
            except:
                pos = len(self.language_order)
            new_order=[]
            if pos>0:
                new_order=self.language_order[0:pos-1]
            new_order.extend(selected_language)
            for langcode in self.language_order:
                if langcode not in new_order:
                    new_order.append(langcode)
            self.language_order=new_order
        elif action=='Move Down' and selected_language:
            new_order=[]
            pos=0
            max_len=len(self.language_order)
            langcode = self.language_order[pos]
            while langcode != selected_language[0] and pos<max_len:
                if langcode not in selected_language:
                    new_order.append(langcode)
                pos=pos+1
                if pos<max_len: langcode = self.language_order[pos]
            next_found=0
            while not next_found and pos<max_len:
                langcode = self.language_order[pos]
                if langcode not in selected_language and langcode not in new_order:
                    next_found = 1
                    new_order.append(langcode)
                pos=pos+1
            new_order.extend(selected_language)
            while pos<max_len:
                langcode = self.language_order[pos]
                if langcode not in selected_language and langcode not in new_order:
                    new_order.append(langcode)
                pos=pos+1
            self.language_order=distinct(new_order)
        elif action=='Move Top' and selected_language:
            new_order=[]
            new_order.extend(selected_language)
            for key in self.language_order:
                if key not in selected_language:
                    new_order.append(key)
            self.language_order=distinct(new_order)
        elif action=='Move Bottom' and selected_language:
            new_order=[]
            for key in self.language_order:
                if key not in selected_language:
                    new_order.append(key)
            new_order.extend(selected_language)
            self.language_order=distinct(new_order)
        elif action=='Add Language' and add_language is not None:
            if add_language not in self.language_order:
                self.language_order.append(add_language)
                self.language_order=distinct(self.language_order)
            self._p_changed=1
            manage_tabs_message="Language %s was added." % (getLanguageByRFC3066(add_language),)
        return self._manage_language_mapping(self, REQUEST=REQUEST
                                , manage_tabs_message=manage_tabs_message
                                , management_view=management_view
                                , deleted_languages=deleted_languages
                                , selected_language=selected_language
                                , add_language=add_language
                                , add_folder_url=add_folder_url or ''
                                )

    security.declareProtected(EasyApplicationObjectView,'getLanguageItems')
    def getLanguageItems(self):
        return map(lambda code: (code, getLanguageByRFC3066(code)), self.language_order)

    security.declareProtected(EasyApplicationObjectView,'getLanguages')
    def getLanguages(self):
        return self.language_order


    security.declareProtected(EasyApplicationObjectChange,'manage_catalogFind')
    manage_catalogFind = DTMLFile('dtml/manage_catalogFind',globals())

    _manage_compare = DTMLFile('dtml/manage_compare',globals())
    security.declareProtected(EasyApplicationObjectView,'manage_compare')
    def manage_compare(self,REQUEST=None):
        """\
        Compares the metadata in the catalog to the content of the database.
        """
        result=SearchResult(self)
        self.ZopeFindAndApply(self, apply_func=result.diffObject, REQUEST=REQUEST, search_sub=1)
        return self._manage_compare(self, REQUEST, result=result)


    def _ZopeFindAndApply(self, obj, obj_ids=None,
                             obj_templates=None,
                             obj_searchterm=None, obj_expr=None,
                             obj_mtime=None, obj_mspec=None,
                             obj_permission=None, obj_roles=None,
                             search_sub=0,
                             REQUEST=None, result=None, pre='',
                         apply_func=None, apply_path=''):
        """\

        """
        if obj_expr is None and 'all' not in obj_templates:
            obj_expr="subclass in [%s]" % ','.join(obj_templates)
        elif 'all' not in obj_templates:
            obj_expr="(%s) and subclass in [%s]" % (obj_expr, ','.join(obj_templates))

        return self.ZopeFindAndApply( obj, obj_ids=obj_ids,
                             obj_metatypes=['EasyObject'],
                             obj_searchterm=obj_searchterm, obj_expr=obj_expr,
                             obj_mtime=obj_mtime, obj_mspec=obj_mspec,
                             obj_permission=obj_permission, obj_roles=obj_roles,
                             search_sub=search_sub,
                             REQUEST=REQUEST, result=result, pre=pre,
                             apply_func=apply_func, apply_path=apply_path
                            )

    security.declareProtected(EasyApplicationObjectChange,'manage_catalogFoundItems')
    def manage_catalogFoundItems(self, REQUEST, RESPONSE, URL2, URL1,
                                 obj_metatypes=None, obj_templates=None,
                                 obj_ids=None, obj_searchterm=None,
                                 obj_expr=None, obj_mtime=None,
                                 obj_mspec=None, obj_roles=None,
                                 obj_permission=None):
        """ Find object according to search criteria and Catalog them
        """
        elapse = time.time()
        c_elapse = time.clock()

        words = 0
        path = '/'.join(self.getPhysicalPath())

        results = self._ZopeFindAndApply(self,
                                        obj_templates=obj_templates,
                                        obj_ids=obj_ids,
                                        obj_searchterm=obj_searchterm,
                                        obj_expr=obj_expr,
                                        obj_mtime=obj_mtime,
                                        obj_mspec=obj_mspec,
                                        obj_permission=obj_permission,
                                        obj_roles=obj_roles,
                                        search_sub=1,
                                        REQUEST=REQUEST,
                                        apply_func=self.catalog_object,
                                        apply_path=path)

        elapse = time.time() - elapse
        c_elapse = time.clock() - c_elapse

        RESPONSE.redirect(URL1 + '/manage_catalogView?manage_tabs_message=' +
                          urllib.quote('Catalog Updated<br>Total time: %s<br>Total CPU time: %s' % (`elapse`, `c_elapse`)))

    #CMF Skinable Interface
    security.declarePrivate('getSkinsFolderName')
    def getSkinsFolderName(self):
        return EasyAppConfigId

    security.declarePrivate('getSkinsFolder')
    def getSkinsFolder(self, default=_marker):
        sf = getattr(self, EasyAppConfigId, default)
        if sf is _marker:
            raise "SkinFolderError", "Can't get SkinFolder"
        return sf

    security.declareProtected(EasyApplicationObjectView,'manage_debug')
    manage_debug = DTMLFile('dtml/debug',globals())

    security.declarePublic('getUniqueId')
    def getUniqueId(self):
        return self._id_serial.getUniqueId()

    security.declarePublic('createGUID')
    def createGUID(self):
        return self._guid_generator.createGUID()


    def manage_afterAdd(self, item, container):
        self._catalog.clear()
        for object in self.objectValues():
            try: s=object._p_changed
            except: s=0
            if getattr(object, '_extended_catalog_aware', 0):
                object.manage_afterAdd(item, container, external=1)
            else:
                object.manage_afterAdd(item, container)
            if s is None: object._p_deactivate()

    def manage_beforeDelete(self, item, container):
        for object in self.objectValues():
            try: s=object._p_changed
            except: s=0
            if getattr(object, '_extended_catalog_aware', 0):
                object.manage_beforeDelete(item, container, external=1)
            else:
                object.manage_beforeDelete(item, container)
            if s is None: object._p_deactivate()

    def manage_afterClone(self, item):
        self._catalog.clear()
        for object in self.objectValues():
            try: s=object._p_changed
            except: s=0
            if getattr(object, '_extended_catalog_aware', 0):
                object.manage_afterClone(item, external=1)
            else:
                object.manage_afterClone(item)
            if s is None: object._p_deactivate()



    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        base=aq_base(self)
        ob_ver=getattr(self,'_object_version', 0.0)
        try: ob_ver=float(ob_ver)
        except: ob_ver=1.7

        ##Delete??
        if not hasattr(base,'default_language'): self.default_language='en'; changed=1
        if not hasattr(base,'language_order'): self.language_order = PersistentList(); changed=1

        if not hasattr(base, 'application_data_manager'):
            from Products.Sessions.SessionDataManager import SessionDataManager
            adm = SessionDataManager('application_data_manager',
                title='Application Data Manager',
                path='/temp_folder/session_data',
                requestName='EASESSION')
            self._setObject('application_data_manager', adm)
            changed=1

        if not hasattr(self, '_id_serial'):
            self._id_serial=PCounter()
            changed=1
        elif self._id_serial.__class__.__module__=="Shared.Torped.PCounter":
            p = PCounter(self._id_serial)
            del self._id_serial
            self._id_serial=p
            changed=1

        if not hasattr(base, EASYNOTIFICATIONID):
            ens=EasyNotification(EASYNOTIFICATIONID, '')
            self._setObject(EASYNOTIFICATIONID, ens)
            ens=self._getOb(EASYNOTIFICATIONID)
            for nid, value in getDefaultNotificationItems():
                not_obj=EasyNotificationInfo(nid, value[0],value[0], value[1], value[2])
                ens._setObject(nid,not_obj)

        if not hasattr(self, '_guid_generator'):
            self._guid_generator=GUIDGenerator()
            changed=1

        ##Remove Lanugage
        #if hasattr(self,'default_language'):
        #    del self.default_language
        #    changed=1
        #if hasattr(self,'language_order'):
        #    del self.language_order
        #    changed=1

        changed, messages=upgrade_indexes(self, changed, DEFAULT_INDEXES, DEPRECATED_INDEXES, REQUIRED_INDEXES, DEFAULT_COLUMNS, DEPRECATED_COLUMNS, REQUIRED_COLUMNS)

        self._object_version = 1.8

        if REQUEST is not None:
            return "<h1>%s</h1>%s"%(changed and 'Changed' or 'Not Changed', ''.join(map(lambda x: '<p>%s</p>'%x, messages)),)
        return changed, messages

    security.declareProtected('Easy Upgrade Manage', 'just_english_in_catalog')
    def just_english_in_catalog(self, REQUEST=None):
        """ re-index everything we can find """

        cat = self._catalog
        paths = cat.paths.values()
        paths = tuple(paths)
        cat.clear()
        for p in paths:
            if p[1]=='en':
                obj = self.resolve_path(p)
                if not obj:
                    obj = self.resolve_url(p, self.REQUEST)
                if obj is not None:
                    self.catalog_object(obj, p)

        return "<h1>just_english_in_catalog</h1>"

    security.declareProtected('Easy Upgrade Manage', 'just_english')
    def just_english(self, REQUEST=None):
        """ttw"""
        for object in self.objectValues(spec='EasyObject'):
            try: s=object._p_changed
            except: s=0
            object.just_english()
            if s is None: object._p_deactivate()
        if REQUEST:
            return "<h1>Just English</h1>"

Globals.InitializeClass(EasyApplication)
InitializeBodyTabs(EasyApplication)


manage_addEasyApplicationForm = DTMLFile('dtml/manage_addEasyApplicationForm', globals())

def manage_addEasyApplication(self, id, title='', REQUEST=None):
    """Add a EasyApplication object"""
    if title == '': title = id
    id=Tutils.xlate2url(id)
    ob=EasyApplication(id, title)
    self._setObject(id, ob)
    ob=self._getOb(id)
    conf=EasyApplicationConf()
    ob._setObject(EasyAppConfigId, conf)

    ens=EasyNotification(EASYNOTIFICATIONID, '')
    ob._setObject(EASYNOTIFICATIONID, ens)
    ens=ob._getOb(EASYNOTIFICATIONID)
    for nid, value in getDefaultNotificationItems():
        not_obj=EasyNotificationInfo(nid, value[0],value[0], value[1], value[2])
        ens._setObject(nid,not_obj)

    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)


def initialize(context):
    """Initialize StyleManagers Product"""

    context.registerClass(
        EasyApplication,
        permission=EasyApplicationAdd,
        icon='icons/EasyApplication.gif',
        constructors = ( manage_addEasyApplicationForm, manage_addEasyApplication ),
        interfaces=(EasyApplicationInterface,),
        meta_type = 'EasyApplication'
    )



