# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyApplication/EasyFixedSchema.py,v 1.1 2004/06/28 10:04:07 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__="""\
EasyFixedSchema
"""
__version__='$Revision: 1.1 $'[11:-2]

import Globals, Products, DateTime

from Globals import DTMLFile
from urllib import quote
from AccessControl import ClassSecurityInfo
from Acquisition import aq_chain, aq_base, aq_inner, aq_parent, Acquired
from OFS.PropertySheets import Virtual, View, FixedSchema
from OFS.PropertySheets import DefaultPropertySheets, vps, PropertySheet

from Products.EasyApplication.EasySecurityConf import EasyApplicationObjectChange


from Products.EasyCore import MLOG_TAB

_marker = []  # Create a new marker object.

class EasyFixedSchema(Virtual, FixedSchema, View):

    security = ClassSecurityInfo()

    _container=None
    def __init__(self, id, base, container, md=None, management_view=None):
        if md == None:
            md = {'xmlns': id}
        if management_view:
            self.management_view = management_view
        FixedSchema.__init__(self, id, base, md)


    #EasyApplicationClassManage
    security.declareProtected(EasyApplicationObjectChange, 'manage_addProperty')
    security.declareProtected(EasyApplicationObjectChange, 'manage_editProperties')
    security.declareProtected(EasyApplicationObjectChange, 'manage_delProperties')
    security.declareProtected(EasyApplicationObjectChange, 'manage_changeProperties')
    security.declareProtected(EasyApplicationObjectChange, 'manage_propertiesForm')
    security.declareProtected(EasyApplicationObjectChange, 'manage_propertyTypeForm')
    security.declareProtected(EasyApplicationObjectChange, 'manage_changePropertyTypes')


    security.declareProtected(EasyApplicationObjectChange, 'manage_main')
    #manage_main=DTMLFile('dtml/manage_fixed_schema_properties', globals())

    #Energy+ hack
    manage_main=DTMLFile('dtml/energy_plus/manage_fixed_schema_properties', globals())


    getWorkflowIcon=Acquired
    getPublishIcon=Acquired
    getLockedIcon=Acquired
    icon=Acquired
    isLocked=Acquired
    getWorkflowStateName=Acquired
    getWorkflowState=Acquired

    def getDefaultLanguage(self):
        return aq_parent(aq_parent(self)).getDefaultLanguage()

    def hasLanguage(self, lang_code):
        return aq_parent(aq_parent(self)).hasLanguage(lang_code)

    def is_language_aware(self):
        return getattr(aq_base(self._base), 'is_language_aware', None)

    def isSheetAWizard(self):
        return getattr(aq_base(self._base), 'is_a_wizard', None)

    def v_self(self):
        obj=aq_parent(aq_parent(self))
        if self.is_language_aware():
            lang_obj = obj.getCurrentObjectVersion()
        else:
            lang_obj = obj.getDefaultObjectVersion()
        if lang_obj is not None:
            return lang_obj
        raise "EasyPropertyError", "Can't get the current object version."

    def p_self(self):
        return aq_parent(aq_parent(self))


    def getProperty(self, id, default=None):
        # Return the property with the given id, returning the optional
        # second argument or None if no such property is found.
        if self.hasProperty(id):
            return getattr(self.v_self(), id, self._base.getProperty(id))
        return default

    def propertyLabel(self, id):
        return self._base.propertyLabel(id)

    def getPropertyDescription(self, id):
        return self._base.getPropertyDescription(id)

    def isFinishedButtonEnable(self):
        return self._base.enable_finished_button

    security.declareProtected(EasyApplicationObjectChange, 'do_editProperties')
    def do_editProperties(self, REQUEST=None):
        """Actually Edit object properties."""
        req=REQUEST or self.REQUEST
        for prop in self._propertyMap():
            name=prop['id']
            if 'w' in prop.get('mode', 'wd'):
                value=req.get(name, _marker)
                try:
                    self._updateProperty(name, value)
                except:
                    raise AttributeError, "Couldn't update property '%s' [%s] with value '%s' [%s]" % (name, prop['type'], value, type(value))

        self.index_object()

        for prop in self._propertyMap():
            name=prop['id']
            if 'w' in prop.get('mode', 'wd'):
                value=REQUEST.get(name, '')
                self._updateProperty(name, value)

    security.declareProtected(EasyApplicationObjectChange, 'manage_editProperties')
    def manage_editProperties(self, REQUEST=None):
        """Edit object properties via the web. Probably never called."""
        self.do_editProperties(REQUEST)
        if REQUEST is not None:
            return self.manage_main(self, REQUEST
                           , manage_tabs_message='Your changes have been saved.')

    security.declareProtected(EasyApplicationObjectChange, 'manage_changeProperties')
    def manage_changeProperties(self, REQUEST=None, **kw):
        """Edit object properties via the web. Probably never called."""
        res = apply(EasyFixedSchema.inheritedAttribute('manage_changeProperties'), (self, REQUEST), kw)
        self.index_object()
        return res

    def manage_options(self):
        """Return a manage option data structure for me instance
        """
        # We doesn't access sheets by its full url any more
        # so we don't need to compensate for that.
        # the manage option of this sheet looks like it's
        # directly on the EasyObject instance (using __bobo_traverse__).

        options = aq_parent(aq_parent(self)).manage_options
        if callable(options): options = options()
        return options

        # Remove this poart later, check with lennart first.
        try: r=self.REQUEST
        except: r=None
        if r is None:
            pre='../../'
        else:
            pre=r['URL']
            for i in (1,2,3):
                l=pre.rfind('/')
                if l >= 0:
                    pre=pre[:l]
            pre=pre+'/'

        r=[]
        options = aq_parent(aq_parent(self)).manage_options
        if callable(options): options = options()
        for d in options:
            path=d['action']
            option={'label': d['label'],
                      'action': pre+path,
                      'path': '../../'+path}
            help=d.get('help')
            if help is not None:
                option['help']=help
            r.append(option)
        return r


    def __bobo_traverse__(self, REQUEST, name):
        # As the sheet is fetch by a virtual method
        # (by __bobo_traverse__ in EasyObject)
        # we need to return the correct object for validating
        # roles.
        # manage_options thinks that the method is called
        # 'manage_ps_' something, but it doesn't
        # exist (because it's virtual).
        # The acqusition has pushed the sheet object at the
        # top of the chain (in __bobo_traverse__ in EasyObject).
        # The filtered_manage_options will use unrestrictedTravers
        # to find an object, and then check for privaleges on that
        # object.
        # So we need to give the unrestrictedTravers the 'manage_main'
        # method if it askes for "manage_ps_"
        # The manage_main method contains the correct (i hope) __roles__
        # settings.
        if hasattr(self, name):
            return getattr(self, name)
        if name[0:10]=="manage_ps_":
            sheet_command = name[10:]
            if sheet_command[0:7]=="manage_":
                #sheet_name = name[21:] #This should be same as self.id, we might want to test it?
                #This is an access for the manage_main method so return it.
                return getattr(self, 'manage_main')
            if sheet_command[0:15]=="editProperties_":
                return getattr(self, 'manage_editProperties')


    def render_next(self):
        return "Next %s" % str(aq_parent(aq_parent(self)).manage_options)

    def render_previous(self):
        return "Previous %s" % str(aq_parent(aq_parent(self)).manage_options)

    def test(self):
        return """
            self._base.meta_type: %s
            aq_parent(aq_inner(self._base)).meta_type: %s
            """ % ( str(self._base.meta_type),
                    str(aq_parent(aq_inner(self._base)).meta_type),
                  )

Globals.InitializeClass(EasyFixedSchema)
