# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyApplication/EasyObjectTemplate.py,v 1.1 2004/06/28 10:04:07 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__='''EasyApplication EasyObjectTemplate'''
__version__='$Revision: 1.1 $'[11:-2]

import Globals, Products
from Globals import DTMLFile
from Acquisition import aq_chain, aq_inner, aq_parent
from AccessControl import ClassSecurityInfo

from OFS.Folder import Folder
from OFS.ObjectManager import IFAwareObjectManager

from Products.ZCatalog.CatalogAwareness import CatalogAware
from Products.EasyCatalog.EasyCatalogAware import EasyCatalogAwareObjectAndPropertyManager

from Products.EasyCore.Utils import Tutils
from Products.EasyCore.interfaces.EasyIcon import EasyIconInterface
from Products.EasyCore.EasyIcon import EasyIcon
from Products.EasyCore.ObjectManager.OrderedObjectManagerIF import OrderedObjectManagerIF
from Products.EasyCore.BodyTabs import InitializeBodyTabs, UpdateTab, BodyTabs

from Products.EasyCore.VirtualPath import VirtualPathBase

from Products.EasyWorkflow.EasyWorkflow import WorkflowManagement
from Products.EasyWorkflow.EasyPublishable import Publishable

from Products.EasyApplication.interfaces.EasyApplication import EasyPluggableDataSheet, EasyPluggableView
from Products.EasyApplication.EasySecurityConf import EasyApplicationClassManage
from Products.EasyApplication.EasyApplicationCatalogAware import EasyApplicationConfCatalogAware
from Products.EasyApplication.EasyApplicationGlobals import EasyAppConfigId, EasyObjectTemplateType, EasyPropertySheetType
from Products.EasyApplication.EasyObject import EasyObject

#Interfaces
from Products.EasyApplication.interfaces.EasyApplication import EasyObjectTemplate
from Products.EasyApplication.interfaces.EasyApplication import EasyPluggableView, EasyPluggableDataSheet
from Products.EasyCore.interfaces.EasyIcon import EasyIconInterface
from Products.EasyApplication.EasyCSVSupport import EasyCSVSupport

from Products.EasyCore import MLOG_TAB


#experimental
from Products.EasyCore.ClassWrappers import DTMLFileWrapper, FunctionTemplate


class EasyObjectTemplate(EasyApplicationConfCatalogAware, OrderedObjectManagerIF, BodyTabs, EasyCatalogAwareObjectAndPropertyManager, Folder):
    """Web application main class"""

    security = ClassSecurityInfo()

    meta_type = EasyObjectTemplateType
    title=''

    default_catalog=EasyAppConfigId


    _product_interfaces = (EasyIconInterface, EasyPluggableDataSheet)

    _template_attributes=('is_visible', 'is_a_wizard', 'getObjectTemplate','manage_test_dtml','manage_test_method','manage_test_method2')

    id_prefix=''
    is_a_folder = 0
    enable_workflow_view = ''
    enable_publish_view=''
    enable_cvs_view=''
    is_a_wizard=0
    is_visible=0
    is_creatable_in = ['[Root]']

    manage_options = Folder.manage_options

    __interfaces__=(EasyObjectTemplate,)

    _properties = Folder._properties + (
                                        {'id':'is_a_folder',          'label':'Is a Folder', 'type':'boolean',            'mode':'w'},
                                        {'id':'is_visible',            'label':'Is Visible', 'type':'boolean', 'mode':'w'},
                                        {'id':'enable_workflow_view', 'label':'Enable Workflow View', 'type':'string', 'mode':'w'},
                                        {'id':'enable_publish_view',  'label':'Enable Publish View', 'type':'string', 'mode':'w'},
                                        {'id':'enable_cvs_view',      'label':'Enable CSV Export', 'type':'string', 'mode':'w'},
                                        {'id':'id_prefix',            'label':'Prefix', 'type':'string',             'mode':'w'},
                                        {'id':'is_a_wizard',          'label':'Is a Wizard', 'type':'boolean', 'mode':'w', },
                                        {'id':'is_creatable_in',      'label':'Is creatable in', 'type':'multiple selection', 'mode':'w', 'select_variable': 'getPlacesForCreation'},

                                       )

    def __init__(self, id, title=''):
        self.id = id
        self.title = title

    security.declareProtected(EasyApplicationClassManage, 'manage_main')
    manage_main=DTMLFileWrapper(OrderedObjectManagerIF.manage_main, globals()
        , interfaces=[EasyPluggableView, EasyPluggableDataSheet]
        , management_view='Contents'
        , return_view='manage_main'
        , body_tab={'order':1, 'label':'View', 'selectable':1, 'management_view':'Contents'})

    security.declareProtected(EasyApplicationClassManage, 'manage_datasheets')
    manage_datasheets=DTMLFileWrapper(OrderedObjectManagerIF.manage_main, globals()
        , interfaces=[EasyPluggableDataSheet]
        , management_view='Contents'
        , return_view='manage_datasheets'
        , body_tab={'order':2, 'label':'Datasheets', 'selectable':1, 'management_view':'Contents'})


    security.declareProtected(EasyApplicationClassManage, 'manage_others')
    manage_others=DTMLFileWrapper(OrderedObjectManagerIF.manage_main, globals()
        , interfaces=[EasyIconInterface]
        , management_view='Contents'
        , return_view='manage_others'
        , body_tab={'order':3, 'label':'Others', 'selectable':1, 'management_view':'Contents'})

    security.declareProtected(EasyApplicationClassManage, 'manage_all')
    manage_all=DTMLFileWrapper(OrderedObjectManagerIF.manage_main, globals()
        , interfaces=[EasyPluggableView, EasyPluggableDataSheet, EasyIconInterface]
        , management_view='Contents'
        , return_view='manage_all'
        , body_tab={'order':4, 'label':'All', 'selectable':1, 'management_view':'Contents'})



    #EasyApplicationClassManage
    security.declareProtected(EasyApplicationClassManage, 'manage_addProperty')
    security.declareProtected(EasyApplicationClassManage, 'manage_editProperties')
    security.declareProtected(EasyApplicationClassManage, 'manage_delProperties')
    security.declareProtected(EasyApplicationClassManage, 'manage_changeProperties')
    security.declareProtected(EasyApplicationClassManage, 'manage_propertiesForm')
    security.declareProtected(EasyApplicationClassManage, 'manage_propertyTypeForm')
    security.declareProtected(EasyApplicationClassManage, 'manage_changePropertyTypes')

    #def filtered_meta_types(self,user=None):
    #    f=lambda x: x['name'] in (EasyPropertySheetType, EasyIcon.meta_type)
    #    return filter(f, Products.meta_types)


    def skin_path(self):
        skin_tool=getattr(self, EasyAppConfigId, None)
        if skin_tool is not None:
            skin_tool_path=skin_tool.getPhysicalPath()
            template_path=self.getPhysicalPath()
            skin_path=['/'.join(template_path[len(skin_tool_path):])]
            return ','.join(skin_path)
        return None


    def getObjectTemplate(self, containment=1):
        if containment:
            return aq_inner(self)
        return self

    ###Energy+ hack

    def getManageOptions(self, options_left=(), options_right=()):

        #A manager should see diffrent options...

        if self.is_a_folder:
            options_left=EasyObject.manage_main_options+options_left
        if self.enable_workflow_view:
            options_right=({'label': self.enable_workflow_view, 'action': 'manage_workflow',},)+options_right
        if self.enable_publish_view:
            options_right=({'label': self.enable_publish_view, 'action': 'manage_publishform',},)+options_right

        #Energy+ hack
        if self.enable_cvs_view:
            options_right=({'label': self.enable_cvs_view, 'action': 'manage_csv',},)+options_right

        manage_propsheets = ()
        for sheet in self.getPropertySheets():
            if hasattr(sheet, 'isVisible') and sheet.isVisible:
                manage_propsheets = manage_propsheets + (
                        {'label': sheet.title,           # TAB: Contents
                         'action': 'manage_ps_manage_%s' % sheet.id,
                        },)
        return options_left+manage_propsheets+options_right


    def getPropertySheets(self):
        skinfolder=self.getSkinsFolder()
        query = {}
        query['meta_type'] = { 'query': EasyPropertySheetType }
        query['physicalParentURL'] = { 'query': skinfolder.physicalURL }
        globalsheets = skinfolder.searchResults( query)
        query['physicalParentURL'] = { 'query': self.physicalURL }
        query['physicalPathIndex'] = { 'query': self.physicalURL }
        localsheets = skinfolder.searchResults( query)
        return globalsheets + localsheets

    #SkinnableTemplate Support

    def template_attributes(self):
        #Indexes the attributes that are allows the skin to
        #load and access them from the template.
        return self._template_attributes

    #security.declareProtected('View','manage_test')
    #def manage_test(self, REQUEST=None):
    #    """ttw"""
    #    return "self.__dict__", str(self.__dict__.keys() )+ str(self.__class__.__dict__.keys())+"<hr>DestinationURL: "+self.DestinationURL()

    #Experimental Factory Support
    #manage_factory_form=DTMLFile('dtml/manage_addEasyObjectForm2', globals())
    #
    #def manage_factory(self, id, subclass, subclass_title, REQUEST=None):
    #    """Add an EasyObject"""
    #    id=Tutils.xlate2url(id)
    #    ob=EasyObject(id, subclass, subclass_title)
    #    self._setObject(id, ob)
    #
    #    if REQUEST is not None:
    #        try: u=self.DestinationURL()
    #        except: u=REQUEST['URL1']
    #        u="%s/%s" % (u,quote(id))
    #        REQUEST.RESPONSE.redirect(u+'/manage_workspace')
    #    else:
    #        return ob

    manage_test_dtml=DTMLFile('manage_test_dtml', globals())

    def _manage_test_method(self, client=None, REQUEST=None):
        """ttw"""
        return """<h1>manage_test_method</h1><hr>
        <h3>self.manage_test_method2(client, REQUEST)</h3>
        %s<hr>%s""" % (self.manage_test_method2(client, REQUEST),self.getPhysicalPath())

    manage_test_method=FunctionTemplate(_manage_test_method, globals())

    def manage_test_method2(self, client=None, REQUEST=None):
        """ttw"""
        return """<h1>manage_test_method2</h1>%s<hr>%s""" % ('',self.getPhysicalPath())

Globals.InitializeClass(EasyObjectTemplate)
InitializeBodyTabs(EasyObjectTemplate)

manage_addEasyObjectTemplateForm = DTMLFile('dtml/manage_addEasyObjectTemplateForm', globals())

def manage_addEasyObjectTemplate(self, id, title='', REQUEST=None):
    """Add a EasyApplication object"""
    if title == '': title = id
    id=Tutils.xlate2url(id)
    ob=EasyObjectTemplate(id, title)
    self._setObject(id, ob)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)


def initialize(context):
    """Initialize StyleManagers Product"""

    context.registerClass(
        EasyObjectTemplate,
        permission=EasyApplicationClassManage,
        icon='icons/EasyObjectTemplate.gif',
        constructors = ( manage_addEasyObjectTemplateForm, manage_addEasyObjectTemplate ),
        meta_type = EasyObjectTemplate.meta_type,
        visibility = None,
    )



