# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyApplication/EasyPropertySheet.py,v 1.1 2004/06/28 10:04:08 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__="""EasyPropertySheet"""
__version__='$Revision: 1.1 $'[11:-2]


import Globals
from Globals import DTMLFile
from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem

from Products.ZCatalog.CatalogAwareness import CatalogAware
from Products.EasyCatalog.EasyCatalogAware import EasyCatalogAwarePropertyManager

from AccessControl import ClassSecurityInfo

#EasyApplication
#from Products.EasyApplication.EasyApplicationGlobals import *
from Products.EasyApplication.EasyApplicationCatalogAware import EasyApplicationConfCatalogAware
from Products.EasyApplication.EasyApplicationGlobals import EasyAppConfigId, EasyPropertySheetType
from Products.EasyApplication.EasySecurityConf import EasyApplicationClassManage
from Products.EasyApplication.interfaces.EasyApplication import EasyPluggableView, EasyPluggableDataSheet

#EasyCore
from Products.EasyCore.EasySortable.EasySortable import EasySortablePropertyManager
from Products.EasyCore.Utils import Tutils

#Interfaces


from Products.EasyCore import MLOG_TAB


class EasyPropertySheet(EasySortablePropertyManager, EasyApplicationConfCatalogAware,
                        EasyCatalogAwarePropertyManager, SimpleItem):

    security = ClassSecurityInfo()

    manage_options = ({'label':'Properties', 'action':'manage_propertiesForm'},) +\
                     ({'label':'Options', 'action':'manage_propertySheetOptionsForm'},) +\
                     SimpleItem.manage_options

    is_language_aware=0
    is_visible=0
    enable_finished_button=0

    __implements__ = (EasyPluggableView,EasyPluggableDataSheet)
    meta_type = EasyPropertySheetType
    default_catalog=EasyAppConfigId

    _properties = (
        {'id':'is_a_wizard',          'label':'Is a Wizard', 'type':'boolean', 'mode':'w', },
        )

    security.declareProtected(EasyApplicationClassManage, 'manage_propertiesForm')
    manage_propertiesForm = DTMLFile('dtml/manage_propertiesTemplate', globals(), property_extensible_schema__=1)

    security.declareProtected(EasyApplicationClassManage, 'manage_addProperty')
    security.declareProtected(EasyApplicationClassManage, 'manage_editProperties')
    security.declareProtected(EasyApplicationClassManage, 'manage_delProperties')
    security.declareProtected(EasyApplicationClassManage, 'manage_changeProperties')
    security.declareProtected(EasyApplicationClassManage, 'manage_propertiesForm')
    security.declareProtected(EasyApplicationClassManage, 'manage_propertyTypeForm')
    security.declareProtected(EasyApplicationClassManage, 'manage_changePropertyTypes')

    security.declareProtected(EasyApplicationClassManage, 'manage_main') #new
    manage_main = manage_propertiesForm
    manage_main._setName('manage_main')

    def __init__(self, id, title='', is_visible=0):
        self.id = id
        self.title = title
        self.is_visible=is_visible

    security.declareProtected(EasyApplicationClassManage, 'manage_propertySheetOptionsForm')
    manage_propertySheetOptionsForm = DTMLFile('dtml/manage_propertySheetOptionsForm', globals())

    security.declareProtected(EasyApplicationClassManage, 'manage_propertySheetOptionsForm')
    def manage_propertySheetOptions(self, is_language_aware=0, is_visible=0, is_a_wizard=0, enable_finished_button=0, title=None, REQUEST=None):
        """Sets options for the property sheets"""

        self.is_language_aware=is_language_aware
        self.is_visible=is_visible
        self.is_a_wizard=is_a_wizard
        self.enable_finished_button=enable_finished_button
        if title is not None:
            self.title=title

        self.index_object()

        if REQUEST is not None:
            message = "Options changed"
            return self.manage_propertySheetOptionsForm(
                REQUEST,
                manage_tabs_message=message,
                management_view='Options')


    def isVisible(self):
        return self.is_visible

    def enableFinishedButton(self):
        return self.enable_finished_button


    def propertyLabel(self, id):
        return self.propdict().get(id, {}).get('label', id)

    def getPropertyDescription(self, id):
        return self.propdict().get(id, {}).get('description', '')

    def _updatePropertyInfo(self, id, **kw):
        # Update the value of an existing property. If value
        # is a string, an attempt will be made to convert
        # the value to the type of the existing property.
        _properties=()
        for prop in self._properties:
            if prop.get('id')==id:
                prop.update(kw)
            _properties=_properties+(prop,)
        self._properties=_properties

    def manage_editProperties(self, REQUEST):
        """Edit object properties via the web.
        The purpose of this method is to change all property values,
        even those not listed in REQUEST; otherwise checkboxes that
        get turned off will be ignored.  Use manage_changeProperties()
        instead for most situations.
        """
        for prop in self._propertyMap():
            name=prop['id']
            if 'w' in prop.get('mode', 'wd'):
                value=REQUEST.get(name, '')
                self._updateProperty(name, value)
                description=REQUEST.get(name+'_description', '')
                label=REQUEST.get(name+'_label', name)
                self._updatePropertyInfo(name, description=description, label=label)
        if REQUEST:
            message="Saved changes."
            return self.manage_propertiesForm(self,REQUEST,
                                              manage_tabs_message=message)

    def manage_changeProperties(self, REQUEST=None, **kw):
        """Change existing object properties.

        Change object properties by passing either a mapping object
        of name:value pairs {'foo':6} or passing name=value parameters
        """
        if REQUEST is None:
            props={}
        else: props=REQUEST
        if kw:
            for name, value in kw.items():
                props[name]=value
        propdict=self.propdict()
        for name, value in props.items():
            if self.hasProperty(name):
                if not 'w' in propdict[name].get('mode', 'wd'):
                    raise 'BadRequest', '%s cannot be changed' % name
                self._updateProperty(name, value)
                description=REQUEST.get(name+'_description', '')
                label=REQUEST.get(name+'_label', name)
                self._updatePropertyInfo(name, description=description, label=label)
        if REQUEST:
            message="Saved changes."
            return self.manage_propertiesForm(self,REQUEST,manage_tabs_message=message)


Globals.InitializeClass(EasyPropertySheet)

manage_addEasyPropertySheetForm = DTMLFile('dtml/manage_addEasyPropertySheetForm', globals())


def manage_addEasyPropertySheet(self, id, title='', REQUEST=None):
    """Add a EasyApplication object"""
    if title == '': title = id
    id=Tutils.xlate2url(id)
    ob=EasyPropertySheet(id, title)
    self._setObject(id, ob)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)


def initialize(context):
    """Initialize StyleManagers Product"""

    context.registerClass(
        EasyPropertySheet,
        permission=EasyApplicationClassManage,
        icon='icons/EasyPropertySheet.gif',
        constructors = ( manage_addEasyPropertySheetForm, manage_addEasyPropertySheet ),
        meta_type = EasyPropertySheet.meta_type,
        interfaces=(EasyPluggableDataSheet, EasyPluggableView),
        visibility = None,
    )



