# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyApplication/EasySkinnable.py,v 1.1 2004/06/28 10:04:08 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__="""\
EasySkinnable.py
"""
__version__='$Revision: 1.1 $'[11:-2]

import Globals
from Globals import DTMLFile
from ExtensionClass import Base
from types import StringType, ListType, TupleType
from AccessControl import ClassSecurityInfo
from Acquisition import aq_chain, aq_self, aq_base, aq_inner, aq_parent, aq_acquire
from Acquisition import ImplicitAcquisitionWrapper, Implicit

from OFS.Folder import Folder
from OFS.ObjectManager import ObjectManager
from ZODB.PersistentMapping import PersistentMapping
from BTrees.IOBTree import IOBTree

#CMFCore
from Products.CMFCore.Skinnable import SkinnableObjectManager

#EasyCore
from Products.EasyCore.Utils.Utils import distinct
from Products.EasyCore.Utils.PCounter import PCounter
from Products.EasyCore.GUID import GUIDGenerator

#EasyPublisher
from Products.EasyLanguageService.EasyLanguageHelpers import getLanguageByRFC3066
from Products.EasyCatalog.EasyCatalogAware import EasyCatalogAwareObjectAndPropertyManager

#EasyApplication
from Products.EasyApplication.EasyApplicationGlobals import EasyAppConfigId


from Products.EasyCore import MLOG_TAB

_marker = []  # Create a new marker object.

DEFAULT_LANGUAGE='en'
DRAFT_VER = 'Draft'
EASYAPPCURRENTLANGUAGE='EasyAppCurrentLanguage'



# superGetAttr is assigned to whatever ObjectManager.__getattr__
# used to do.
try:
    superGetAttr = Folder.__getattr__
except:
    try:
        superGetAttr = Folder.inheritedAttribute('__getattr__')
    except:
        superGetAttr = None

class VeryTinyItem(Base, Globals.Persistent, Implicit):

    def copy(self):
        """Make a shallow copy of myself"""
        newitem = VeryTinyItem()
        #newitem.__dict__ = self.__dict__.copy()
        for attrib in self.__dict__.keys():
            setattr(newitem, attrib, getattr(self, attrib))
        return newitem


class EasySkinnableAppObjectManager(SkinnableObjectManager):
    """\
    Base class for skinnable object managers.
    Handles "skins" from Templates folders as well as
    versions and language versions.
    """

    security = ClassSecurityInfo()

    meta_type = 'EasyApplicationSkinnable'

    #_v_skinattr  hold the ids the skintemplate provide.
    #  this is because the __of__ sets up the skin in a lazy fasion.
    #  It only saves a referense to the skinfolder (as a tuple in
    #  _v_skinfolder to avoid it to be wrapt in the local context.)
    #  __getattr__ checks if the name it's looking up is in the _v_skinattr
    #  in which case the skin is loaded and saved as the fourth
    #  value in the tuple _v_skindata.
    #
    #_v_skindata also contains information about the propertysheets.
    #

    _v_skinattr=None
    _v_skindata=None        #(None, None, {}, None)
    _v_skinfolder=None

    default_language=None

    def __init__(self):
        self._language_data = PersistentMapping()
        self._revision_counter=PCounter()
        self._revision_data = IOBTree()

    def template_attributes(self):
        #Indexes the attributes that are allows the skin to
        #load and access them from the template.
        return ()

    def __getattr__(self, name):
        if not name.startswith('_') and not name.startswith('aq_'):
            cl = self._v_skindata

            if cl is not None and len(cl)==4 and cl[1] is not None:
                request, ob, ignore, skinobj = cl
                skinattr=self._v_skinattr
                if skinobj is None and type(skinattr) in (ListType, TupleType)  and name in skinattr:
                    skinobj = self.loadSkin()
                    self._v_skindata=request, ob, ignore, skinobj
                if not ignore.has_key(name):
                    subob = getattr(ob, name, _marker)
                    if subob is not _marker:
                        # Return it in context of self, forgetting
                        # its location and acting as if it were located
                        # in self.
                        return aq_base(subob)
                    elif skinobj is not None and skinattr and name in skinattr:
                        subob = getattr(skinobj, name, _marker)
                        if subob is not _marker:
                            # Return it in context of self, forgetting
                            # its location and acting as if it were located
                            # in self.
                            return aq_base(subob)
                        else:
                            ignore[name] = 1
                    else:
                        ignore[name] = 1

        if superGetAttr is None:
            raise AttributeError, name
        return superGetAttr(self, name)

    def __of__(self, parent):
        """\
        Overload the EasySkinnableAppObjectManager __of__ to handle PropertySheets.
        """
        w_self = ImplicitAcquisitionWrapper(aq_base(self), parent)
        if aq_parent(parent) is not None:
            w_self._setupPropertySheets(parent)
            w_self.setupCurrentSkin()
        return w_self


    security.declarePrivate('setupCurrentSkin')
    def setupCurrentSkin(self, REQUEST=None):

        if REQUEST is None:
            REQUEST = getattr(self, 'REQUEST', None)
        if REQUEST is None: return

        if self._v_skindata is not None and self._v_skindata[0] is REQUEST:
            # Already set up for this request.
            return
        self._v_skindata = None
        lang_code = self.getCurrentLanguage()
        if self.hasObjectVersion(DRAFT_VER, lang_code):
            self.changeCurrentVersion(DRAFT_VER, lang_code)
        else:
            lang_code = self.getDefaultLanguage()
            self.changeCurrentVersion(DRAFT_VER, lang_code)

        self.changeSkin(None)


    security.declarePublic('changeSkin')
    def changeSkin(self, skinname):
        # Because skins acctually are template of an object
        # it doesn't have to be submitted. It's accessible in the
        # self.subclass anyway.
        #
        sf = getattr(self, EasyAppConfigId, None)
        if sf is not None:
            self._v_skinattr = sf.getSkinAttributeIds(self.subclass)
            self._v_skinfolder=(sf,) # A hack. I save the referense as a tuple to avoid it to be contained by the object.


    def loadSkin(self):
        skinname=self.subclass
        if skinname:
            sf = self._v_skinfolder
            if sf is not None:
                sf=sf[0]
            if sf is not None:
                #return sf.getSkinByName(skinname)
                skinobj = sf.getSkinByName(skinname)
                return skinobj


    def _setupPropertySheets(self, obj=None):
        raise "EasySkinnableError", "Not Implemented"



    #CMF Skinable Interface
    security.declarePrivate('getSkinsFolderName')
    def getSkinsFolderName(self):
        return EasyAppConfigId

    security.declarePrivate('getSkinsFolder')
    def getSkinsFolder(self, default=_marker):
        sf = getattr(self, self.getSkinsFolderName(), default)
        if sf is _marker:
            raise "SkinFolderError", "Can't access SkinFolder %s" % self.getSkinsFolderName()
        return sf



    ##Revision Interface

    security.declarePrivate('addNewRevision')
    def addRevision(self, rev_object):
        rev_nr=self._revision_counter.getUniqueId()
        self._revision_data[rev_nr]=rev_object
        return rev_nr

    security.declarePrivate('changeRevision')
    def changeRevision(self, rev_object, rev_nr=None):
        if rerev_nr is None:
            rev_nr=self._revision_data.maxKey()
        self._revision_data[rev_nr]=rev_object
        return rev_nr

    security.declarePrivate('getRevision')
    def getRevision(self, rev_nr=None, default=None):
        try:
            rev_nr=int(rev_nr)
        except:
            rev_nr=None
        if rev_nr is None:
            rev_nr=self._revision_data.maxKey()
        return self._revision_data.get(rev_nr, default)

    security.declarePrivate('getRevisionIds')
    def getRevisionIds(self):
        return self._revision_data.keys()

    security.declarePrivate('getLastRevisionId')
    def getLastRevisionId(self):
        return self._revision_data.maxKey()


    #######
    # Language Interface
    #######

    security.declarePublic('hasLanguage')
    def hasLanguage(self, lang_code):
        return self._language_data.has_key(lang_code)

    security.declarePublic('getDefaultLanguage')
    def getDefaultLanguage(self):
        #Upgrade
        if getattr(self,'default_language', None) is None:
            if hasattr(self,'_language_data') and self._language_data:
                self.default_language=self._language_data.keys()[0]
        return self.default_language

    def acquireDefaultLanguage(self, default=None):
        """Override this to add other languages sources."""
        return default

    def changeDefaultLanguage(self, lang_code, raise_exp=0):
        """\
        Change the Default Language
        """
        if self._language_data.has_key(lang_code):
            self.default_language=lang_code
            return
        if raise_exp:
            raise "EasySkinnableError", "Can't change the default language to %s. The language doesn't exist" % lang_code

    security.declarePublic('getDefaultVersion')
    def getDefaultVersion(self, default=None):
        return DRAFT_VER

    security.declarePublic('getCurrentVersion')
    def getCurrentVersion(self, default=None):
        return DRAFT_VER

    security.declarePublic('getCurrentLanguage')
    def getCurrentLanguage(self, default=_marker):
        adm = getattr(self, 'application_data_manager', None)
        lang_code=None
        if adm:
            sd=adm.getSessionData(create=1)
            lang_code = sd.get(EASYAPPCURRENTLANGUAGE, self.getDefaultLanguage())
        if lang_code is not None:
            return lang_code
        if default is _marker:
            default=self.getDefaultLanguage()
        return default

    security.declarePrivate('setCurrentLanguage')
    def setCurrentLanguage(self, lang_code):
        adm = getattr(self, 'application_data_manager', None)
        if adm:
            sd=adm.getSessionData(create=1)
            sd.set(EASYAPPCURRENTLANGUAGE, lang_code)

    def getCurrentObjectVersion(self, REQUEST=None):
        if REQUEST is None:
            REQUEST = getattr(self, 'REQUEST', None)
        if self._v_skindata is not None and self._v_skindata[0] is REQUEST:
            return self._v_skindata[1]
        lang_code=self.getCurrentLanguage()
        version_id=self.getCurrentVersion()
        return self.getObjectVersion(version_id, lang_code)


    security.declarePublic('getLanguages')
    def getLanguages(self):
        return self._language_data.keys()

    security.declarePublic('getAllLanguages')
    def getAllLanguages(self):
        languages=()
        current  = self.getCurrentLanguage()
        for lang in self._language_data.keys():
            languages=languages+({
                 'lang': lang
               , 'rfc3066': getLanguageByRFC3066(lang)
               , 'selected': lang == current and 'SELECTED' or ''
              },)
        return languages

    security.declarePublic('acquireAllLanguages')
    def acquireAllLanguages(self):
        """Override this to add other languages sources."""
        return []

    security.declarePublic('getLanguageByRFC3066')
    def getLanguageByRFC3066(self, lang):
        return getLanguageByRFC3066(lang)

    security.declarePublic('getPossibleLanguages')
    def getPossibleLanguages(self):
        existing = self._language_data.keys()
        languages=()
        for lang in self.acquireAllLanguages():
            if lang not in existing:
                languages=languages+({
                     'lang': lang
                   , 'rfc3066': getLanguageByRFC3066(lang)
                  },)
        return languages


    security.declarePrivate('addObjectVersion')
    def addObjectVersion(self, version, lang_code):
        # Check to see if there is old data:
        default_lang = self.getDefaultLanguage()
        default_obj = self._language_data.get(default_lang, None)
        if lang_code == default_lang and default_obj is None:
            self._language_data[lang_code] = VeryTinyItem()
            return
        if default_obj is not None:
            self._language_data[lang_code]=default_obj.copy()
            return
        raise "EasySkinnableError", "Trying to create a not-default langugae before the default language."

    security.declarePrivate('addDefaultObjectVersion')
    def addDefaultObjectVersion(self):
        lang=self.acquireDefaultLanguage(DEFAULT_LANGUAGE)
        self.default_language=lang
        self.addObjectVersion(DRAFT_VER, lang)
        return self._language_data.get(lang)


    security.declarePrivate('changeCurrentVersion')
    def changeCurrentVersion(self, version, lang_code):
        if self.hasObjectVersion(version, lang_code):
            self._v_skindata = None
            object=self.getObjectVersion(version, lang_code)
            self.setCurrentLanguage(lang_code)
            self._v_skindata = (self.REQUEST, object, {}, None)
        #else:
        #    raise "EasySkinnableError", "The object version ('%s', '%s') doesn't exist." % (version, lang_code)

    security.declarePrivate('getLanguageOb')
    def getLanguageOb(self, lang_code=None, default=_marker):
        if self._language_data.has_key(lang_code):
            return self._language_data.get(lang_code)
        if default is _marker:
            default=self.getDefaultLanguage()
            if self._language_data.has_key(default):
                return self._language_data.get(default)
            else:
                #Add default language
                return self.addDefaultObjectVersion()
        return default


    security.declarePrivate('hasObjectVersion')
    def hasObjectVersion(self, version=None, lang_code=None):
        return self._language_data.has_key(lang_code)

    security.declarePrivate('getObjectVersion')
    def getObjectVersion(self, version=None, lang_code=None):
        default_lang = self.getDefaultLanguage()
        d_object = self.getLanguageOb(default_lang)
        if lang_code is not None and default_lang!=lang_code:
            c_object = self.getLanguageOb(lang_code)
            if c_object is not None:
                d_object=c_object.__of__(d_object)
        return d_object

    security.declarePrivate('getDefaultObjectVersion')
    def getDefaultObjectVersion(self, version=None, lang_code=None):
        default_lang = self.getDefaultLanguage()
        d_object = self.getLanguageOb(default_lang)
        return d_object


    #######
    # Skinnable Catalog Helpers
    #######

    def getCatalogItemPath(self):
        raise "EasySkinnableError", "Not Implemented"

Globals.InitializeClass(EasySkinnableAppObjectManager)

