## $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyCore/EasyConverters.py,v 1.1 2004/06/28 10:04:10 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#


__doc__="""EasyConverters

$Colliberty: EasyConverters.py,v 0.0.1 2001/05/14 22:06:00 johanc Exp $"""
__version__='$Revision: 1.1 $'[11:-2]


import re
from string import strip
from types import ListType, TupleType, UnicodeType
from DateTime import DateTime
from cgi import escape

from Products.EasyCore import MLOG_TAB


def runnable(v):
    if hasattr(v,'read'): return v.read()
    elif isinstance(v,UnicodeType) :
        return v
    else:
        if callable(v): v=v()
        return v

def stringify(v):
    return str(runnable(v))

def intify(v):
    return int(runnable(v))

def longify(v):
    return long(runnable(v))

def floatify(v):
    return float(runnable(v))

def booleanify(v):
    return not not runnable(v)

def listify(v):
    return [str(v)]

def nullconverter(v):
    return v


def value2date(v):
    """\
    Accepts empty strings (pass throw, no convertion).
    In Easy Publisher a NullDate is an empty string (not None)
    """
    v = runnable(v)
    if v=='': return v
    try:
        v = DateTime(v)
    except DateTime.SyntaxError, e:
        raise DateTime.SyntaxError, escape(e)
    return v

def value2text(v, nl=re.compile('\r\n|\n\r').search):
    if hasattr(v,'read'): v=v.read()
    else: v=str(v)
    mo = nl(v)
    if mo is None: return v
    l = mo.start(0)
    r=[]
    s=0
    while l >= s:
        r.append(v[s:l])
        s=l+2
        mo=nl(v,s)
        if mo is None: l=-1
        else:          l=mo.start(0)

    r.append(v[s:])

    return '\n'.join(r)


#FLOAT CONVERTERS

float_converters =  {
                    'string': stringify ,
                    'text': stringify ,
                    'dtml': stringify ,
                    'int': intify,
                    'long': longify,
                    'boolean': booleanify,
                    'list':listify,
                    }
#INT CONVERTERS

int_converters =   {
                    'string': stringify ,
                    'text': stringify ,
                    'dtml': stringify ,
                    'float': floatify,
                    'long': longify,
                    'boolean': booleanify,
                    'list':listify,
                    }


#LONG CONVERTERS

long_converters =   {
                    'string': stringify ,
                    'text': stringify ,
                    'dtml': stringify ,
                    'int': intify ,
                    'float': floatify ,
                    'boolean': booleanify ,
                    'list': listify ,
                    }

#BOOLEAN CONVERTERS

def boolean2string(v):
    if runnable(v):
        return "True"
    return "False"

boolean_converters =   {
                    'string': boolean2string ,
                    'text': boolean2string ,
                    'dtml': boolean2string ,
                    'int': intify,
                    'float': floatify,
                    'long': longify,
                    'list': listify,
                    }

#STRING CONVERTERS

def string2list(v):
    v=runnable(v)
    v=re.split(',|;', v)
    v=map(strip, v)
    v=map(lambda x: x and x[0]=='"' and x[-1]=='"' and x[1:-2] or x, v)
    return v


string_converters =   {
                    'date':    value2date,
                    'text':    value2text,
                    'dtml':    value2text,
                    'int':     intify,
                    'float':   floatify,
                    'long':    longify,
                    'boolean': booleanify,
                    'list':    string2list,
                    }

#DATE CONVERTERS

def date2string(v, fmt=None):
    if not fmt: fmt='%Y%m%d %H:%M:%S'
    v=runnable(v)
    v.strftime(fmt)

date_converters =   {
                    'string': date2string ,
                    'text': date2string ,
                    'dtml': date2string ,
                    'int':     intify,
                    'float':   floatify,
                    'long':    longify,
                    'boolean': booleanify,
                    'list': listify,
                    }

#TEXT CONVERTERS

def text2string(v):
    v=runnable(v)
    return str(' '.join(v.split('\n')))


def text2list(v):
    v=runnable(v)
    return v.split('\n')

text_converters =   {
                    'string':  text2string,
                    'dtml':    nullconverter ,
                    'int':     intify,
                    'float':   floatify,
                    'long':    longify,
                    'boolean': booleanify,
                    'list':    text2list,
                    }

#LIST CONVERTERS

def list2string(v):
    v=runnable(v)
    MLOG_TAB("list2string::v", v, type(v))
    return ', '.join(v)

def list2text(v):
    v=runnable(v)
    return '\n'.join(v)

def list2int(v):
    v=runnable(v)
    return int(reduce(lambda x, y: x + y, v, 0))

def list2long(v):
    v=runnable(v)
    return long(reduce(lambda x, y: x + y, v, 0))

def list2float(v):
    v=runnable(v)
    return float(reduce(lambda x, y: x + y, v, 0))

list_converters =   {
                    'string':  list2string,
                    'text':    list2text,
                    'dtml':    list2text ,
                    'int':     list2int,
                    'float':   list2float,
                    'long':    list2long,
                    'boolean': booleanify,
                    }

#DTML CONVERTERS

def dtml2string(v):
    return str(v)

def dtml2text(v):
    return str(v) #RENDER

def dtml2list(v):
    return str(v.split('\n'))

dtml_converters =   {
                    'string':  dtml2string,
                    'text':    dtml2text,
                    'int':     intify,
                    'float':   floatify,
                    'long':    longify,
                    'boolean': booleanify,
                    'list':    dtml2list,
                    }


easy_type_converters = {
    'float': float_converters,
    'int':   int_converters,
    'long': long_converters,
    'boolean': boolean_converters,
    'string': string_converters,
    'date': date_converters,
    'text': text_converters,
    'list':list_converters,
    'dtml':dtml_converters,
    }

default_values = {
    'float':   0.0,
    'int':     0,
    'long':    0L,
    'boolean': 0,
    'string':  '',
    'date':    '',
    'text':    '',
    'list':    [],
    'dtml':    '',
}

def getConverterFromTo(from_vt, to_vt):
    if from_vt == to_vt: return nullconverter
    from_converters=easy_type_converters.get(from_vt, None)
    if from_converters is None: return None
    return from_converters.get(to_vt, None)

def getTypeDefaultValue(value_type, default=None):
    return default_values.get(value_type, default)
