# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyCore/EasyCorePermissions.py,v 1.4 2004/07/13 11:05:22 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__ = """EasyCorePermissions."""
__version__='$Revision: 1.4 $'[11:-2]


import Globals, AccessControl, Products
from AccessControl import Permissions
from Products.CMFCore import CMFCorePermissions

# General Zope permissions
View = Permissions.view
AccessContentsInformation = Permissions.access_contents_information
UndoChanges = Permissions.undo_changes
ChangePermissions = Permissions.change_permissions
ViewManagementScreens = Permissions.view_management_screens
ManageProperties = Permissions.manage_properties
FTPAccess = Permissions.ftp_access
ManageUsers = Permissions.manage_users
DeleteObjects = Permissions.delete_objects

def setDefaultRoles(permission, roles):
    '''
    Sets the defaults roles for a permission.
    '''
    # XXX This ought to be in AccessControl.SecurityInfo.
    registered = AccessControl.Permission._registeredPermissions
    if not registered.has_key(permission):
        registered[permission] = 1
        Products.__ac_permissions__=(
            Products.__ac_permissions__+((permission,(),roles),))
        mangled = AccessControl.Permission.pname(permission)
        setattr(Globals.ApplicationDefaultPermissions, mangled, roles)

# Setting defaults for Zope Core Permissions
# Called on EasyRoot DB init
def setupZopeCorePermissions():
    permission = AccessControl.Permission.pname(AccessContentsInformation)
    setattr(Globals.ApplicationDefaultPermissions, permission, ('Anonymous', 'Manager', 'Author', 'Reviewer',))

    permission = AccessControl.Permission.pname(View)
    setattr(Globals.ApplicationDefaultPermissions, permission, ('Anonymous', 'Manager', 'Author', 'Reviewer',))

    permission = AccessControl.Permission.pname(ViewManagementScreens)
    setattr(Globals.ApplicationDefaultPermissions, permission, ('Manager', 'Author', 'Reviewer',))

    permission = AccessControl.Permission.pname(DeleteObjects)
    setattr(Globals.ApplicationDefaultPermissions, permission, ('Manager', 'Author', 'Reviewer',))

#
# Easy Core Permissions
#


#EasyEditor

EasyEditorAdd = 'Easy Editor Add'
setDefaultRoles(EasyEditorAdd, ('Manager',))

EasyTemplateAdd = 'Easy Template Add'
setDefaultRoles(EasyTemplateAdd, ('Manager',))

EasyEditorStyleAdd = 'Easy EditorStyle Add'
setDefaultRoles( EasyEditorStyleAdd, ( 'Manager',) )

EasyTemplateChange = 'Easy Template Change'
setDefaultRoles(EasyTemplateChange, ('Manager',))


#EasyRoot
EasyRootAdd = 'Easy Root Add'
setDefaultRoles(EasyRootAdd, ('Manager', ))

EasyRootChange = 'Easy Root Change'
setDefaultRoles(EasyRootChange, ('Manager', ))


#EasyDocuments

EasyUnrestricted = 'Easy Unrestricted'
setDefaultRoles(EasyUnrestricted, ('Manager',))

EasyDocumentAdd = 'Easy Document Add'
setDefaultRoles(EasyDocumentAdd, ('Manager', 'Author',))

EasyDocumentChange = 'Easy Document Change'
setDefaultRoles(EasyDocumentChange, ('Manager', 'Author',))

EasyImageAdd = 'Easy Image Add'
setDefaultRoles(EasyImageAdd, ('Manager', 'Author',))

EasyImageChange = 'Easy Image Change'
setDefaultRoles(EasyImageChange, ('Manager', 'Author',))

EasyFileAdd = 'Easy File Add'
setDefaultRoles(EasyFileAdd, ('Manager', 'Author',))

EasyFileChange = 'Easy File Change'
setDefaultRoles(EasyFileChange, ('Manager', 'Author',))

EasyTemplateViewChange = 'Easy Template/View Change'
setDefaultRoles(EasyTemplateViewChange, ('Manager', 'Author',))

EasyNavigationChange = 'Easy Navigation Change'
setDefaultRoles(EasyNavigationChange, ('Manager', 'Author',))

EasyMetadataChange = 'Easy Metadata Change'
setDefaultRoles(EasyMetadataChange, ('Manager', 'Author', 'Reviewer',))

EasyViewAhead = 'Easy View Ahead'
setDefaultRoles(EasyViewAhead, ('Manager',  'Author',))

EasyViewArchived = 'Easy View Archived'
setDefaultRoles(EasyViewArchived, ('Manager',  'Author',))

EasyNewsItemAdd = 'Easy NewsItem Add'
setDefaultRoles(EasyNewsItemAdd, ('Manager',  'Author',))


#EasyContentForm

EasyContactFormsSend = 'Easy ContactForms Send'
setDefaultRoles(EasyContactFormsSend, ('Manager', 'Anonymous',))

EasyContactFormChange = 'Easy ContactForm Change'
setDefaultRoles(EasyContactFormChange, ('Manager', 'Author',))

EasyContactFormAdd = 'Easy ContactForm Add'
setDefaultRoles(EasyContactFormAdd, ('Manager', 'Author',))


#EasyResources

EasyResourceChange = 'Easy Resource Change'
setDefaultRoles(EasyResourceChange, ('Manager',  'Author',))


#EasyServices

EasyMailServiceAdd = 'Easy Mail Service Add'
setDefaultRoles(EasyMailServiceAdd, ('Manager',))

EasyNotificationServiceAdd = 'Easy Notification Service Add'
setDefaultRoles(EasyNotificationServiceAdd, ('Manager',))

EasyNotificationManage = 'Easy Notification Manage'
setDefaultRoles(EasyNotificationManage, ('Manager',))


EasyNavigationServiceAdd = 'Easy Navigation Service Add'
setDefaultRoles(EasyNavigationServiceAdd, ('Manager',))

EasyUserFolderServiceAdd = 'Easy UserFolder Service Add'
setDefaultRoles(EasyUserFolderServiceAdd    , ('Manager',))

EasyNewsServiceAdd = 'Easy News Service Add'
setDefaultRoles(EasyNewsServiceAdd    , ('Manager',))

EasySearchServiceAdd = 'Easy Search Service Add'
setDefaultRoles(EasySearchServiceAdd    , ('Manager',))


#EasyWorkflow

EasyWorkflowServiceAdd = 'Easy WorkflowService Add'
setDefaultRoles(EasyWorkflowServiceAdd, ('Manager',))

EasyWorkflowServiceManage = 'Easy WorkflowService Manage'
setDefaultRoles(EasyWorkflowServiceManage, ('Manager',))

EasyWorkflowManagerAdd = 'Easy WorkflowManager Add'
setDefaultRoles(EasyWorkflowManagerAdd, ('Manager',))

EasyWorkflowView = 'Easy Workflow View'
setDefaultRoles(EasyWorkflowView, ('Manager', 'Author', 'Reviewer',))

EasyWorkflowSubmit = 'Easy Workflow Submit'
setDefaultRoles(EasyWorkflowSubmit, ('Manager', 'Author',))

##EasyWorkflowReview = 'Easy Workflow Review'
##setDefaultRoles(EasyWorkflowReview, ('Manager', 'Reviewer',))
##EasyWorkflowReview has been replace by EasyWorkflowPublish and EasyWorkflowDeny

EasyWorkflowPublish = 'Easy Workflow Publish'
setDefaultRoles(EasyWorkflowPublish, ('Manager', 'Reviewer',))

EasyWorkflowDeny = 'Easy Workflow Deny'
setDefaultRoles(EasyWorkflowDeny, ('Manager', 'Reviewer',))

EasyWorkflowRetract = 'Easy Workflow Retract'
setDefaultRoles(EasyWorkflowRetract, ('Author',))

EasyWorkflowManage = 'Easy Workflow Manage'
setDefaultRoles(EasyWorkflowManage, ('Manager', 'Author', 'Reviewer',))

EasyWorkflowChangeDate = 'Easy Workflow Change Date'
setDefaultRoles(EasyWorkflowChangeDate, ('Manager', 'Author', 'Reviewer',))

EasyWorkflowArchive = 'Easy Workflow Archive'
setDefaultRoles(EasyWorkflowArchive, ('Manager', 'Reviewer',))

#Editing permissions
EasyWorkflowEditPublished = 'Easy Workflow Edit Published'
setDefaultRoles(EasyWorkflowEditPublished, ('Manager', 'Reviewer',))

EasyWorkflowEditArchived = 'Easy Workflow Edit Archived'
setDefaultRoles(EasyWorkflowEditArchived, ('Manager', ))

EasyWorkflowEditDraft = 'Easy Workflow Edit Draft'
setDefaultRoles(EasyWorkflowEditDraft, ('Manager', 'Author',))

EasyWorkflowEditSubmitted = 'Easy Workflow Edit Submitted'
setDefaultRoles(EasyWorkflowEditSubmitted, ('Manager', ))


EasyAuthorsManage = 'Easy Authors Manage'
setDefaultRoles(EasyAuthorsManage, ('Manager',))

EasyReviewersManage = 'Easy Reviewers Manage'
setDefaultRoles(EasyReviewersManage, ('Manager',))

EasyUpgradeManage = 'Easy Upgrade Manage'
setDefaultRoles(EasyUpgradeManage, ('Manager',))

EasyIntrospektManage = 'Easy Introspekt Manage'
setDefaultRoles(EasyIntrospektManage, ('Manager',))



#EasyUsers

EasyOwnershipChange = 'Easy Ownership Change'
setDefaultRoles(EasyOwnershipChange, ('Manager', 'Owner', 'Reviewer',))


# EasyDiscussion

ReplyToItem = CMFCorePermissions.ReplyToItem

EasyDiscussionManage = 'Easy Discussion Manage'
setDefaultRoles(EasyDiscussionManage, ('Manager', 'Reviewer',))


#EasyVirtualHostMonster

EasyVHMManage = 'Easy VHM Manage'
setDefaultRoles(EasyVHMManage, ('Manager',))

#EasyLanguageServiceManage
EasyLanguageServiceManage = 'Easy LanguageService Manage'
setDefaultRoles(EasyLanguageServiceManage, ('Manager',))


#EasySynchronization


EasySynchronizeManage = 'Easy Synchronize Manage'
setDefaultRoles(EasySynchronizeManage, ('Manager',))

EasySynchronizeView = 'Easy Synchronize View'
setDefaultRoles(EasySynchronizeView, ('Manager',))


#EasySimpleObject

EasySimpleObjectAdd = 'Easy Simple Object Add'
setDefaultRoles(EasySimpleObjectAdd, ('Manager',))

EasySimpleObjectEdit = 'Easy Simple Object Edit'
setDefaultRoles(EasySimpleObjectEdit, ('Manager',))


def setupPermissions(self):
    # Set up some suggested role to permission mappings.
    mp = self.manage_permission

    mp(EasyEditorAdd,        ['Manager',],    0)
    mp(EasyTemplateAdd,        ['Manager',],    0)
    mp(EasyEditorStyleAdd,        ['Manager',],    0)
    mp(EasyTemplateChange,        ['Manager',],    0)
    mp(EasyRootAdd,        ['Manager',],    0)
    mp(EasyRootChange,        ['Manager',],    0)
    mp(EasyUnrestricted,        ['Manager',],    0)
    mp(EasyDocumentAdd,        ['Manager', 'Author',],    0)
    mp(EasyDocumentChange,        ['Manager', 'Author',],    0)

    mp(EasyImageAdd,        ['Manager', 'Author',],    0)
    mp(EasyImageChange,        ['Manager', 'Author',],    0)

    mp(EasyFileAdd,        ['Manager', 'Author',],    0)
    mp(EasyFileChange,        ['Manager', 'Author',],    0)
    mp(EasyTemplateViewChange,        ['Manager', 'Author',],    0)
    mp(EasyNavigationChange,        ['Manager', 'Author',],    0)
    mp(EasyMetadataChange,        ['Manager', 'Author', 'Reviewer',],    0)
    mp(EasyViewAhead,        ['Manager', 'Author',],    0)
    mp(EasyViewArchived,        ['Manager', 'Author',],    0)
    mp(EasyNewsItemAdd,        ['Manager', 'Author',],    0)

    #EasyContentForm
    mp(EasyContactFormsSend,        ['Manager', 'Anonymous'],    0)
    mp(EasyContactFormChange,        ['Manager', 'Author',],    0)
    mp(EasyContactFormAdd,        ['Manager', 'Author',],    0)

    #EasyResources
    mp(EasyResourceChange,        ['Manager', 'Author',],    0)

    #EasyServices
    mp(EasyMailServiceAdd,        ['Manager',],    0)
    mp(EasyNotificationServiceAdd,        ['Manager',],    0)
    mp(EasyNotificationManage,        ['Manager',],    0)
    mp(EasyNavigationServiceAdd,        ['Manager',],    0)
    mp(EasyUserFolderServiceAdd,        ['Manager',],    0)
    mp(EasyNewsServiceAdd,        ['Manager',],    0)
    mp(EasySearchServiceAdd,        ['Manager',],    0)

    #EasyWorkflow
    mp(EasyWorkflowServiceAdd,        ['Manager',],    0)
    mp(EasyWorkflowServiceManage,        ['Manager',],    0)
    mp(EasyWorkflowManagerAdd,        ['Manager',],    0)
    mp(EasyWorkflowView,        ['Manager', 'Author', 'Reviewer',],    0)
    mp(EasyWorkflowSubmit,        ['Manager', 'Author',],    0)
    mp(EasyWorkflowPublish,        ['Manager', 'Reviewer',],    0)
    mp(EasyWorkflowDeny,        ['Manager', 'Reviewer',],    0)
    mp(EasyWorkflowRetract,        ['Manager', 'Author',],    0)
    mp(EasyWorkflowManage,        ['Manager', 'Author', 'Reviewer',],    0)
    mp(EasyWorkflowChangeDate,        ['Manager', 'Author', 'Reviewer',],    0)
    mp(EasyWorkflowArchive,        ['Manager', 'Reviewer',],    0)

    #Editing permissions
    mp(EasyWorkflowEditPublished,        ['Manager', 'Reviewer',],    0)
    mp(EasyWorkflowEditArchived,        ['Manager',],    0)
    mp(EasyWorkflowEditDraft,        ['Manager', 'Author',],    0)
    mp(EasyWorkflowEditSubmitted,        ['Manager',],    0)
    mp(EasyAuthorsManage,        ['Manager',],    0)
    mp(EasyReviewersManage,        ['Manager',],    0)
    mp(EasyUpgradeManage,        ['Manager',],    0)
    mp(EasyIntrospektManage,        ['Manager',],    0)

    #EasyUsers
    mp(EasyOwnershipChange,        ['Manager', 'Owner', 'Reviewer',],    0)

    # EasyDiscussion
    mp(ReplyToItem,        ['Manager', 'Author',],    0)
    mp(EasyDiscussionManage,        ['Manager', 'Reviewer',],    0)
    mp(EasyVHMManage,        ['Manager',],    0)

    #EasyLanguageServiceManage
    mp(EasyLanguageServiceManage,        ['Manager',],    0)

    #EasySynchronization
    mp(EasySynchronizeManage,        ['Manager',],    0)
    mp(EasySynchronizeView,        ['Manager',],    0)

    #EasySimpleObject
    mp(EasySimpleObjectAdd,        ['Manager',],    0)
    mp(EasySimpleObjectEdit,        ['Manager',],    0)

    #Standard
    mp(View,        ['Manager', 'Author', 'Reviewer', 'Anonymous'],    0)
    mp(AccessContentsInformation,        ['Manager', 'Author', 'Reviewer', 'Anonymous'],    0)
    mp(UndoChanges,        ['Manager', 'Reviewer'],    0)
    mp(ChangePermissions,        ['Manager',],    0)
    mp(ViewManagementScreens,        ['Manager', 'Author', 'Reviewer'],    0)
    mp(ManageProperties,        ['Manager', 'Author', 'Reviewer'],    0)
    mp(FTPAccess,        ['Manager', 'Author', 'Reviewer'],    0)
    mp(ManageUsers,        ['Manager',],    0)
    mp(DeleteObjects,        ['Manager', 'Reviewer'],    0)


    mp('View History',        ['Manager', 'Author', 'Reviewer'],    0)
    mp('View source',        ['Manager', 'Author', 'Reviewer'],    0)
    mp('WebDAV Lock items',        ['Manager', 'Author', 'Reviewer'],    0)
    mp('WebDAV Unlock items',        ['Manager', 'Author', 'Reviewer'],    0)
    mp('WebDAV access',        ['Manager', 'Author', 'Reviewer'],    0)
