# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyCore/EasyObjectLink.py,v 1.1 2004/06/28 10:04:10 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyObjectLink"""
__version__='$Revision: 1.1 $'[11:-2]


import Globals
from types import StringType, TupleType

from ExtensionClass import Base
from Persistence import Persistent
from Acquisition import Implicit
from OFS.DefaultObservable import DefaultObservable
from AccessControl import ClassSecurityInfo
from AccessControl import Permissions

# General Zope permissions
View = Permissions.view
AccessContentsInformation = Permissions.access_contents_information

import os
DEBUG_OBSERVABLE = os.environ.has_key('DEBUG_OBSERVABLE')

from Products.EasyCore import EasyCorePermissions
EasyCorePermissions.EasyLinkManage = "Easy Link Manage"
EasyCorePermissions.setDefaultRoles(EasyCorePermissions.EasyLinkManage, ('Manager',))

from interfaces.EasyObjectLink import EasyObjectLinkable, EasyLinkPolicy
from LinkAssessments import *


from Products.EasyCore.Utils.DebugUtils import MLOG_TAB, C_MLOG_TAB


class PolicyMismatchError(Exception): pass
class LinkNameError(Exception): pass
class LinkPathError(Exception): pass


class ObjectLinkPolicyBase(Persistent, Implicit, Base):

    __implements__ = (EasyLinkPolicy, )

    def isLinkCreationAllowed(self, link_name, local_obj, remote_obj):
        return 1

    def isLinkDestructionAllowed(self, link_name, local_obj, remote_obj):
        return 1

    def getLinkCreationAssessment(self, link_name, local_obj, remote_obj):
        return None

    def getLinkDestructionAssessment(self, link_name, local_obj, remote_obj):
        return None

class DefaultLinkPolicy(ObjectLinkPolicyBase): pass

class SingleLinkPolicy(ObjectLinkPolicyBase):

    def isLinkCreationAllowed(self, link_name, local_obj, remote_obj):
        return len(local_obj._object_links.get(link_name,[]))==0

    def getLinkCreationAssessment(self, link_name, local_obj, remote_obj):
        if len(local_obj._object_links.get(link_name,[]))!=0:
            return MultipleLinksNotAllowed()
        return None

class EasyObjectLink(Base):
    """
    """

    security = ClassSecurityInfo()


    __implements__ = (EasyObjectLinkable, )

    _link_policys = {}
    #_object_links = {} #things tend to get screwed-up with this defined in the class? Use upaged instead.

    def __init__(self):
        self._object_links = {}

    def _setPath(self, link_name, path):
        links = self._object_links.get(link_name,[])
        if path not in links:
            links.append(path)
            self._object_links[link_name] = links
            self._p_changed=1

    def _removePath(self, link_name, path):
        object_links=self._object_links
        links = object_links.get(link_name,None)
        if links is not None:
            if path in links:
                links.remove(path)
                if len(links)==0:
                    del object_links[link_name]
                else:
                    object_links[link_name]=links
                self._object_links=object_links
                return 1
        return 0

    def _unpackLink(self, link_name):
        if link_name is None: return None, None
        if type(link_name)==StringType:
            return (link_name, 'reverse_'+link_name), ('reverse_'+link_name, link_name)
        elif type(link_name)==TupleType and len(link_name)==2:
            return (link_name[0], link_name[1]), (link_name[1], link_name[0])
        raise LinkNameError, "The link name [%s] was invalid." % (link_name,)

    def _reverse_link(self, link_name):
        if type(link_name)!=TupleType or len(link_name)!=2:
            link_name = self._unpackLink(link_name)
        return(link_name[1],link_name[0])


    #I'm not sure about this path stuff.
    #Shouldn't we use Links instead???
    #Path is just a part of the implementation

    security.declareProtected(AccessContentsInformation, 'getPaths')
    def getPaths(self, link_name, suppress_exception=0):
        forward_link, reverse_link = self._unpackLink(link_name)
        paths = self._object_links.get(forward_link, None)
        if paths: return list(paths)
        elif paths is None and self._link_policys.has_key(forward_link):
            return []
        if suppress_exception: return []
        raise LinkNameError, "Link %s doesn't exists." % (link_name,)

    #And this should probably be validate link instead???
    def validatePath(self, path):
        """Returns true if the path if valide (e.g. point to an object)"""
        remoteObj = self.restrictedTraverse( path, None )
        return not not remoteObj

    security.declareProtected(AccessContentsInformation, 'hasLink')
    def hasLink(self, link_name):
        forward_link, reverse_link = self._unpackLink(link_name)
        return self._object_links.has_key(forward_link) or self._link_policys.has_key(forward_link)

    security.declareProtected(AccessContentsInformation, 'isLinkEmpty')
    def isLinkEmpty(self, link_name, suppress_exception=0):
        forward_link, reverse_link = self._unpackLink(link_name)
        paths = self._object_links.get(forward_link, None)
        if paths is not None:
            return len(paths)==0
        else:
            if self._link_policys.has_key(forward_link):
                return 1
        if suppress_exception: return 1
        raise LinkNameError, "Link %s doesn't exists." % (link_name,)

    security.declareProtected(AccessContentsInformation, 'getLinkIds')
    def getLinkIds(self):
        return self._object_links.keys()

    security.declareProtected(AccessContentsInformation, 'getLocalLinkIds')
    def getLocalLinkIds(self):
        return map(lambda x: x[0], self._object_links.keys())

    security.declareProtected(AccessContentsInformation, 'getLinkItems')
    def getLinkItems(self, link_name):
        forward_link, reverse_link = self._unpackLink(link_name)
        return map(lambda x, link_name=forward_link: (link_name,x), self._object_links.get(forward_link,[]))

    security.declareProtected(AccessContentsInformation, 'getAllLinkItems')
    def getAllLinkItems(self):
        return self._object_links.items()

    security.declareProtected(AccessContentsInformation, 'getLinkValues')
    def getLinkValues(self, link_name):
        forward_link, reverse_link = self._unpackLink(link_name)
        return map(lambda x:  x, self._object_links.get(forward_link,[]))

    security.declareProtected(AccessContentsInformation, 'getAllLinkValues')
    def getAllLinkValues(self):
        return self._object_links.values()

    def _getPolicy(self, link_name, policy=None):
        existing_policy = self._link_policys.get(link_name, None)
        if existing_policy is None and policy is not None:
            self._link_policys[link_name] = policy
            return policy
        elif existing_policy is not None:
            if policy is not None and existing_policy is not policy:
                raise PolicyMismatchError, "The requested LinkPolicy doesn't match the existing LinkPolicy."
            return existing_policy
        policy = DefaultLinkPolicy()
        self._link_policys[link_name] = policy
        return policy

    security.declareProtected(EasyCorePermissions.EasyLinkManage, 'createLink')
    def createLink( self, link_name, path, policy=None):
        #This method should raise an exception when failing!
        forward_link, reverse_link = full_name = self._unpackLink(link_name)
        remoteObj = self.restrictedTraverse( path, None )
        if remoteObj is not None:
            policy = self._getPolicy(forward_link, policy)
            locally_verified = policy.getLinkCreationAssessment(forward_link, self, remoteObj)
            if locally_verified is None:
                remote_verified = remoteObj.verifyLinkCreation(reverse_link, self.getPhysicalPath())
                if remote_verified:
                    self._setPath(forward_link, path)
                    return None
            else:
                return locally_verified
        self._removePath(forward_link, path)
        return LinkRemovedOnInvalidRemoteLink()


    security.declarePrivate('verifyLinkCreation')
    def verifyLinkCreation( self, link_name, path ):
        forward_link, reverse_link = full_name = self._unpackLink(link_name)
        remoteObj = self.restrictedTraverse( path, None )
        if link_name is not None and remoteObj is not None:
            policy = self._getPolicy(forward_link)
            locally_verified = policy.getLinkCreationAssessment(forward_link, self, remoteObj)
            if locally_verified is None:
                remote_verified = remoteObj.verifyLinkCreation(None, path)
                links=self._object_links.get(forward_link, [])
                if remote_verified:
                    if path not in links:
                        self._setPath(forward_link, path)
                    return 1
                else:
                    if path in links:
                        self._removePath(forward_link, path)
                    return 0
        elif link_name is None and remoteObj is not None:
            return 1
        return 0

    security.declareProtected(EasyCorePermissions.EasyLinkManage, 'deleteLink')
    def deleteLink( self, link_name, path):
        full_name = self._unpackLink(link_name)
        #Pass along assessment or None (don't force deletion on invalid path)
        return self._deleteLink(full_name, path)

    def _deleteLink(self, full_name, path, force_remove=0):
        forward_link, reverse_link = full_name
        remoteObj = self.restrictedTraverse( path, None)
        if remoteObj is not None:
            policy = self._getPolicy(forward_link)
            locally_verified = policy.getLinkDestructionAssessment(forward_link, self, remoteObj)
            if locally_verified is None:
                #should we evaluate that the remove deletion is ok before
                #proceding with the local deletion?
                remoteObj.deleteReference(reverse_link, self.getPhysicalPath())
                self._removePath(forward_link, path)
            return locally_verified
        #TODO: Should we silently ignore the fact that the remote object can be found?
        if force_remove:
            self._removePath(forward_link, path)
            return LinkRemovedOnInvalidRemoteLink()
        # Should this return None instead, do we want an message to be formed?
        return LinkUnchanged()

    security.declarePrivate('deleteReference')
    def deleteReference( self, link_name, path, suppress_exception=0):
        remoteObj = self.restrictedTraverse( path, None )
        if remoteObj:
            policy = self._getPolicy(link_name)
            locally_verified = policy.getLinkDestructionAssessment(link_name, self, remoteObj)
            if locally_verified is None:
                self._removePath(link_name, path)
            return locally_verified
        if suppress_exception: return
        raise LinkPathError, "Invalid path to remove object %s." % (path,)

    security.declareProtected(EasyCorePermissions.EasyLinkManage, 'deleteAllLinkByLinkName')
    def deleteAllLinkByLinkName(self, link_name):
        failed_assessments=[]
        forward_link, reverse_link = full_name = self._unpackLink(link_name)
        paths=self.getPaths(forward_link)
        for path in paths:
            #Force deletion here!
            #Pass along assessment or None (force deletion on invalid path)
            assessment = self._deleteLink(full_name, path, force_remove=1)

            if assessment is not None:
                assessment.full_name=forward_link
                assessment.reverse_link=reverse_link
                assessment.path=path
                failed_assessments.append(assessment)
        return failed_assessments

    security.declarePrivate('disableRemoteLinks')
    def disableRemoteLinks(self):
        local_path=self.getPhysicalPath()
        for link_name, links in self._object_links.items():
            rev_link_name = self._reverse_link(link_name)
            for path in links:
                remoteObj = self.restrictedTraverse( path, None )
                if remoteObj:
                    #_removePath doesn't care about what the
                    #policy say, this may lead to conflict
                    #when the link is recreated later (in a cut&paste situation)
                    remoteObj._removePath(rev_link_name, local_path)


    security.declarePrivate('recreateRemoteLinks')
    def recreateRemoteLinks(self):
        local_path=self.getPhysicalPath()
        for link_name, links in self._object_links.items():
            rev_link_name = self._reverse_link(link_name)
            for path in links:
                remoteObj = self.restrictedTraverse( path, None )
                if remoteObj:
                    #This may be a problem if the policy
                    #is not willing to recreate the link in
                    #the target.
                    remoteObj.verifyLinkCreation(rev_link_name, local_path)

    def manage_afterAdd(self, item, container):
        self.recreateRemoteLinks()

    def manage_beforeDelete(self, item, container):
        self.disableRemoteLinks()

    def manage_afterClone(self, item):
        pass

        #We need to add a handler that checks remotely if
        #links shall be created by a cloning object add
        #the remote object.
        #The links local policy should check first if cloning
        #links is allowed if it is we should prodceed to ask
        #the remote object if they allow cloned links.
        #This will be tested for each link and for each remote
        #object individually.
        #It's beoned the scoop for this project.
        #The current behavior is that the local link
        #remains and gets recreaded (by manage_afterAdd) which
        #happens before manage_afterClone. This leads to that
        #links get cloned for object that allow more tha one link.
        #In other words MasterCopy object in the EasyMasterSlaveCopy
        #module should not be cloned, but slave copies can be cloned.

    def __setstate__(self, state):
        Persistent.__setstate__(self, state)
        if not hasattr(self, '_object_links'): self._object_links = {}

    security.declareProtected(EasyCorePermissions.EasyLinkManage, 'clearInvalidLinks')
    def clearInvalidLinks(self, just_show=0):
        """Clear invalid slaves"""
        invalide=[]
        for LINKNAME, links in self._object_links.items():
            links=list(links)
            for link_path in links:
                path=link_path[:-1]
                linkedObj = self.restrictedTraverse(link_path, None)
                if linkedObj is None:
                    invalide.append(link_path)
                    if not just_show:
                        self.deleteLink(LINKNAME, link_path)
                        #if self._removePath(MASTER_LINK, slave_path):
        return invalide

    security.declareProtected(EasyCorePermissions.EasyLinkManage, 'peek')
    def peek(self, REQUEST=None):
        """peek"""
        out='<pre>'
        for key,value in self._object_links.items():
            out=out+"%s:\n" % str(key)
            for v in value:
                out=out+"....%s\n" % str(v)
        if REQUEST is not None:
            return "_object_links", out
        return out


Globals.InitializeClass(EasyObjectLink)


