# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyCore/EasySkinnable.py,v 1.1 2004/06/28 10:04:10 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__="""\
EasySkinnable.py
"""
__version__='$Revision: 1.1 $'[11:-2]

import Globals
from types import StringType
from ExtensionClass import Base
#import Products, DateTime
from Products.CMFCore.Skinnable import SkinnableObjectManager
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder
from OFS.ObjectManager import ObjectManager
from Globals import DTMLFile
from Acquisition import aq_chain, aq_self, aq_base, aq_inner, aq_parent, aq_acquire
from Acquisition import ImplicitAcquisitionWrapper, Implicit
from ZODB.PersistentMapping import PersistentMapping
from BTrees.IOBTree import IOBTree
#from BTrees.Length import Length
#from BTrees.IOBTree import union, intersection, difference


from Products.EasyLanguageService.EasyLanguageHelpers import getLanguageByRFC3066
from Products.EasyCore.Utils.Utils import distinct
from Products.EasyCore.Utils.PCounter import PCounter
from Products.EasyCore.GUID import GUIDGenerator

from Products.EasyCatalog.EasyCatalogAware import EasyCatalogAwareObjectAndPropertyManager


_marker = []  # Create a new marker object.


# superGetAttr is assigned to whatever ObjectManager.__getattr__
# used to do.
try:
    superGetAttr = Folder.__getattr__
except:
    try:
        superGetAttr = Folder.inheritedAttribute('__getattr__')
    except:
        superGetAttr = None


class VeryTinyItem(Base, Globals.Persistent, Implicit):
    """Needed for older version of EasyApplication"""

    def copy(self):
        """Make a shallow copy of myself"""
        newitem = VeryTinyItem()
        for attrib in self.__dict__.keys():
            setattr(newitem, attrib, getattr(self, attrib))
        return newitem


class EasySkinnable(EasyCatalogAwareObjectAndPropertyManager, ObjectManager):
    """\
    Base class for skinnable object managers.
    Handles "skins" from Templates folders as well as
    versions and language versions.
    """

    security = ClassSecurityInfo()

    _v_templateid=None
    _v_skindata = None


    def getSkinsFolderName(self):
        return 'EasyEditor'

    security.declarePublic('changeSkin')
    def changeSkin(self, skinname):
        self._v_skindata = None
        skinobj = self.getSkin(skinname)
        if skinobj is not None:
            self._v_skindata = (self.REQUEST, skinobj, {})

    security.declarePublic('setupCurrentSkin')
    def setupCurrentSkin(self, REQUEST=None):
        '''
        Sets up _v_skindata so that __getattr__ can find it.
        Can also be called manually, allowing the user to change
        skins in the middle of a request.
        '''
        if REQUEST is None:
            REQUEST = getattr(self, 'REQUEST', None)
        if REQUEST is None:
            # self is not fully wrapped at the moment.  Don't
            # change anything.
            return
        if self._v_skindata is not None and self._v_skindata[0] is REQUEST:
            # Already set up for this request.
            return
        templateid = self.getTemplateId()
        self._v_templateid = templateid
        REQUEST.set('templateid', templateid)
        skinname = self.getSkinNameFromRequest(REQUEST)
        self.changeSkin(skinname)


    def __getattr__(self, name):
        '''
        Looks for the name in an object with wrappers that only reach
        up to the root skins folder.  This should be fast, flexible,
        and predictable.
        '''
        if not name.startswith('_') and not name.startswith('aq_'):
            sd = self._v_skindata
            if sd is not None:
                request, ob, ignore = sd
                #Lacy SkinLookup.
                ##if type(ob) is StringType: self.changeSkin(ob)
                if not ignore.has_key(name):
                    subob = getattr(ob, name, _marker)
                    if subob is not _marker:
                        # Return it in context of self, forgetting
                        # its location and acting as if it were located
                        # in self.
                        return aq_base(subob)
                    else:
                        ignore[name] = 1
        raise AttributeError, name

    security.declarePrivate('getSkin')
    def getSkin(self, name=None):
        """Returns the requested skin.
        """
        skinob = None
        skinstool = None
        sfn = self.getSkinsFolderName()

        if sfn is not None:
            sf = getattr(self, sfn, None)
            if sf is not None:
               if name is not None:
                   skinob = sf.getSkinByName(name)
               if skinob is None:
                   skinob = sf.getSkinByName(sf.getDefaultSkin())
                   if skinob is None:
                       skinob = sf.getSkinByPath('')
        return skinob


    security.declarePublic('getSkinNameFromRequest')
    def getSkinNameFromRequest(self, REQUEST=None):
        ''' returns the skin name from the Request'''
        sfn = self.getSkinsFolderName()
        if sfn is not None:
            sf = getattr(self, sfn, None)
            if sf is not None:
                return REQUEST.get(sf.getRequestVarname(), None)



    def __of__(self, parent):
        '''
        Sneakily sets up the portal skin then returns the wrapper
        that Acquisition.Implicit.__of__() would return.
        '''
        w_self = ImplicitAcquisitionWrapper(aq_base(self), parent)
        try:
            w_self.setupCurrentSkin()
        except:
            # This shouldn't happen, even if the requested skin
            # does not exist.
            import sys
            from zLOG import LOG, ERROR
            LOG('CMFCore', ERROR, 'Unable to setupCurrentSkin()',
                error=sys.exc_info())
        return w_self


    def _checkId(self, id, allow_dup=0):
        '''
        Override of ObjectManager._checkId().  Allows the user
        to create objects with IDs that match the ID of a skin
        object.
        '''
        ##superCheckId = ObjectManager.inheritedAttribute('_checkId')
        if not allow_dup:
            # Temporarily disable _v_skindata.
            # Note that this depends heavily on Zope's current thread
            # behavior.
            sd = self._v_skindata
            self._v_skindata = None
            try:
                base = getattr(self,  'aq_base', self)
                if not hasattr(base, id):
                    # Cause _checkId to not check for duplication.
                    ##return superCheckId(self, id, allow_dup=1)
                    return ObjectManager._checkId(self, id, allow_dup=1)
            finally:
                self._v_skindata = sd

        return ObjectManager._checkId(self, id, allow_dup)
        ##return superCheckId(self, id, allow_dup)


    def valid_property_id(self, id):
        """
        Override of EasyCatalogAwareObjectAndPropertyManager.valid_property_id().
        Allows the user to create properties with IDs that match the ID of
        a skin property.
        """
        ##super_valid_property_id = EasyCatalogAwareObjectAndPropertyManager.valid_property_id
        # Temporarily disable _v_skindata.
        # Note that this depends heavily on Zope's current thread
        # behavior.
        if getattr(self, 'overide_properties', 1):
            sd = self._v_skindata
            self._v_skindata = None
            try:
                if not id or id[:1]=='_' or (id[:3]=='aq_') \
                   or (' ' in id) or hasattr(aq_base(self), id):
                    return 0
                return 1
            finally:
                self._v_skindata = sd
        return EasyCatalogAwareObjectAndPropertyManager.valid_property_id(self, id)
        ##return super_valid_property_id(self, id)


Globals.InitializeClass(EasySkinnable)
