# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyCore/EasySortable/EasySortable.py,v 1.1 2004/06/28 10:04:11 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__ = """EasySortable"""
__version__='$Revision: 1.1 $'[11:-2]

import Globals
from Globals import DTMLFile
from AccessControl import ClassSecurityInfo
from ExtensionClass import Base

from Products.EasyCore.Utils.DebugUtils import MLOG_TAB, C_MLOG_TAB

ManageProperties='Manage properties'
OrderItems='Easy Order Items'

def getItemOrAttr(item, id, default=None):
    if hasattr(item, id):
        return getattr(item, id)
    if hasattr(item, 'get'):
        return item.get(id, default)
    return default

def dropIds(thislist, id_name='id'):
    return map(lambda x, id_name=id_name: getItemOrAttr(x, id_name), thislist)

def splitListOnMovedIds(moved_ids, item_list, id_name='id'):
    """\
    Returns a tuple of:
    - the items in moved_ids in the order as
      they appear in item_list
    - The sorted list of item in moved_ids, excluding any faulty ids.
    - the remaining items in item_list
      (as originaly order expet for the moved_ids)
    """
    sorted_moved_items = []
    items_before = []
    items_after=list(item_list)
    items_after.reverse()
    moved_ids=list(moved_ids)

    found_item=items_after.pop()
    while getItemOrAttr(found_item, id_name) not in moved_ids and len(items_after):
        items_before.append(found_item)
        found_item=items_after.pop()
    sorted_moved_items.append(found_item)
    moved_ids.remove(getItemOrAttr(found_item, id_name))

    items_after.reverse()
    while len(moved_ids):
        found_id=None
        for item in items_after:
            if getItemOrAttr(item, id_name) in moved_ids:
                found_id=getItemOrAttr(item, id_name)
                found_item=item
                break
        if found_id is not None:
            sorted_moved_items.append(found_item)
            items_after.remove(found_item)
            moved_ids.remove(found_id)
        else:
            #No item id match the moved_ids,
            #this means that moved_ids only
            #contains faulty ids.
            #Break main loop
            break

    return (items_before, sorted_moved_items, items_after)



class EasySortable(Base):

    security = ClassSecurityInfo()

    def moveDown(self, moved_ids, item_list, id_name='id', steps=1):
        """\
        Move moved_ids in list down.

        Returns the sorted list.
        """
        if not moved_ids: return item_list
        steps=max(steps,1)
        before,moved,after=splitListOnMovedIds(moved_ids, item_list, id_name=id_name)
        return before+after[0:steps]+moved+after[steps:]


    def moveUp(self, moved_ids, item_list, id_name='id', steps=1):
        """\
        Move moved_ids in list up.

        Returns the sorted list.
        """
        if not moved_ids: return item_list
        steps=max(steps,1)
        before,moved,after=splitListOnMovedIds(moved_ids, item_list, id_name=id_name)
        return before[:-steps]+moved+before[-steps:]+after


    def moveTop(self, moved_ids, item_list, id_name='id'):
        """\
        Move moved_ids in list to the top.

        Returns the sorted item_list.
        """
        if not moved_ids: return item_list
        before,moved,after=splitListOnMovedIds(moved_ids, item_list, id_name=id_name)
        return moved+before+after


    def moveBottom(self, moved_ids, item_list, id_name='id'):
        """\
        Move moved_ids in list to the bottom.

        Returns the sorted list.
        """
        if not moved_ids: return item_list
        before,moved,after=splitListOnMovedIds(moved_ids, item_list, id_name=id_name)
        return before+after+moved

Globals.InitializeClass(EasySortable)


class EasySortablePropertyManager(EasySortable):

    security = ClassSecurityInfo()

    security.declareProtected(ManageProperties, 'manage_property_moveDown')
    def manage_property_moveDown(self, ids, steps=1, REQUEST=None):
        """\
        Move the selected properties down steps.
        """
        self._properties=tuple(self.moveDown(ids, self._properties, steps=steps))
        if REQUEST is not None:
            return self.manage_propertiesForm(self, REQUEST, ids=ids)

    security.declareProtected(ManageProperties, 'manage_property_moveUp')
    def manage_property_moveUp(self, ids, steps=1, REQUEST=None):
        """\
        Move the selected properties down steps.
        """
        self._properties=tuple(self.moveUp(ids, self._properties, steps=steps))
        if REQUEST is not None:
            return self.manage_propertiesForm(self, REQUEST, ids=ids)

    security.declareProtected(ManageProperties, 'manage_property_moveTop')
    def manage_property_moveTop(self, ids, REQUEST=None):
        """\
        Move the selected properties down steps.
        """
        self._properties=tuple(self.moveTop(ids, self._properties))
        if REQUEST is not None:
            return self.manage_propertiesForm(self, REQUEST, ids=ids)

    security.declareProtected(ManageProperties, 'manage_property_moveBottom')
    def manage_property_moveBottom(self, ids, REQUEST=None):
        """\
        Move the selected properties to the bottom.
        """
        self._properties=tuple(self.moveBottom(ids, self._properties))
        if REQUEST is not None:
            return self.manage_propertiesForm(self, REQUEST, ids=ids)


Globals.InitializeClass(EasySortablePropertyManager)
