
from zLOG import LOG, INFO, ERROR, TRACE, DEBUG, BLATHER

class NullWriter:

    def __init__(self, REQUEST=None):
        self.request=REQUEST
        self.response=getattr(REQUEST, 'RESPONSE', None)

    def write(self, out, log_level=INFO):
        pass

    def flush(self):
        pass

    def cleanup(self):
        pass



class HTMLWriter:

    cache=[]

    def __init__(self, header=None
                     , items=None
                     , footer=None, use_cache=1, REQUEST=None):
        self.request=REQUEST
        self.response=getattr(REQUEST, 'RESPONSE', None)
        self.header=header
        self.header_done=0
        self.items=items
        self.footer=footer
        self.footer_done=0
        self.use_cache=use_cache
        self.cache=[]
        self.add_to_cache=self.cache.append

    def write(self, log_level=INFO, out=None):
        if not self.header_done and self.header:
            if callable(self.header):
                self.add_to_cache(self.header(self, REQUEST))
        if self.items and out is not None:
            self.add_to_cache(self.items(self, REQUEST, message=out))
        if not self.use_cache:
            self.flush()

    def flush(self):
        cache=self.cache
        cache.reverse()
        while cache:
            self.response.write(cache.pop())

    def cleanup(self):
        if self.footer:
            self.add_to_cache(self.footer(self, REQUEST))
        self.flush()

class LogWriter:

    def __init__(self, title="LogWriter", REQUEST=None):
        self.request=REQUEST
        self.response=getattr(REQUEST, 'RESPONSE', None)
        self.title=title

    def write(self, log_level=INFO, out=None):
        if out is not None:
            LOG(self.title, log_level, out)

    def flush(self):
        pass

    def cleanup(self):
        pass


class Results:

    log_level=ERROR
    result_index={}
    needs_reindexing=0
    writers=()
    header="Results"
    level=0

    def __init__(self, reindex=0, log_level=ERROR, writers=None, header="Results", level=0):
        self.log_level=log_level
        self.result_index={}
        self.needs_reindexing=reindex
        if writers is None: self.writers=()
        else: self.writers=writers
        self.header=header
        self.level=level

    def setResult(self, result):
        self.result_index[result.id]=result

    def getResult(self, object, create=1):
        try:
            id=object.getPhysicalPath()
        except:
            id=object.absolute_url(relative=1)
        if create:
            return self.result_index.get(id, Result(id, object, self))
        return self.result_index.get(id, None)

    def keys(self):
        return self.result_index.keys()

    def values(self):
        return self.result_index.values()

    def items(self):
        return self.result_index.items()

    def setDoReindex(self):
        self.needs_reindexing=1

    def getDoReindex(self):
        return self.needs_reindexing

    def write(self, log_level=INFO, *args):
        if self.log_level>=log_level:
            for writer in self.writers:
                for arg in args:
                    writer.write(log_level, arg)

    def flush(self):
        for writer in self.writers:
            writer.flush()


class Result:

    #{'id':ob.id(), 'result':2, 'meta_type':ob.meta_type, 'error_msg':''}
    def __init__(self, id, obj, parent, results=(), messages=(), errors=(), log_level=ERROR):
        self.id = id
        self.object_id=obj.getId()
        meta_type = obj.meta_type
        if callable(meta_type): meta_type=meta_type()
        self.object_meta_type=meta_type
        self.prev_revision=getattr(obj, '_object_version', '--')
        self.messages=messages
        self.errors=errors
        self.results=results
        self.parent=parent
        self.has_been_reindexed=0
        self.log_level=log_level

    def addMessage(self, message):
        self.messages=self.messages+(message,)

    def addError(self, error):
        self.errors=self.errors+(error,)

    def addResult(self, result):
        self.results=self.results+(result,)

    def setReindexed(self):
        self.has_been_reindexed=1

    def show(self, log_level=None):
        self.log_level=log_level or INFO

    def hide(self, log_level=None):
        self.log_level=log_level or ERROR

    def isVisible(self):
        return self.log_level >= 0

    def write(self, *args, **kw):
        return apply(self.parent.write, arg, kw)

    def write_all(self, log_level=INFO):
        if self.log_level >=INFO:
            for m in self.messages:
                self.parent.write(INFO, str(m))
        if self.log_level >=ERROR:
            for e in self.errors:
                self.parent.write(ERROR, str(e))

    def flush(self):
        self.results.flush()

