
__doc__='''parse_parameters module.'''
__version__='1.8'

#import Globals
import re
from types import StringType, TupleType, ListType, IntType, LongType, FloatType
SeqvenceType = (ListType,TupleType)

from string import split, strip, count
from ZPublisher.Converters import type_converters
from DateTime import DateTime
from time import strftime

#This module should define a parameter class, not module methods!?


#string format:
# id:type=default_value
#
#data format
# _parameters = ({'id':'id', 'type': 'type', 'default': default_value},)

TUPLE_DESC_TYPE = "tuple"
LIST_DESC_TYPE = "list"

from Products.EasyCore import MLOG_TAB

def str2str(value, do_strip=1):
    value=str(value)
    if do_strip: value=strip(value)
    if not value: return
    if (value[0]=='"' and value[-1]=='"') or (value[0]=="'" and value[-1]=="'"):
        value=value[1:-1]
    elif (value[0:3]=='"""' and value[-3:]=='"""') or (value[0:3]=="'''" and value[-3:]=="'''"):
        value=value[3:-3]
    return value

isMultilineText=lambda v: '\n' in v

def field2parameter(parameter, para_re=re.compile(r'([^ =:]+)(:([^ =:]+))?(=(.+))?')):
    pid, ptype, pdefault= '#Error', 'error', ''
    parsed=para_re.match(parameter)
    if para_re:
        pid, ptype, pdefault= parsed.group(1),parsed.group(3),parsed.group(5)
    else:
        pid, ptype, pdefault= '#Error', 'error', parameter
        return pid, ptype, pdefault
    if pid is None:
        pid, ptype, pdefault= '#Error', 'error', parameter
        return pid, ptype, pdefault

    converter=local_type_converters.get(ptype, None)
    if converter is not None:
        pdefault=converter(pdefault)
    else:
        try: pdefault=eval(pdefault, {}, {})
        except:pass

    if not type_defaults.has_key(ptype) and type_to_string_mapping.has_key(type(pdefault)):
        ptype = type_to_string_mapping.get(type(pdefault), ptype)
    elif type_defaults.has_key(ptype) and not type_to_string_mapping.has_key(type(pdefault)):
        pdefault = type_defaults.get(ptype, pdefault)

    if ptype=='string' and isMultilineText(pdefault): ptype='text'

    return pid, ptype, pdefault


def fields2parameters(parameters, include_errors=1):
    _parameters = ()
    _error_parameters = ()
    for prop in parameters:
        pid, ptype, pdefault = field2parameter(prop)
        if ptype!='error':
            _parameters = _parameters + ({'id':pid, 'type':ptype, 'default':pdefault },)
        else:
             _error_parameters = _error_parameters + ({'id':pid, 'type':ptype, 'default':pdefault },)
    if include_errors:
        return _parameters, _error_parameters
    return _parameters


def parameter2field(parameter, include_errors=1):
    result, ptype, pdefault=parameter['id'], parameter['type'], parameter['default']
    if ptype and (ptype!='error' or include_errors):
        result=result + ":" + ptype
        converter=type_templates.get(ptype, None)
        if converter:
            result=result + "=" + converter(pdefault)
        else:
            result=result + "=" + str(pdefault)
    return result

def parameters2fields(parameters):
    result = []
    for parameter in parameters:
        result.append(parameter2field(parameter))
    return result

NullDate = ""

type_defaults = {
    'float':    0.0,
    'int':      0,
    'long':     0L,
    'string':   '',
    'date':     NullDate,
    'required': 1,
    'tokens':   [],
    'lines':    [],
    'text':     '',
    'boolean':  0,
    'tuple':    (),
    'list':     [],
    'dict':     {},
    'error':    '',
    }

type_to_string_mapping = {
    type(0.0):          'float',
    type(0):            'int',
    type(0L):           'long',
    type(''):           'string',
    type(DateTime()):   'date',
    type(()):           'tuple',
    type([]):           'list',
    type({}):           'dict',
}

def strftime_t(value, format="%Y-%m-%d H:%M:%S +0000"):
    return strftime(value, format)

def quote_boolean(value):
    return not not value

def text_repr(v):
    return '"""%s"""' % repr(v)[1:-1]

def null(v):
    return v

def safe_eval(expr):
    try: return eval(expr, {}, {})
    except: return expr


type_templates = {
    'float':    str,
    'int':      str,
    'long':     repr,
    'string':   repr,
    'date':     strftime_t,
    'required': repr,
    'tokens':   repr,
    'lines':    repr,
    'tuple':    repr,
    'list':     repr,
    'dict':     repr,
    'text':     text_repr,
    'boolean':  quote_boolean,
    'error':    str,
    }


local_type_converters=type_converters.copy()
local_type_converters.update({
    'dict':     safe_eval,
    'tuple':    lambda x: tuple(safe_eval(x)),
    'list':     lambda x: list(safe_eval(x)),
    'error':    null,
    'string':   lambda x: str(safe_eval(x)),
    'text':     lambda x: str(safe_eval(x)),
    })

