# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyCore/VirtualPath.py,v 1.1 2004/06/28 10:04:10 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""VirtualPath mixin class"""
__version__='$Revision: 1.1 $'[11:-2]



import Globals
from ExtensionClass import Base
from string import join, split
from urllib import quote,unquote, splittype, splithost
from AccessControl import ClassSecurityInfo
from zLOG import LOG, TRACE, DEBUG, BLATHER, INFO, PROBLEM, ERROR, PANIC
from ComputedAttribute import ComputedAttribute
from VirtualPathHelpers import *


EASY_VHM = 'EasyVHM'


class VirtualPathBase(Base):
    """Mixin to give objects access to a virtual path
    tuple, similare to the getPhysicalPath but with
    account for Virtual SiteRoots, e.g. the presens
    of a VirtualRootPhysicalPath attribute in the context.
    """

    security = ClassSecurityInfo()


    def _getPhysicalPath(self):
        return self.getPhysicalPath()

    security.declarePublic('physicalPath')
    physicalPath = ComputedAttribute(_getPhysicalPath, 1)


    def _getVirtualPath(self):
        vhm = getattr(self, EASY_VHM, None)
        p_path = self.getPhysicalPath()
        if vhm:
            return vhm.get_virtual_path(p_path)
        return p_path

    security.declarePublic('virtualPath')
    virtualPath = ComputedAttribute(_getVirtualPath, 1)


    def _getPhysicalURL(self, ending_slash=1):
        return path2url(self.getPhysicalPath(), ending_slash);

    security.declarePublic('physicalURL')
    physicalURL = ComputedAttribute(_getPhysicalURL, 1)


    def _getVirtualURL(self):
        return path2url(self._getVirtualPath(), ending_slash=1)

    security.declarePublic('virtualURL')
    virtualURL = ComputedAttribute(_getVirtualURL, 1)

    def _getPhysicalPathIndex(self):
        return path2url(self.getPhysicalPath(), 0);

    security.declarePublic('physicalPathIndex')
    physicalPathIndex = ComputedAttribute(_getPhysicalPathIndex, 1)

    security.declarePublic('virtualURLXD')
    def _getVirtualURLXD(self):
        """\
        Returns a virtualURL.
        If the domain that is being accessed doesn't match
        the domain that the menu item is in the URL is remapped
        to include the protocol, domain name, and port.

        XD stand for Cross Domain
        """
        vhm = getattr(self, EASY_VHM, None)
        if vhm:
            try:
                item_domain=vhm.get_domain(self.getPhysicalPath())
                if self.getCurrentDomainName()!=item_domain:
                    return vhm.get_site_root_url(self.physicalPath,1) + self.virtualURL
            except:
                pass
        return self.virtualURL
    virtualURLXD = ComputedAttribute(_getVirtualURLXD, 1)

    security.declarePublic('isLocalDomain')
    def isLocalDomain(self):
        """\
        Test if the domain that is being accessed doesn't match
        the domain that the menu item is in.
        """
        vhm = getattr(self, EASY_VHM, None)
        if vhm:
            try:
                item_domain=vhm.get_domain(self.physicalPath)
                return self.getCurrentDomainName()==item_domain
            except:
                pass
        return 1

    #DEPRECATED
    def _getAbsoluteURL(self):
        LOG('VirtualPath', INFO, 'VirtualPath', '_getAbsoluteURL deprecated. Use virtualURL attribute.\n')
        return self._getVirtualURL()

    #DEPRECATED
    security.declarePublic('absoluteURL')
    absoluteURL = ComputedAttribute(_getAbsoluteURL, 1)


    security.declarePublic('getPhysicalParentPath')
    def getPhysicalParentPath(self):
        return self.getPhysicalPath()[:-1]

    security.declarePublic('physicalParentPath')
    physicalParentPath = ComputedAttribute(getPhysicalParentPath, 1)

    security.declarePublic('getPhysicalParentURL')
    def getPhysicalParentURL(self):
        return self.path2url(self.getPhysicalParentPath())

    security.declarePublic('physicalParentURL')
    physicalParentURL = ComputedAttribute(getPhysicalParentURL, 1)

    security.declarePublic('getParentsPath')
    getParentsPath=getPhysicalParentPath

    # Renamed from getVirtualParentPath to _getVirtualParentPath
    security.declarePublic('_getVirtualParentPath')
    def _getVirtualParentPath(self):
        return self._getVirtualPath()[:-1]

    security.declarePublic('virtualParentPath')
    virtualParentPath = ComputedAttribute(_getVirtualParentPath, 1)


    security.declarePublic('getVirtualPathMap')
    def getVirtualPathMap(self):
        aPath = ()
        map = []
        virtualpath = self._getVirtualPath()
        for name in virtualpath:
            qname = quote(name)
            aPath = aPath + (qname,)
            map.append(
                    {
                        'id': qname,
                        'path': aPath,
                        'url': join(aPath,'/'),
                    }
                )
        return map

    security.declarePublic('getPhysicalPathMap')
    def getPhysicalPathMap(self, path=None):
        aPath = ()
        map = []
        if path is None: path=self.getPhysicalPath()
        for name in path:
            qname = quote(name)
            aPath = aPath + (qname,)
            map.append(
                    {
                        'id': qname,
                        'path': aPath,
                        'url': join(aPath,'/'),
                    }
                )
        return map


    #Virtual Site API

    security.declarePublic('getDomainFromPath')
    def getDomainFromPath(self, path=None):
        if path is None: path=self.getPhysicalPath()
        vhm = getattr(self, EASY_VHM, None)
        if vhm:
            return vhm.get_domain(path)
        return ''


    security.declarePublic('getVirtualRootRule')
    def getVirtualRootRule(self, p_path=None):
        """Returns a tuple containing the (Domain,SiteRootURL)"""
        if p_path is None: p_path=self.getPhysicalPath()
        vhm = getattr(self, EASY_VHM, None)
        if vhm:
            return vhm.get_root_rule(p_path)
        return ('','/')


    security.declarePublic('getVirtualRootPath')
    def getVirtualRootPath(self, path=None):
        if path is None: path=self.getPhysicalPath()
        vhm = getattr(self, EASY_VHM, None)
        if vhm:
            return vhm.get_site_root_path(path)
        return ('',)

    security.declarePublic('splitPhysicalPath')
    def splitPhysicalPath(self, p_path=None):
        vhm = getattr(self, EASY_VHM, None)
        if p_path is None: p_path=self.getPhysicalPath()
        if vhm:
            return vhm.split_physical_path(p_path)
        return ('',), p_path

    security.declarePublic('getSiteRoot')
    def getSiteRoot(self):
        return self.unrestrictedTraverse(self.getVirtualRootPath())

    security.declarePublic('getSiteRootURL')
    def getSiteRootURL(self):
        root_path = self.getVirtualRootPath()
        return path2url(root_path, ending_slash=1)

    #Deprecated
    security.declarePublic('getSiteRootPath')
    getSiteRootPath= getVirtualRootPath


    security.declarePublic('isRoot')
    def isRoot(self, p_path=None):
        if p_path is None: p_path=self.getPhysicalPath()
        vhm = getattr(self, EASY_VHM, None)
        if vhm:
            return vhm.is_root(p_path)
        return p_path==('',)

    security.declarePublic('isEasyRoot')
    def isEasyRoot(self):
        if hasattr(self,'_isEasyRoot'):
            return self._isEasyRoot
        else: return 0

    # Virtual to physical convertions:
    # Move (Virtual to physical convertions) implementations to
    # VirtualPathHelpers for unit testing reasons.
    security.declarePublic('virtualPathToPhysicalPath')
    def virtualPathToPhysicalPath(self, path):
        return virtualPathToPhysicalPath(self.getVirtualRootPath(), path)

    security.declarePublic('virtualPathToPhysicalURL')
    def virtualPathToPhysicalURL(self, path):
        return virtualPathToPhysicalURL(self.getVirtualRootPath(), path)
        #return join(mergePaths(self.getVirtualRootPath(), path), '/')

    security.declarePublic('virtualURLToPhysicalURL')
    def virtualURLToPhysicalURL(self, url):
        return virtualURLToPhysicalURL(self.getVirtualRootPath(), path)
        #path = url2path(url)
        #return join(mergePaths(self.getVirtualRootPath(), path), '/')

    security.declarePublic('virtualURLToPhysicalPath')
    def virtualURLToPhysicalPath(self, url):
        return virtualURLToPhysicalPath(self.getVirtualRootPath(), url)
        #path = url2path(url)
        #return mergePaths(self.getVirtualRootPath(), path)


    # physical to virtual conversions:
    security.declarePublic('physicalPathToVirtualPath')
    def physicalPathToVirtualPath(self, path):
        vroot, vpath = self.splitPhysicalPath(path)
        return vpath

    security.declarePublic('physicalPathToVirtualURL')
    def physicalPathToVirtualURL(self, path):
        vroot, vpath = self.splitPhysicalPath(path)
        return self.path2url(vpath)

    security.declarePublic('physicalURLToVirtualPath')
    def physicalURLToVirtualPath(self, url):
        vroot, vpath = self.splitPhysicalPath(self.url2path(url))
        return vpath

    security.declarePublic('physicalURLToVirtualURL')
    def physicalURLToVirtualURL(self, url):
        vroot, vpath = self.splitPhysicalPath(self.url2path(url))
        return self.path2url(vpath)

    security.declarePublic('virtualParents')
    def virtualParents(self):
        pass


    # This methods are for access from DTML, they should be
    # replaced by higher level methods and other solutions.
    # This would clear up the DTML.

    security.declarePublic('mergePaths')
    def mergePaths(self,path1, path2):
        return mergePaths(path1, path2)

    security.declarePublic('path2url')
    def path2url(self, path, ending_slash=1):
        return path2url(path, ending_slash)

    security.declarePublic('path2root_safe_url')
    def path2root_safe_url(self, path):
        LOG('VirtualPath', INFO, 'VirtualPath', 'path2root_safe_url deprecated. Use path2url.\n')
        return path2url(path)

    security.declarePublic('url2path')
    def url2path(self, url):
        return url2path(url)

    security.declarePublic('urls2paths')
    def urls2paths(self, paths):
        return map(lambda x,url2path=url2path: url2path(x),paths)

    security.declarePublic('getIdFromPath')
    def getIdFromPath(self, path):
        return path[-1:][0]

    security.declarePublic('urlencode')
    def urlencode(self, value):
        return quote(value)

    security.declarePublic('urldecode')
    def urldecode(self, value):
        return unquote(value)

    security.declarePublic('isAbove')
    def isAbove(self, path1, path2):
        return isAbove(path1, path2)

    security.declarePublic('isBellow')
    def isBellow(self, path1, path2):
        return isBellow(path1, path2)

    security.declarePublic('isCurrent')
    def isCurrent(self, path1, path2):
        return isCurrent(path1, path2)

    security.declarePublic('isChild')
    def isChild(self, path1, path2):
        return isChild(path1, path2)

    security.declarePublic('isSibling')
    def isSibling(self, path1, path2):
        return isSibling(path1, path2)

    security.declarePublic('isUrl')
    def isUrl(self, path):
        return isUrl(path)

    security.declarePublic('isPath')
    def isPath(self, path):
        return isPath(path)

    security.declarePublic('isPathOrUrl')
    def isPathOrUrl(self, value):
        return isPathOrUrl(value)

    #"Stolen" from CatalogAwareness
    security.declarePublic('url')
    def url(self, ftype=splittype, fhost=splithost):
        """Return a SCRIPT_NAME-based url for an object."""
        if hasattr(self, 'DestinationURL') and \
           callable(self.DestinationURL):
            url='%s/%s' % (self.DestinationURL(), self.id)
        else: url=self.absolute_url()
        type, uri=ftype(url)
        host, uri=fhost(uri)
        script_name=self.REQUEST['SCRIPT_NAME']
        __traceback_info__=(`uri`, `script_name`)
        if script_name:
            uri=filter(None, split(uri, script_name))[0]
        if not uri:
            uri = '/'
        if uri[0] != '/':
            uri = '/' + uri
        return unquote(uri)


    security.declarePublic('root_safe_url')
    def root_safe_url(self, url):
        if url=='': url="/"
        return url

    #TODO: lgg till __url frn EasyCatalogaware, och kanske ta bort url()?


    def getCurrentDomainName(self, default=None):
        try: return split(self.REQUEST['HTTP_HOST'], ':')[0]
        except: pass
        return default

    def getCurrentPort(self):
        try: return int(split(self.REQUEST['HTTP_HOST'], ':')[1])
        except: pass
        return 80

    def getCurrentProtocol(self):
        req=getattr(self,'REQUEST',None)
        if req:
            https=req.get('HTTPS',None)
            if https.lower()=='on': return 'https'
            elif (req.has_key('SERVER_PORT_SECURE') and req.get['SERVER_PORT_SECURE'] == "1"):
                return 'https'
        return 'http'



Globals.InitializeClass(VirtualPathBase)

