
import re
from types import StringType
import string

from xml.dom import Node
#from xml.dom import NotSupportedErr, NamespaceErr
#from xml.dom import implementation
#from xml.dom import EMPTY_NAMESPACE, XMLNS_NAMESPACE
from xml.dom.html import HTML_4_TRANSITIONAL_INLINE
#from xml.xpath import Context, Evaluate, Compile
from xml.dom.ext import StripXml, Print, PrettyPrint
from xml.dom.html.HTMLDocument import HTMLDocument

from Products.ParsedXML.DOM.Traversal import NodeFilter, TreeWalker

from xml.dom.ext.Printer import PrintWalker
from xml.dom.ext import SeekNss


from StringIO import StringIO

from TextPrinter import TextVisitor
from HTMLBodyPrinter import HTMLBodyVisitor



#From objects
N_TYPE     = 0 # node's type
N_NAME     = 1 # node's label (to process xpath)
N_VALUE    = 2 # node's value
N_CHILDS   = 3 # nodes's childs list
N_PARENT   = 4 # node's parent
N_ISSUE    = 5 # node's total issue number
N_XNUM     = 6 # to compute node's xpath
NSIZE = 7 #number of items in a list which represent a node

# NODE TYPES
#NT_SYST = 0 # SYSTEM node (added by parser) /!\ depricated
NT_NODE = 1 # ELEMENT node
NT_ATTN = 2 # ATTRIBUTE NAME node
NT_ATTV = 3 # ATTRIBUTE VALUE node
NT_TEXT = 4 # TEXT node
NT_COMM = 5 # COMMENT node
NT_ROOT = 6 # root node
NODES_TYPES=('NT','NN','AN','AV','T','C', 'R') # for printing


TYPE_NAME = {
        Node.ATTRIBUTE_NODE:                'Attribute',
        Node.CDATA_SECTION_NODE:            'CDATA Section',
        Node.COMMENT_NODE:                  'Comment',
        Node.DOCUMENT_FRAGMENT_NODE:        'Document Fragment',
        Node.DOCUMENT_NODE:                 'Document',
        Node.DOCUMENT_TYPE_NODE:            'DocumentType',
        Node.ELEMENT_NODE:                  'Element',
        Node.ENTITY_NODE:                   'Entity',
        Node.ENTITY_REFERENCE_NODE:         'Entity Reference',
        Node.NOTATION_NODE:                 'Notation',
        Node.PROCESSING_INSTRUCTION_NODE:   'Processing Instruction',
        Node.TEXT_NODE:                     'Text',
   }


UNKNOWN=0
INLINE_NODE=1
TEXT_NODE=2
BLOCK_NODE=3
PRINCIPAL_BLOCK=4
WHITE_SPACE=5

category_labels={
    UNKNOWN:'Unknown',
    INLINE_NODE:'Inline',
    TEXT_NODE:'Text',
    BLOCK_NODE:'Block Box',
    PRINCIPAL_BLOCK:'Principal Block Box',
    WHITE_SPACE:'White Space',

}

#Helpers

def SimpleTreeWalker(node, do_before, do_after, input, output):
    do_before(node, input, output)
    if node.nodeType==Node.ELEMENT_NODE:
        for child in node.childNodes[:]:
            SimpleTreeWalker(child, do_before, do_after, input, output)
    do_after(node, input, output)


#NormalizeCompNode

def normalizeCompNode(node):
    if getattr(node, '__is_comp__', 0):
        return node.node
    return node

def normalizeCompNodes(nodes):
    nnodes=[]
    for node in nodes[:]:
        if getattr(node, '__is_comp__', 0):
            node=node.node
        nnodes.append(node)
    return nnodes



#Inspectors

def getDepth(node):
    node=normalizeCompNode(node)
    count=0
    pnode=node.parentNode
    while pnode:
        count+=1
        pnode=pnode.parentNode
    return count

#HTML Inspectors

def isInline(node):
    node=normalizeCompNode(node)
    if node.nodeType!=Node.ELEMENT_NODE:
        return 1
    if node.nodeName in HTML_4_TRANSITIONAL_INLINE:
        return 1
    return 0

def isBlock(node):
    node=normalizeCompNode(node)
    if node.nodeType!=Node.ELEMENT_NODE:
        return 0
    if node.nodeName not in HTML_4_TRANSITIONAL_INLINE:
        return 1
    return 0

def isPrincipalBlock(node):
    node=normalizeCompNode(node)
    if isBlock(node):
        for child in node.childNodes[:]:
            if isBlock(child):
                return 1
    return 0

def isNonPrincipalBlock(node):
    node=normalizeCompNode(node)
    if isBlock(node):
        for child in node.childNodes[:]:
            if isBlock(child):
                return 0
        return 1
    return 0

def isTextNode(node):
    node=normalizeCompNode(node)
    return node.nodeType==Node.TEXT_NODE

def isWhitespaceOnly(textnode):
    textnode=normalizeCompNode(textnode)
    if textnode.nodeType!=Node.TEXT_NODE:
        return 0
    if len(textnode.nodeValue) == 0:
        return 1
    if textnode.nodeValue.strip():
        return 0
    return 1

def isNodesHomogeneous(node_list):
    node_list=node_list[:]
    if len(node_list):
        parent=node_list[0].parentNode

        for child in node_list[1:]:
            if child.parentNode is not parent:
                return 0
    return 1

def isVersionNode(node):
    node=normalizeCompNode(node)
    return node.nodeType==Node.ELEMENT_NODE and node.tagName.upper() in ('INS','DEL')

#NodeNavigation

def nextNode(node):
    child=node.firstChild
    if child:
        return child
    return node.nextSibling

def previousNode(node):
    previousSibling=node.previousSibling
    if previousSibling:
        lastchild=previousSibling.lastChild
        if lastchild:
            return lastchild
    return node.parentNode

#HTML Renderers

def getNodeTypeText(node):
    return TYPE_NAME[node.nodeType]

def renderAttributes(node):
    node=normalizeCompNode(node)
    attrs=''
    for attribute in node.attributes:
        attrs=attrs + ' %s="%s"' % (attribute.name, attribute.value)
    return attrs

def renderText(node, strip=1):
    if isNonPrincipalBlock(node):
        io=StringIO()
        pw=PrintWalker(TextVisitor(io), node)
        pw.run()
        io.pos=0
        return io.read()
    return ''

def PrintBodyContent(root, encoding='UTF-8'):
    io=StringIO()
    if not hasattr(root, "nodeType"):
        return
    nss = SeekNss(root)
    visitor = HTMLBodyVisitor(io, encoding, nsHints=nss)
    PrintWalker(visitor, root).run()
    io.pos=0
    return io.read()

def doPrint(root, encoding='UTF-8'):
    io=StringIO()
    Print(root,io)
    io.pos=0
    return io.read()

def doPrettyPrint(root, encoding='UTF-8'):
    io=StringIO()
    PrettyPrint(root,io)
    io.pos=0
    return io.read()

#PreProcess and Normalize
def splitTextNodes(dom, splitter=re.compile('(\s*)')):
    walker = TreeWalker(dom.documentElement, NodeFilter.SHOW_ALL, None, 1)
    while 1:
        current_node=walker.currentNode
        current_node.isWhiteSpace=isWhitespaceOnly(current_node)
        if current_node.nodeType==Node.TEXT_NODE and not current_node.isWhiteSpace:
            value=current_node.nodeValue
            values=filter(None, splitter.split(value))
            for value in values:
                new_node=dom.createTextNode(value)
                new_node.isWhiteSpace=isWhitespaceOnly(new_node)
                current_node.parentNode.insertBefore(new_node, current_node)
            current_node.parentNode.removeChild(current_node)
            walker.currentNode=new_node
        next = walker.nextNode()
        if next is None: break
    return dom



def isChildsDeleted(node):
    node=normalizeCompNode(node)

    input={}
    output={'count':0, 'delete_childs':1}

    def do_before(node, input, output):
        output['delete_childs'] = output['delete_childs'] and getattr(node, 'deleteNode', 0)

    def do_after(node, input, output):
        if output['delete_childs']:
            node.deleteTree=1

    SimpleTreeWalker(node, do_before, do_after, input, output)


def isChild(child, parent):
    return child.parentNode is parent


def isHTMLDocument(node):
    return getattr(node, '__class__', None) is HTMLDocument

def insertNode(newnode, insert_point, parent, owner, tagName='INS'):
    newnode=owner.importNode(newnode,1)
    ins=owner.createElement(tagName)
    ins.appendChild(newnode)
    parent.insertBefore(ins, insert_point)

def deleteNode(deleted_object, owner, tagName='DEL'):
    isChildsDeleted(deleted_object)
    newnode=owner.createElement(tagName)
    deleted_object.parentNode.replaceChild(newnode, deleted_object)
    newnode.appendChild(deleted_object)

def ProcessSimple(seq1, seq2, label=''):
    import difflib
    differ=difflib.SequenceMatcher(lambda x: x.node.isWhiteSpace, seq1, seq2)

    owner=seq1[0].node.ownerDocument

    for tag, alo, ahi, blo, bhi in differ.get_opcodes():
        if tag == 'replace':
            insert_point=seq1[alo].node
            parent=insert_point.parentNode
            current=None

            for newline in seq2[blo:bhi]:
               if not current or newline.node.parentNode is not current:
                    insertNode(newline.node, insert_point, parent, owner, 'INS')
                    current=newline.node

            for delline in seq1[alo:ahi]:
                deleted_object=delline.node
                newnode=owner.createElement('DEL')
                deleted_object.parentNode.replaceChild(newnode, deleted_object)
                newnode.appendChild(deleted_object)

        elif tag == 'delete':
            for delline in seq1[alo:ahi]:
                deleted_object=delline.node
                deleteNode(deleted_object, owner)

        elif tag == 'insert':
            insert_point=seq1[alo].node
            parent=insert_point.parentNode
            current=None

            for newline in seq2[blo:bhi]:
                if not current or newline.node.parentNode is not current:
                    insertNode(newline.node, insert_point, parent, owner)
                    current=newline.node


        elif tag == 'equal':
            pass

        else:
            self.fail("Tag '%s' doesn't exist." % tag)


