import unittest

from xml.dom import Node
from xml.dom import implementation
from xml.dom.NodeFilter import NodeFilter

from xml.dom.ext.reader import HtmlLib, Sax2
from xml.dom.ext import StripXml, Print, PrettyPrint

from logilab.xmldiff.input import tree_from_stream, tree_from_dom
from logilab.xmldiff.objects import N_ISSUE, N_CHILDS
from logilab.xmldiff.format import XUpdatePrinter, InternalPrinter
from logilab.xmldiff.ezs import EzsCorrector
from logilab.xmldiff.fmes import FmesCorrector

#input import
from logilab.xmldiff.parser import SaxHandler
from xml.sax import make_parser, SAXNotRecognizedException
from xml.sax.handler import feature_namespaces
from xml.sax.handler import feature_external_ges, feature_external_pes
from xml.sax.handler import property_lexical_handler
from xml.dom.ext.c14n import Canonicalize


import CompNode
import DOMUtils
import difflib
import HTMLDiffer
import AdvancedHTMLDiffer

from logilab.xmldiff.format import InternalPrinter, XUpdatePrinter, DOMXUpdateFormatter

from StringIO import StringIO

B_HTML="<HTML><BODY>"
E_HTML="</BODY></HTML>"

OFF=1
ON=0


class TestXMLDIFF(unittest.TestCase):


    def testHTMLDiffer1(self):

        HTML1="""
        <p>Old</p>
        """
        HTML2="""
        <p>New</p>
        """
        RESULT="""<HTML><BODY>
        <INS><P>New</P></INS><DEL><P>Old</P></DEL>
        </BODY></HTML>"""

        differ=HTMLDiffer.HTMLDiffer(HTML1, HTML2)
        differ.processDiff()
        result=differ.Print()
        self.assertEquals(result, RESULT)


    def testHTMLDiffer2(self):

        NEW="""
        <P class=header1 dir=ltr style="MARGIN-RIGHT: 0px" align=left>438 Energy+ Cold Appliances</p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG>Quelle</strong> officially joined the Energy+ suppliers. The number of models available from 13 manufacturers has doubled again for the Spring 2003 Energy+ appliances List Update.<A title="Energy+ News" href="news" target="">Changed</a></p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG><A title="The European Energy + Awards" href="awards" target="">Join the European Energy+ Awards + competition NOW!</a></strong></p>
        <p>New line</p>
        """
        OLD="""
        <P class=header1 dir=ltr style="MARGIN-RIGHT: 0px" align=left>438 Energy+ Cold Appliances</p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG>Quelle</strong> officially joined the Energy+ suppliers. The number of models available from 13 manufacturers has doubled again for the Spring 2003 Energy+ appliances List Update.<A title="Energy+ News" href="news" target="">Read more</a></p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG><A title="The European Energy + Awards" href="awards" target="">Join the European Energy+ Awards + competition NOW!</a></strong></p>
        """
        RESULT="""<HTML><BODY>
        <P class='header1' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'>438 Energy+ Cold Appliances</P>
        <INS><P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'><STRONG>Quelle</STRONG> officially joined the Energy+ suppliers. The number of models available from 13 manufacturers has doubled again for the Spring 2003 Energy+ appliances List Update.<A target title='Energy+ News' href='news'>Changed</A></P></INS><DEL><P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'><STRONG>Quelle</STRONG> officially joined the Energy+ suppliers. The number of models available from 13 manufacturers has doubled again for the Spring 2003 Energy+ appliances List Update.<A target title='Energy+ News' href='news'>Read more</A></P></DEL>
        <P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'><STRONG><A target title='The European Energy + Awards' href='awards'>Join the European Energy+ Awards + competition NOW!</A></STRONG></P>
        <INS><P>New line</P></INS></BODY></HTML>"""

        differ=HTMLDiffer.HTMLDiffer(OLD, NEW)
        differ.processDiff()
        result=differ.Print()
        self.assertEquals(result, RESULT)


    def testHTMLDiffer3(self):

        NEW="""
        <P class=header1 dir=ltr style="MARGIN-RIGHT: 0px" align=left>438 Energy+ Cold Appliances</p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG>Quelle</strong> officially joined the Energy+ suppliers. The number of models available from 13 manufacturers has doubled again for the Spring 2003 Energy+ appliances List Update.<A title="Energy+ News" href="news" target="">Changed</a></p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG><A title="The European Energy + Awards" href="awards" target="">Join the European Energy+ Awards + competition NOW!</a></strong></p>
        <UL><LI>dsjds</LI></UL>
        <p>New line</p>
        """
        OLD="""
        <P class=header1 dir=ltr style="MARGIN-RIGHT: 0px" align=left>438 Energy+ Cold Appliances</p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG>Quelle</strong> officially joined the Energy+ suppliers. The number of models available from 13 manufacturers has doubled again for the Spring 2003 Energy+ appliances List Update.<A title="Energy+ News" href="news" target="">Read more</a></p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG><A title="The European Energy + Awards" href="awards" target="">Join the European Energy+ Awards + competition NOW!</a></strong></p>
        <UL><LI>dsjds</LI></UL>
        """
        RESULT="""<HTML><BODY>
        <P class='header1' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'>438 Energy+ Cold Appliances</P>
        <INS><P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'><STRONG>Quelle</STRONG> officially joined the Energy+ suppliers. The number of models available from 13 manufacturers has doubled again for the Spring 2003 Energy+ appliances List Update.<A target title='Energy+ News' href='news'>Changed</A></P></INS><DEL><P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'><STRONG>Quelle</STRONG> officially joined the Energy+ suppliers. The number of models available from 13 manufacturers has doubled again for the Spring 2003 Energy+ appliances List Update.<A target title='Energy+ News' href='news'>Read more</A></P></DEL>
        <P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'><STRONG><A target title='The European Energy + Awards' href='awards'>Join the European Energy+ Awards + competition NOW!</A></STRONG></P>
        <UL><LI>dsjds</LI></UL>
        <INS><P>New line</P></INS></BODY></HTML>"""

        differ=HTMLDiffer.HTMLDiffer(OLD, NEW)
        differ.processDiff()
        result=differ.Print()
        self.assertEquals(result, RESULT)


    def testHTMLDiffer4(self):

        NEW="""
        <P class=header1 dir=ltr style="MARGIN-RIGHT: 0px" align=left>600 <IMG title=aplus height=12 alt=aplus hspace=0 src="aplus" width=20 border=0> and <IMG title=Aplusplus height=12 alt=Aplusplus hspace=0 src="Aplusplus" width=20 border=0> Cold Appliances</p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left>Energy+ has officially adopted the new EC cold appliances directive.<STRONG> Arcelik</strong>,<STRONG> EWZ</strong>,<STRONG> Foron</strong>,<STRONG> Gorenje</strong>,<STRONG> </strong><STRONG> Migros</strong>,<STRONG> Neckermann</strong>,<STRONG> </strong>and  <STRONG>Otto</strong> officially joined Energy+. The number of models available from all 19 manufacturers has increased again for the Autumn 2003 Energy+ Appliances List Update.<A title="Energy+ News" href="news" target="">Read more</a></p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG><A title="The European Energy + Awards" href="awards" target="">Join the European Energy+ Awards competition NOW!</a></strong></p>
        """
        OLD="""
        <P class=header1 dir=ltr style="MARGIN-RIGHT: 0px" align=left>438 Energy+ Cold Appliances</p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG>Quelle</strong> officially joined the Energy+ suppliers. The number of models available from 13 manufacturers has doubled again for the Spring 2003 Energy+ appliances List Update.<A title="Energy+ News" href="news" target="">Read more</a></p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG><A title="The European Energy + Awards" href="awards" target="">Join the European Energy+ Awards + competition NOW!</a></strong></p>
        """
        RESULT="""<HTML><BODY>
        <INS><P class='header1' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'>600 <IMG hspace='0' height='12' src='aplus' border='0' alt='aplus' title='aplus' width='20'> and <IMG hspace='0' height='12' src='Aplusplus' border='0' alt='Aplusplus' title='Aplusplus' width='20'> Cold Appliances</P></INS><INS><P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'>Energy+ has officially adopted the new EC cold appliances directive.<STRONG> Arcelik</STRONG>,<STRONG> EWZ</STRONG>,<STRONG> Foron</STRONG>,<STRONG> Gorenje</STRONG>,<STRONG> </STRONG><STRONG> Migros</STRONG>,<STRONG> Neckermann</STRONG>,<STRONG> </STRONG>and  <STRONG>Otto</STRONG> officially joined Energy+. The number of models available from all 19 manufacturers has increased again for the Autumn 2003 Energy+ Appliances List Update.<A target title='Energy+ News' href='news'>Read more</A></P></INS><INS><P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'><STRONG><A target title='The European Energy + Awards' href='awards'>Join the European Energy+ Awards competition NOW!</A></STRONG></P></INS><DEL><P class='header1' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'>438 Energy+ Cold Appliances</P></DEL>
        <DEL><P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'><STRONG>Quelle</STRONG> officially joined the Energy+ suppliers. The number of models available from 13 manufacturers has doubled again for the Spring 2003 Energy+ appliances List Update.<A target title='Energy+ News' href='news'>Read more</A></P></DEL>
        <DEL><P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'><STRONG><A target title='The European Energy + Awards' href='awards'>Join the European Energy+ Awards + competition NOW!</A></STRONG></P></DEL>
        </BODY></HTML>"""


        differ=HTMLDiffer.HTMLDiffer(OLD, NEW)
        differ.processDiff()
        result=differ.Print()
        self.assertEquals(result, RESULT)


    def testHTMLDiffer5(self):

        NEW="""
        <P class=header1 dir=ltr style="MARGIN-RIGHT: 0px" align=left>600 <IMG title=aplus height=12 alt=aplus hspace=0 src="aplus" width=20 border=0> and <IMG title=Aplusplus height=12 alt=Aplusplus hspace=0 src="Aplusplus" width=20 border=0> Cold Appliances</p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left>Energy+ has officially adopted the new EC cold appliances directive.<STRONG> Arcelik</strong>,<STRONG> EWZ</strong>,<STRONG> Foron</strong>,<STRONG> Gorenje</strong>,<STRONG> </strong><STRONG> Migros</strong>,<STRONG> Neckermann</strong>,<STRONG> </strong>and  <STRONG>Otto</strong> officially joined Energy+. The number of models available from all 19 manufacturers has increased again for the Autumn 2003 Energy+ Appliances List Update.<A title="Energy+ News" href="news" target="">Read more</a></p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG><A title="The European Energy + Awards" href="awards" target="">Join the European Energy+ Awards competition NOW!</a></strong></p>
        """
        OLD="""
        """
        RESULT="""<HTML><BODY>
        <INS><P class='header1' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'>600 <IMG hspace='0' height='12' src='aplus' border='0' alt='aplus' title='aplus' width='20'> and <IMG hspace='0' height='12' src='Aplusplus' border='0' alt='Aplusplus' title='Aplusplus' width='20'> Cold Appliances</P></INS><INS><P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'>Energy+ has officially adopted the new EC cold appliances directive.<STRONG> Arcelik</STRONG>,<STRONG> EWZ</STRONG>,<STRONG> Foron</STRONG>,<STRONG> Gorenje</STRONG>,<STRONG> </STRONG><STRONG> Migros</STRONG>,<STRONG> Neckermann</STRONG>,<STRONG> </STRONG>and  <STRONG>Otto</STRONG> officially joined Energy+. The number of models available from all 19 manufacturers has increased again for the Autumn 2003 Energy+ Appliances List Update.<A target title='Energy+ News' href='news'>Read more</A></P></INS><INS><P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'><STRONG><A target title='The European Energy + Awards' href='awards'>Join the European Energy+ Awards competition NOW!</A></STRONG></P></INS></BODY></HTML>"""

        differ=HTMLDiffer.HTMLDiffer(OLD, NEW)
        differ.processDiff()
        result=differ.Print()
        self.assertEquals(result, RESULT)


    def testHTMLDiffer6(self):

        NEW="""
        <P class=header1 dir=ltr style="MARGIN-RIGHT: 0px" align=left>600 <IMG title=aplus height=12 alt=aplus hspace=0 src="aplus" width=20 border=0> and <IMG title=Aplusplus height=12 alt=Aplusplus hspace=0 src="Aplusplus" width=20 border=0> Cold Appliances</p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left>Energy+ has officially adopted the new EC cold appliances directive.<STRONG> Arcelik</strong>,<STRONG> EWZ</strong>,<STRONG> Foron</strong>,<STRONG> Gorenje</strong>,<STRONG> </strong><STRONG> Migros</strong>,<STRONG> Neckermann</strong>,<STRONG> </strong>and  <STRONG>Otto</strong> officially joined Energy+. The number of models available from all 19 manufacturers has increased again for the Autumn 2003 Energy+ Appliances List Update.<A title="Energy+ News" href="news" target="">Read more</a></p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG><A title="The European Energy + Awards" href="awards" target="">Join the European Energy+ Awards competition NOW!</a></strong></p>
        """
        OLD="""
        <P class=header1 dir=ltr style="MARGIN-RIGHT: 0px" align=left>438 Energy+ Cold Appliances</p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG>Quelle</strong> officially joined the Energy+ suppliers. The number of models available from 13 manufacturers has doubled again for the Spring 2003 Energy+ appliances List Update.<A title="Energy+ News" href="news" target="">Read more</a></p>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG><A title="The European Energy + Awards" href="awards" target="">Join the European Energy+ Awards + competition NOW!</a></strong></p>
        """
        RESULT="""<HTML><BODY>
        <P class='header1' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'>438 Energy+ Cold Appliances</P>
        <P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'><STRONG>Quelle</STRONG> officially joined the Energy+ suppliers. The number of models available from 13 manufacturers has doubled again for the Spring 2003 Energy+ appliances List Update.<A target title='Energy+ News' href='news'>Read more</A></P>
        <P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'><STRONG><A target title='The European Energy + Awards' href='awards'>Join the European Energy+ Awards + competition NOW!</A></STRONG></P>
        </BODY></HTML>"""

        differ=AdvancedHTMLDiffer.AdvancedHTMLDiffer(OLD, NEW)
        result=differ.Print()
        self.assertEquals(result, RESULT)

    def testHTMLDiffer7(self):

        OLD="""
        <p>Line number 1</p>
        <p>Line number 2</p>
        <p>Line number 3</p>
        <p>Line number 4</p>
        <p>Line number 5</p>
        <p>Line number 6</p>
        <p>Line number 7</p>
        <UL>
        <LI>List row 1</LI>
        <LI>List row 2</LI>
        </UL>
        <p>Line number 8</p>
        <p>Line number 9</p>
        <UL>
        <LI>List 9 row 1</LI>
        <LI>List 9 row 2</LI>
        </UL>
        <p>Line number 10</p>
        """

        NEW="""
        <p>Line number 1</p>
        <p>Line number 2</p>
        <p>This is new 1</p>
        <p>Line number 3</p>
        <UL>
        <LI>List row 1</LI>
        <LI>List row 1 </LI>
        <LI>List row 2</LI>
        </UL>
        <p>Line number 4</p>
        <p>Line number 5</p>
        <p>Line number 6</p>
        <UL>
        <LI>List row 1</LI>
        <LI>List row 3</LI>
        <LI>List row 4</LI>
        </UL>
        <p>Line number 8</p>
        <p>Line number 9</p>
        <p>Line number 10</p>
        """
        RESULT="""<HTML><BODY>
        <P>Line number 1</P>
        <P>Line number 2</P>
        <INS><P>This is new 1</P></INS><P>Line number 3</P>
        <INS><UL>
        <LI>List row 1</LI>
        <LI>List row 1 &frac12;</LI>
        <LI>List row 2</LI>
        </UL></INS><P>Line number 4</P>
        <P>Line number 5</P>
        <P>Line number 6</P>
        <DEL><P>Line number 7</P></DEL>
        <UL>
        <LI>List row 1</LI>
        <INS><LI>List row 3</LI></INS><INS><LI>List row 4</LI></INS><DEL><LI>List row 2</LI></DEL>
        </UL>
        <P>Line number 8</P>
        <P>Line number 9</P>
        <DEL><UL>
        <DEL><LI>List 9 row 1</LI></DEL>
        <DEL><LI>List 9 row 2</LI></DEL>
        </UL></DEL>
        <P>Line number 10</P>
        </BODY></HTML>"""

        differ=AdvancedHTMLDiffer.AdvancedHTMLDiffer(OLD, NEW)
        differ.processDiff()
        result1=differ.Print()
        self.assertEquals(result1, RESULT)

        differ=HTMLDiffer.HTMLDiffer(OLD, NEW)
        differ.processDiff()
        result2=differ.Print()
        self.assertEquals(result2, RESULT)

        self.assertEquals(result1, result2)


    def testEnergyPlus1(self):

        ENGLISH="""
        <P class=header1 dir=ltr style="MARGIN-RIGHT: 0px" align=left>600&nbsp;<IMG title=aplus height=12 alt=aplus hspace=0 src="aplus" width=20 border=0> and&nbsp;<IMG title=Aplusplus height=12 alt=Aplusplus hspace=0 src="Aplusplus" width=20 border=0> Cold Appliances</P>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left>Energy+ has officially adopted the new EC cold appliances directive.<STRONG> Arcelik</STRONG>,<STRONG>&nbsp;EWZ</STRONG>,<STRONG> Foron</STRONG>,<STRONG> Gorenje</STRONG>,<STRONG>&nbsp;</STRONG><STRONG> Migros</STRONG>,<STRONG> Neckermann</STRONG>,<STRONG>&nbsp;</STRONG>and&nbsp; <STRONG>Otto</STRONG> officially joined Energy+. The number of models available from all 19 manufacturers has increased again for the Autumn 2003 Energy+ Appliances List Update.<A title="Energy+ News" href="news" target="">Read more</A></P>
        <P class=normal dir=ltr style="MARGIN-RIGHT: 0px" align=left><STRONG><A title="The European Energy + Awards" href="awards" target="">Join the European Energy+ Awards competition NOW!</A></STRONG></P>
        """

        FRENCH="""
        """
        RESULT="""<HTML><BODY>
        <DEL><P class='header1' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'>600&nbsp;<IMG hspace='0' height='12' src='aplus' border='0' alt='aplus' title='aplus' width='20'> and&nbsp;<IMG hspace='0' height='12' src='Aplusplus' border='0' alt='Aplusplus' title='Aplusplus' width='20'> Cold Appliances</P></DEL>
        <DEL><P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'>Energy+ has officially adopted the new EC cold appliances directive.<STRONG> Arcelik</STRONG>,<STRONG>&nbsp;EWZ</STRONG>,<STRONG> Foron</STRONG>,<STRONG> Gorenje</STRONG>,<STRONG>&nbsp;</STRONG><STRONG> Migros</STRONG>,<STRONG> Neckermann</STRONG>,<STRONG>&nbsp;</STRONG>and&nbsp; <STRONG>Otto</STRONG> officially joined Energy+. The number of models available from all 19 manufacturers has increased again for the Autumn 2003 Energy+ Appliances List Update.<A target title='Energy+ News' href='news'>Read more</A></P></DEL>
        <DEL><P class='normal' align='left' dir='ltr' style='MARGIN-RIGHT: 0px'><STRONG><A target title='The European Energy + Awards' href='awards'>Join the European Energy+ Awards competition NOW!</A></STRONG></P></DEL>
        </BODY></HTML>"""


        differ=AdvancedHTMLDiffer.AdvancedHTMLDiffer(ENGLISH, FRENCH)
        differ.processDiff()
        result1=differ.Print()
        self.assertEquals(result1, RESULT)

        differ=HTMLDiffer.HTMLDiffer(ENGLISH, FRENCH)
        differ.processDiff()
        result2=differ.Print()
        self.assertEquals(result2, RESULT)
        self.assertEquals(result1, result2)

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestXMLDIFF, 'test'))
    return suite
