# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/EasyDocument.py,v 1.2 2004/06/29 14:14:15 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__="""EasyDocument"""
__version__='$Revision: 1.2 $'[11:-2]

import sys, Globals, DateTime
from Acquisition import aq_base, aq_chain
from Globals import DTMLFile
from string import join
from AccessControl import ClassSecurityInfo
from Products.EasyCore import EasyCorePermissions
from Products.EasyCore.Utils import Tutils
from urllib import quote
from zLOG import LOG, INFO, BLATHER, ERROR
from copy import deepcopy

from Products.EasyCore.UpgradeSupport.Utils import upgradeProperties
from Products.EasyCore.EasyCorePermissions import *
from Products.EasyCore.EasyEditorConf import EASYEDITORID, DEFAULTTEMPLATEID,DEFAULTVIEWNAME, DEFAULTVIEWATTR, ACQUIRETEMPLATE, ACQUIREVIEW
from Products.EasyEditor.EasyEditable import EasyEditable
from Products.EasyCore.ManageOptionBase import ManageOptionBase
from Products.EasyNavigation.EasyNavigatable import Navigatable
from Products.EasyWorkflow.EasyPublishable import Publishable
from Products.EasyWorkflow.EasyWorkflowBase import NullDate
from Products.EasyWorkflow.HistoryTracker import HistoryTracker
from Products.EasyWorkflow.EasyWorkflowBase import wfDRAFT
from Products.EasyDublinCore.EasyDublinCore import EasyDublinCore
from Products.EasyCore.EasyNavigationConf import NAVIGATION_CATALOG_ID

from Products.EasyCore.BodyTabs import InitializeBodyTabs
from Products.EasyCore import MLOG_TAB

class EasyDocument(ManageOptionBase, Navigatable, EasyEditable,  Publishable):
    """A EasyDocument Object """

    meta_type = 'EasyDocument'
    __catalogs__=(NAVIGATION_CATALOG_ID,)

    security = ClassSecurityInfo()

    use_manage_options = EasyEditable.use_manage_options + \
                        Publishable.use_manage_options + \
                        Navigatable.use_manage_options

    _auto_add_to_navigation = 1

    _isEasyRoot = 0
    _workflowable=1

    skey_default='id'

    _properties = Navigatable._properties + EasyDublinCore._properties+({'id':'skey_default', 'label':'Default Object Sorting', 'type':'selection', 'mode':'wd','select_variable':'possible_obj_sort_columns'},)

    _view_defaults = {
             DEFAULTVIEWATTR: ('default', 'Default', 0),
             '_printview': ('printview', 'Print View', 1)
             }



    def __init__(self
                    , id
                    , template=None
                    , title=''
                    , navtitle=''
                    , description=''
                    , language=''
                    , category=()
                    , subject=()
                    , effective_date=NullDate
                    , expiration_date=NullDate
                    , format='text/html'
                    , add_to_navigation=1
                ):
        EasyEditable.__init__(self,id, title, template)
        Navigatable.__init__(self, title, navtitle)
        Publishable.__init__(self)
        EasyDublinCore.__init__(self
                                , title=title
                                , description=description
                                , language=language
                                , category=category
                                , subject=subject
                                , effective_date=effective_date
                                , expiration_date=expiration_date
                                , format=format
                                )
        self._auto_add_to_navigation = add_to_navigation
        self.language = language
        self.description = description
        if category is not None:
            self.category=category
        if subject is not None:
            self.subject=subject

    # webDAV support

    def addNewDocument(self , id
                            , template = None
                            , title=''
                            , navtitle=''
                            , description=''
                            , language=''
                            , category=()
                            , subject=()
                            , effective_date=NullDate
                            , expiration_date=NullDate
                            , format='text/html'
                            , add_to_navigation=None
                        ):
        if add_to_navigation is None:
            add_to_navigation=self._auto_add_to_navigation
        if template is None:
            template=self._template
        try:
            ob=EasyDocument( id
                            , template=template
                            , title=title
                            , navtitle=navtitle
                            , description=description
                            , language=language
                            , category=category
                            , subject=subject
                            , effective_date=effective_date
                            , expiration_date=expiration_date
                            , format=format
                            , add_to_navigation = add_to_navigation
                            )
            ob._isEasyRoot = 0
            self._setObject(id, ob)
            ob = self._getOb(id)
            ob.initcontent()
        except:
            import sys
            exc = sys.exc_info()
            LOG('EasyDocument', ERROR, "Can't create sub document. %s (%s)" % exc[:2], error=exc)

    def cloneSubobjects(self, clone, inv_spec=('EasySlaveCopy','EasyMasterCopy')):
        filtered_ids=[]
        for id, obj in self.objectItems():
            if not getattr(aq_base(obj), 'isAnObjectManager', 0) and obj.meta_type not in inv_spec:
                filtered_ids.append(id)
        cp=self.manage_copyObjects(filtered_ids)
        clone.manage_pasteObjects(cp)

    security.declarePrivate('cloneTo')
    def cloneTo( self, container ):
        """\
        Make a copy of this objekt, including any sub objekt
        not being an ObjectManager.
        """
        excluded_ids=['notification_list','creation_date','EasySlaveCopy','EasyMasterCopy',
                      'publish_history', '_workflow_history', 'created_by', 'approved_by',
                      'effective_date', 'effective_date', 'effective_date_set_by',
                      'expiration_date_set_by', '_template'
                      '_master_data_copy', '_objects', 'last_modified_by']
        self_id = self.getId()
        excluded_ids.extend(self.objectIds())
        clone=EasyDocument( id = self_id
                            , template = self._template
                            , title=self.title
                            , navtitle=self.navigation_title
                            , description=self.description
                            , language=self.language
                            , category=self.category
                            , subject=self.subject
                            , effective_date=self.effective_date
                            , expiration_date=self.expiration_date
                            , format=self.format
                            , add_to_navigation=self._auto_add_to_navigation)
        for id, obj in self.__dict__.items():
            if id not in excluded_ids:
                if hasattr(aq_base(obj), 'inheritedAttribute'):
                    clone.__dict__[id] = aq_base(obj)
                else:
                    clone.__dict__[id] = deepcopy(aq_base(obj))
        container._setObject(self_id, clone)
        clone = container._getOb(self_id)
        self.cloneSubobjects(clone)



    #EVENT TRIGGERS
    #Added this to EasyContactForm, EasyNewsITem, EasyFolder,
    #EasyDocument (this) and EasyEditable to enable overlaying
    #manage_afterAdd, manage_beforeDelete, manage_afterClone
    #For instance EasyObjectLink needs to overlay this
    #With overlay I meen that a mix-in class needs to implement these methods
    #without necessasarilly being a direct super class.
    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        Navigatable.manage_afterAdd(self, item, container)
        EasyEditable.manage_afterAdd(self, item, container)

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        Navigatable.manage_beforeDelete(self, item, container)
        EasyEditable.manage_beforeDelete(self, item, container)

    security.declarePrivate('manage_afterClone')
    def manage_afterClone(self, item):
        Navigatable.manage_afterClone(self, item)
        EasyEditable.manage_afterClone(self, item)



    #Skinnable
    #NEW#__of__=EasyEditable.__of__
    #NEW#__getattr__=EasyEditable.__getattr__
    #NEW#_checkId=EasyEditable._checkId
    #NEW#valid_property_id=EasyEditable.valid_property_id

    #NEW#security.declareProtected('View', 'testskins')
    #NEW#def testskins(self, REQUEST=None, changed=0):
    #NEW#    """ """
    #NEW#    res=[]
    #NEW#    write=res.append

    #NEW#    write("ptest1:");write(str(getattr(self, 'ptest1', 'No ptest1')))
    #NEW#    write("ptest2:");write(str(getattr(self, 'ptest2', 'No ptest2')))
    #NEW#    write("ptest3:");write(str(getattr(self, 'ptest3', 'No ptest3')))
    #NEW#    write("ptest4:");write(str(getattr(self, 'ptest4', 'No ptest4')))
    #NEW#    write("ptest5:");write(str(getattr(self, 'ptest5', 'No ptest5')))
    #NEW#    write("ptest6:");write(str(getattr(self, 'ptest6', 'No ptest6')))
    #NEW#    write("ptest7:");write(str(getattr(self, 'ptest7', 'No ptest7')))
    #NEW#    write("ptest8:");write(str(getattr(self, 'ptest8', 'No ptest8')))
    #NEW#    write("ptest9:");write(str(getattr(self, 'ptest9', 'No ptest9')))
    #NEW#    write("only_in_root:");write(str(getattr(self, 'only_in_root', 'No only_in_root')))

    #NEW#    write("EDITOR::self.defaultviews()");write(str(self.defaultviews()))
    #NEW#    write("EDITOR::self.getDefaultTemplateId()");write(str(self.getDefaultTemplateId()))
    #NEW#    write("EDITOR::self.featureIds()");write(str(self.featureIds()))
    #NEW#    write("EDITOR::self.storageIds()");write(str(self.storageIds()))

    #NEW#    write("TEMPLATE::self.getDefaultStyle()");write(str(self.getDefaultStyle()))
    #NEW#    write("TEMPLATE::self.styleIds()");write(str(self.styleIds()))
    #NEW#    write("TEMPLATE::self.getRegionStorageIds()");write(str(self.getRegionStorageIds()))
    #NEW#    write("TEMPLATE::self.getRegionIds()");write(str(self.getRegionIds()))

    #NEW#    write("TAL::self.try_me()");write(str(self.try_me()))


    #NEW#    return "<body><h1>Test</h1><p>%s</p></body>" % join(res, '<br>')


    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        #if getattr(aq_base(self),'_object_version', None)!='1.7': self._object_version = '1.7'
        if getattr(aq_base(self),'_object_version', None)!='1.8': self._object_version = '1.8' ; changed=1
        if not hasattr(aq_base(self),'_object_links'):
            self._object_links = {} ; changed=1
        upgradeProperties(self, {'skey_default':'position',}, ())
        if REQUEST is not None:
            return "<h1>Done</h1><p>Changed: %s</p>" % str(changed)
        return changed


Globals.InitializeClass(EasyDocument)
InitializeBodyTabs(EasyDocument)

addEasyDocumentForm=DTMLFile('dtml/manage_addEasyDocumentForm', globals())
addEasyDocumentWithTemplateForm=DTMLFile('dtml/addEasyDocumentWithTemplateForm', globals())

def addEasyDocument(self
                    , id
                    , template=None
                    , title=''
                    , navtitle=''
                    , description=''
                    , language=''
                    , category=()
                    , subject=()
                    , effective_date=NullDate
                    , expiration_date=NullDate
                    , format='text/html'
                    , add_to_navigation=1
                    , views=[]
                    , REQUEST=None
                    ):
    """Add EasyDocument"""
    id=Tutils.xlate2url(id)
    # BUG
    # Something isn't right here, for instance if a EasyEditor
    # has a none default viewname the created EasyDocument
    # would try to render DEFAULTVIEWNAME instead of EasyDocument.default_view
    if template=='**default':
        template = DEFAULTTEMPLATEID
        ob=EasyDocument(  id
                        , template=template
                        , title=title
                        , navtitle=navtitle
                        , description=description
                        , language=language
                        , category=category
                        , subject=subject
                        , effective_date=effective_date
                        , expiration_date=expiration_date
                        , format=format
                        , add_to_navigation = 0
                        )
        ob._isEasyRoot = 1
        self._setObject(id, ob)
        ob = self._getOb(id) # get object in context
        if views:
            ob.setViews(views)
        ob.initViews()
    else:
        if template=='__ACQUIRE' or template is None:
            template = ACQUIRETEMPLATE
        ob=EasyDocument(id
                        , template=template
                        , title=title
                        , navtitle=navtitle
                        , description=description
                        , language=language
                        , category=category
                        , subject=subject
                        , effective_date=effective_date
                        , expiration_date=expiration_date
                        , format=format
                        , add_to_navigation = add_to_navigation
                        )
        self._setObject(id, ob)
        ob = self._getOb(id) # get object in context
        if views:
            ob.setViews(views)
        ob.initcontent()
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        REQUEST.RESPONSE.redirect(u+'/manage_main')



def initialize(context):
    """Initialize EasyDocument Product"""

    context.registerClass (
            EasyDocument,
            permission=EasyCorePermissions.EasyDocumentAdd,
            icon='icons/EasyDocument.gif',
            constructors=(addEasyDocumentForm,
                          addEasyDocument, addEasyDocumentWithTemplateForm),
            meta_type = EasyDocument.meta_type,
            category='Easy Publisher',
            )

    context.registerViewDefault(
            EasyDocument._view_defaults,
            meta_type = EasyDocument.meta_type
            )

    context.registerEasyType(EasyDocument)
