# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/EasyEditable.py,v 1.6 2004/07/19 11:55:05 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyEditable Object"""
__version__='$Revision: 1.6 $'[11:-2]



import Globals
import Products
from Persistence import Persistent
from Globals import DTMLFile, MessageDialog
from AccessControl import getSecurityManager, Unauthorized
from AccessControl import ClassSecurityInfo
from Products.EasyCore.EasyCorePermissions import *
from OFS.Folder import Folder
import DateTime
from Products.EasyEditor.Extensions.striphtml import striphtml
from webdav.NullResource import NullResource
from string import atoi

from Products.EasyCore.Utils.HTMLFormHelpers import selectTemplateDTML
from Products.EasyEditor.EasyTemplates.EasyTemplates import EasyTemplate
from Products.EasyCore.EasyEditorConf import EASYEDITORID, DEFAULTTEMPLATEID, DEFAULTVIEWNAME, DEFAULTVIEWATTR, ACQUIRETEMPLATE, ACQUIREVIEW
from Products.EasyEditor.EasyEditorRegistry import ViewDefaults, ViewDefaultsByMetaType
from Products.EasyCore.ManageOptionBase import ManageOptionBase
from Products.EasyCatalog.EasyCatalogAware import EasyCatalogAwareObjectAndPropertyManager, EasyCatalogAware
from Products.EasyCore.VirtualPath import VirtualPathBase
from Products.EasyEditor.EasyDialogSupport import DialogSupport
from Products.EasyWorkflow.EasyWorkflowBase import wfDRAFT, wfSUBMITTED, wfDENIED, wfPUBLISHED, NullDate
from Products.EasyEditor.BrowserRegistry import BrowserRegistry
from Products.EasyEditor.EasyPublisher import EasyPublisherBase
from Products.EasyEditor.EasyWebDAVSupport import EasyWebDAVSupport
from Products.EasyCore.Utils import Tutils



#DUBLIN CORE
from Products.EasyDublinCore.EasyDublinCore import EasyDublinCore

from zLOG import LOG, ERROR
from sys import exc_info
from Acquisition import aq_self, aq_base, aq_inner, aq_parent, aq_chain

#from Products.EasyCore.EasyManagmentTabs import EasyManagmentTabs, createManageIndex
from Products.EasyWorkflow.EasySync import EasySyncAware

#NEW#from Products.EasyCore.EasySkinnable import EasySkinnable

from Products.EasyCore import MLOG_TAB



_marker=[]


class EasyEditable(ManageOptionBase,
                    EasySyncAware, EasyCatalogAwareObjectAndPropertyManager,
                    EasyCatalogAware, EasyDublinCore, DialogSupport,
                    EasyWebDAVSupport, VirtualPathBase,
                    Folder,
                    #NEW#EasySkinnable,
                    EasyPublisherBase):
    """\

    EasyEditable gives support for classes that should support Templates
    and Views and should be editable through the EasyEditor editor interface.
    """
    security = ClassSecurityInfo()

    _isEasyRoot = 0
    _v_temp_old_name=None
    _v_temp_new_name=None

    __catalogs__=()

    manage_options_left = Folder.manage_options[:1]
    manage_options_right = Folder.manage_options[1:]

    def allow_edit(self):
        state=self.getWorkflowState()
        now = DateTime.now()
        if self.expiration_date <= now and self.expiration_date != NullDate and \
            not getSecurityManager().checkPermission(EasyWorkflowEditArchived, self):
            #If this is archive you also need the EasyWorkflowEditArchived permission
            #in addition to the regulare permissions.
            return 0
        if state==wfDRAFT:
            return getSecurityManager().checkPermission(EasyWorkflowEditDraft, self)
        elif state==wfSUBMITTED:
            return getSecurityManager().checkPermission(EasyWorkflowEditSubmitted, self)
        elif state==wfPUBLISHED:
            return getSecurityManager().checkPermission(EasyWorkflowEditPublished, self)
        elif state==wfDENIED:
            return getSecurityManager().checkPermission(EasyWorkflowEditDraft, self)
        return 0



    security.declareProtected(ViewManagementScreens, 'manage_options_editable')
    def manage_options_editable(self):
        options = () #+ Folder.manage_options[:1]
        allow_edit=self.allow_edit()
        if hasattr(self,EASYEDITORID):
            if BrowserRegistry.isFunctionSupportedFor('DHTMLEditionComponent5.5', self.REQUEST.HTTP_USER_AGENT) and allow_edit:
                options = options + ({'label':'Editor', 'action':'manage_editor'},)
            if allow_edit:
                options = options + ({'label':'HTML', 'action':'manage_html'},)
            if getSecurityManager().checkPermission(EasyTemplateViewChange, self) and allow_edit:
                options = options + ({'label':'Template', 'action':'manage_template'},)
        if self.isSyncronized and allow_edit:
            options = options + self.easysyncaware_options
        return options #+ Folder.manage_options[1:]

    use_manage_options = ((manage_options_left, manage_options_editable, manage_options_right),)


    def filtered_meta_types(self,user=None):
        user=getSecurityManager().getUser()
        if hasattr(self, 'EasyEditor'):
            template=self.EasyEditor.getTemplate(self.getTemplateId())
            easyunrestricted=user.has_permission(EasyUnrestricted,self)
            if template and (template.show_template_types or template.show_meta_types):
                gmt=[]
                if template.show_template_types and template.use_easy_class:
                    entry = None
                    for each in Products.meta_types:
                        if each['name'] == template.use_easy_class:
                            entry = each.copy()
                            break
                    if entry:
                        action = entry['action']
                        for each in template.getTemplateTypes():
                            entry['action'] = 'manage_addProduct/EasyEditor/AddByTemplate-%s-%s' % (each.id, template.use_easy_class)
                            entry['name'] = each.title
                            entry['template'] = 1
                            gmt.append(entry.copy())

                if template.show_meta_types:
                    meta_types = template.getMetaTypes()
                    user=getSecurityManager().getUser()
                    if callable(self.all_meta_types):
                        all=self.all_meta_types()
                    else:
                        all=self.all_meta_types
                    for meta_type in all:
                        if easyunrestricted or meta_type['name'] in meta_types:
                            if meta_type.has_key('permission'):
                                if user.has_permission(meta_type['permission'],self):
                                    gmt.append(meta_type)
                            else:
                                gmt.append(meta_type)

                return gmt

        # Return a list of the types for which the user has
        # adequate permission to add that type of object.

        meta_types=[]
        if callable(self.all_meta_types):
            all=self.all_meta_types()
        else:
            all=self.all_meta_types
        for meta_type in all:
            if meta_type.has_key('permission'):
                if user.has_permission(meta_type['permission'],self):
                    meta_types.append(meta_type)
            else:
                meta_types.append(meta_type)
        return meta_types


    # default attributes that holds view ids.
    _view_defaults = {
                 DEFAULTVIEWATTR: ('default', 'Default', 0)
                 }


    def __init__(self, id, title, template):
        EasyDublinCore.__init__(self, title=title)
        EasySyncAware.__init__(self)
        self.id = id
        self.title = title
        self._template = template


    security.declarePrivate('initcontent')
    def initcontent(self):
        template=self.EasyEditor.getTemplate(self.getTemplateId())
        pre_init=getattr(template, 'pre_init', None)
        if callable(pre_init):  pre_init(self, template)
        self.initViews()
        regions=template.regionValues()
        for region in regions:
            region.initcontent(self)
        post_init=getattr(template, 'post_init', None)
        if callable(pre_init):  post_init(self, template)


    security.declarePrivate('initViews')
    def initViews(self):
        try:
            getdefaultview = self.EasyEditor.getDefaultView
        except:
            getdefaultview = None
        for key, value in self._view_defaults.items():
            if not hasattr(self, key):
                if getdefaultview:
                    setattr(self, key, getdefaultview(key, value))
                else:
                    setattr(self, key, value[0])



    _size_changes={
        'Bigger': (5,5),
        'Smaller': (-5,-5),
        'Narrower': (0,-5),
        'Wider': (0,5),
        'Taller': (5,0),
        'Shorter': (-5,0),
        }

    def _er(self,SUBMIT,dtpref_cols,dtpref_rows,REQUEST):
        dr,dc = self._size_changes[SUBMIT]
        rows=max(1,atoi(dtpref_rows)+dr)
        cols=max(40,atoi(dtpref_cols)+dc)
        e=(DateTime.DateTime('GMT') + 365).rfc822()
        REQUEST.RESPONSE.setCookie('dtpref_rows',str(rows),path='/',expires=e)
        REQUEST.RESPONSE.setCookie('dtpref_cols',str(cols),path='/',expires=e)
        return cols, rows


    def getIcon(self):
        try:
            editor = aq_base(getattr(self,EASYEDITORID)).__of__(self)
            templateid = self.getTemplateId()
            template = editor.getTemplate(templateid)
            if hasattr(template, 'custom_icon'):
                return template.custom_icon.absolute_url()
            return self.icon
        except:
            return self.icon


    #Gets the View, Template and Editor

    security.declareProtected(AccessContentsInformation, 'getTemplateService')
    def getTemplateService(self):
        REQUEST = getattr(self, 'REQUEST', None)
        if REQUEST and REQUEST.has_key('editor'): return REQUEST['editor']
        return getattr(self,EASYEDITORID)

    security.declareProtected(AccessContentsInformation, 'getTemplate')
    def getTemplate(self):
        REQUEST = getattr(self, 'REQUEST', None)
        if REQUEST and REQUEST.has_key('template'): return REQUEST['template']
        service = self.getTemplateService()
        return service.getTemplate(self.getTemplateId())


    security.declareProtected(AccessContentsInformation, 'getView')
    def getView(self):
        REQUEST = getattr(self, 'REQUEST', None)
        if REQUEST and REQUEST.has_key('view'): return REQUEST['view']
        template=self.getTemplate()
        if REQUEST is not None: viewname = REQUEST.get('epview', None)
        if viewname is None or not template.hasView(viewname):
            viewname = self.getViewName()
        return template.getView(viewname)


    #
    #
    security.declareProtected(EasyDocumentChange, 'setViewName')
    def setViewName(self, view, viewattr = DEFAULTVIEWATTR):
        setattr(self, viewattr, view)

    security.declarePrivate('_getViewName')
    def _getViewName(self, viewattr = DEFAULTVIEWATTR):
        this = aq_base(self)
        if hasattr(this, viewattr):
            return getattr(this, viewattr)
        if hasattr(self,'EasyEditor'):
            ed = self.EasyEditor
            if ed.hasDefaultViewFor(viewattr):
                viewname = ed.getDefaultView(viewattr)
                if viewname: return viewname
                return DEFAULTVIEWNAME
        if self._view_defaults.has_key(viewattr):
            return self._view_defaults.get(viewattr)
        return DEFAULTVIEWNAME

    security.declareProtected(AccessContentsInformation, 'getViewName')
    def getViewName(self, viewattr = None, get_aq_mark=None):
        if viewattr is None:
            try:
                viewdefaults = ViewDefaultsByMetaType.get(self.meta_type, None)
                viewattr = viewdefaults[0][0]
            except:
                viewattr = DEFAULTVIEWATTR
        view = self._getViewName(viewattr)
        pre = ""
        if view == ACQUIREVIEW:
            try:
                pre = "**"
                view = self.aq_parent.getViewName()
            except:
                pre = "*"
                view = DEFAULTVIEWNAME
        if get_aq_mark: return pre, view
        else: return view



    security.declareProtected(AccessContentsInformation, 'getDefaultViewNames')
    def getDefaultViewNames(self):
        return ViewDefaults

    security.declareProtected(AccessContentsInformation, 'getViewNames')
    def getViewNames(self):
        order_view_defaults = self._view_defaults.items()
        order_view_defaults.sort(lambda x, y: cmp(x[1][2],y[1][2]))
        return order_view_defaults

    security.declareProtected(AccessContentsInformation, 'getTemplateId')
    def getTemplateId(self):
        try:
            template = self._template
        except:
            try:
                template = self._template = self.EasyEditor.getDefaultTemplateId()
            except:
                template = self._template = DEFAULTTEMPLATEID
        if template == ACQUIRETEMPLATE:
            try:
                template = self.aq_parent.getDefaultTemplateId()
            except:
                try:
                    template = self.EasyEditor.getDefaultTemplateId()
                except:
                    template = DEFAULTTEMPLATEID
        return template

    security.declareProtected(AccessContentsInformation, 'isTemplateIdAcquired')
    def isTemplateIdAcquired(self):
        return self._template == ACQUIRETEMPLATE

    security.declareProtected(AccessContentsInformation, 'istemplateidAcquired')
    istemplateidAcquired=isTemplateIdAcquired


    security.declareProtected(AccessContentsInformation, 'isViewNameAcquired')
    def isViewNameAcquired(self, viewattr=DEFAULTVIEWATTR):
        return hasattr(self,viewattr) and getattr(self,viewattr) == ACQUIREVIEW

    security.declareProtected(AccessContentsInformation, 'isviewnameAcquired')
    isviewnameAcquired=isViewNameAcquired

    # This method should be made dynamic in regard to differen editable regions defined in
    # the Template. Maybe by calling a manage_save method on the template?
    security.declareProtected(EasyDocumentChange, 'manage_edit')
    def manage_edit(self, client=None, REQUEST=None
                    , title=_marker
                    , SUBMIT=None
                    , regionnames=None
                    , dtpref_cols='50'
                    , dtpref_rows='20'
                    ):
        """Save method for Content Regions
        Can only save value of content regions if:
          1. A REQUEST dictionary exists
          2. There exists a EasyEditor instance in the acquisition context.

        The names of the content regions to save are in the regionnames argument.
        The REQUEST dictionary should contain each name in the regionnames in
        order to be saved.
        """
        if self._size_changes.has_key(SUBMIT):
            rows, cols = self._er(SUBMIT,dtpref_cols,dtpref_rows,REQUEST)
            return self.manage_html(
                dtpref_rows=rows, dtpref_cols=cols,
                manage_tabs_message='Textarea sizes change to (%s,%s)'%(rows, cols),
                management_view="HTML",client=client, REQUEST=REQUEST)

        if title is not _marker: self.title = title
        unabletosave = None
        if REQUEST is None: REQUEST = self.REQUEST
        if REQUEST is not None and regionnames is not None:
            if hasattr(self,EASYEDITORID):
                editor = aq_base(getattr(self,EASYEDITORID)).__of__(self)
            else:
                return MessageDialog(
                    title  ="Can't find Editor",
                    message= "Can't find an Editor named: %s" % (EASYEDITORID),
                    action ='manage_workspace')
            unabletosave = []
            templateid = self.getTemplateId()
            template = editor.getTemplate(templateid)
            contentregions= template.regionItems()
            for id, region in contentregions:
                if id in regionnames and REQUEST.has_key(id):
                    try:
                        region.getStorage().manage_save(self, region, REQUEST[id])
                    except:
                        unabletosave.append(id)
                        LOG("StorageError", ERROR, "Unable to save to storage (%s) in manage_edit (page: %s) in EasyEditable.py"%(id,self.getId()))

        self._updateContributers()
        self.index_object()
        if REQUEST is not None:
            if unabletosave == None:
                manage_tabs_message = "Only the properties was saved."
            elif unabletosave == []:
                manage_tabs_message = "The document has been saved."
            else:
                manage_tabs_message = "The document has been saved, except for: "
                for id in unabletosave:
                    manage_tabs_message = manage_tabs_message + id + ", "
                manage_tabs_message = manage_tabs_message[:-2]
            return self.manage_html(
                manage_tabs_message=manage_tabs_message,
                management_view="HTML",client=client, REQUEST=REQUEST)


    # This method doesn't handle multiple editable regions.
    security.declareProtected(EasyDocumentChange, 'manage_html_form')
    manage_html_form = DTMLFile('dtml/manage_html_EasyEditable',globals())

    security.declareProtected(EasyDocumentChange, 'manage_html')
    def manage_html(self, manage_tabs_message="", management_view=None
                    , dtpref_cols='50', dtpref_rows='20'
                    , client=None, REQUEST=None):
        """Gets the template and the content regions
        """
        if not self.allow_edit():
            #message=self.getWorkflowStateMessage()
            state_name=self.getWorkflowStateName()
            return MessageDialog(
                title  = "Unsufficent privaledge.",
                message= "Unsufficent privaledge<br>Not allowed to edit documents in the %s state." % state_name,
                action ='manage_workspace')
        if REQUEST is None: REQUEST = self.REQUEST
        if hasattr(self,EASYEDITORID):
            editor = aq_base(getattr(self,EASYEDITORID)).__of__(self)
        else:
            return MessageDialog(
                title  ="Can't find Editor",
                message= "Can't find an Editor named: %s" % (EASYEDITORID),
                action ='manage_workspace')
        templateid = self.getTemplateId()
        try:
            template = editor.getTemplate(templateid)
        except:
            manage_tabs_message="Can find the template [%s]. You need to select a different template before you can edit the HTML of this document." % templateid
            return  self.manage_template(client,REQUEST,
                manage_tabs_message=manage_tabs_message,
                management_view='Template')

        contentregions= template.regionItems()
        return  self.manage_html_form(client,REQUEST,template=template,
                contentregions=contentregions, manage_tabs_message=manage_tabs_message,
                management_view='HTML')


    # We need to be able to pass a viewname to the Editor!
    security.declareProtected(EasyDocumentChange, 'manage_editor')
    def manage_editor(self, action=None, editable_region_map=[], meta_data_map=[], management_url='', management_view='', manage_tabs_message='', document_url='', client=None, REQUEST=None):
        """Call the Editor to edit the document"""
        if not self.allow_edit():
            #message=self.getWorkflowStateMessage()
            state_name=self.getWorkflowStateName()
            return MessageDialog(
                title  = "Unsufficent privaledge.",
                message= "Unsufficent privaledge<br>Not allowed to edit documents in the %s state." % state_name,
                action ='manage_workspace')
        if REQUEST is None: REQUEST = self.REQUEST
        if hasattr(self,EASYEDITORID) and BrowserRegistry.isFunctionSupportedFor('DHTMLEditionComponent5.5', self.REQUEST.HTTP_USER_AGENT):
            #This is more simple that in manage_editable, index_html etc.
            editor = getattr(self,EASYEDITORID)
        else:
            if BrowserRegistry.isFunctionSupportedFor('DHTMLEditionComponent5.5', self.REQUEST.HTTP_USER_AGENT):
                reason="There is not an EasyEditor installed in this Site"
            else:
                reason="Your browser [%s] is not supported by EasyEditor." % self.REQUEST.HTTP_USER_AGENT
            return MessageDialog(
                title  ="Your not able to use EasyEditor",
                message= "Your not able to use EasyEditor<br>%s" % (reason),
                action ='manage_workspace')
        if action=="_save":
            template = editor.getTemplate(self.getTemplateId())
            template.manage_save(editable_region_map, meta_data_map, self, REQUEST)
            self._updateContributers()
            self.index_object()
        if not REQUEST.has_key('management_page_charset'):
            REQUEST.set('management_page_charset', getattr(self,'management_page_charset', 'UTF-8'))
        if REQUEST is not None:
            if management_url=="index_html":
                return self.index_html(self,REQUEST)
            elif management_url=='manage_html':
                return self.manage_html(manage_tabs_message="The HTML has been updated.", management_view='HTML', client=client, REQUEST=REQUEST)
            elif management_url and management_url!="manage_editor":
                management_method = getattr(self, management_url, None)
                if management_method:
                    return management_method(self,self,REQUEST,manage_tabs_message=manage_tabs_message,management_view=management_view)
            templateid = self.getTemplateId()
            return editor.manage_editor(document=self, templateid=templateid, document_url=document_url, REQUEST=REQUEST)

    security.declareProtected(EasyDocumentChange, 'manage_editable')
    def manage_editable(self, client=None, REQUEST=None, viewname=None):
        """Render document for editor"""
        if REQUEST is None: REQUEST = self.REQUEST
        editor = getattr(self,EASYEDITORID)
        editor = aq_base(editor).__of__(self)
        template = editor.getTemplate(self.getTemplateId())
        if REQUEST is not None and REQUEST.has_key('epview'):
            viewname = REQUEST.get('epview')
        if viewname is None or not template.hasView(viewname):
            view = template.getEditView() # change to getEditViewId?

            if callable(view.id):
                viewname = view.id()
            else:
                viewname = view.id
        else:
            view =  template.getView(viewname)
        browserfolder = editor.getBrowserFolderByHUA(REQUEST.HTTP_USER_AGENT)
        REQUEST.set('browserfolder',browserfolder)
        REQUEST.set('viewid',viewname)
        REQUEST.set('templateid',template.id)
        REQUEST.set('view',view)
        REQUEST.set('viewmode','edit')
        REQUEST.set('here',self)
        REQUEST.set('template',template)
        REQUEST.set('editor',editor)
        REQUEST.set('SITEROOTURL',editor.getSiteRootURL())
        if view:
            return view(client=self,REQUEST=REQUEST)
        else:
            raise "No default view", "This document doesn't have a default view associated to it."

    security.declarePrivate('manage_load')
    def manage_load(self, name, REQUEST=None, renderDTML=1, default=None, **kw):
        if REQUEST is None: REQUEST = self.REQUEST
        if REQUEST is not None and REQUEST.has_key('template'):
            value = REQUEST['template'].manage_load(name, self, default, REQUEST)
            if type(value)==type(''): return value
            if hasattr(value,'meta_type') and value.meta_type=='DTML Method':
                if renderDTML and not (REQUEST.has_key('viewmode') and REQUEST['viewmode']=='edit'):
                    return apply(value, (self,REQUEST), kw)
                else:
                    return value.document_src()
        return "Can't find Template."

    security.declareProtected(View, 'index_html')
    def index_html(self, client=None, REQUEST=None, viewname=None):
        """View Document"""
        if REQUEST is None: REQUEST = self.REQUEST
        now = DateTime.now()
        if (not getSecurityManager().checkPermission(EasyViewAhead, self) and (self.getWorkflowState() != wfPUBLISHED or (self.effective_date != NullDate and self.effective_date > now))):
            raise Unauthorized, "This document has not been published yet."
        if (not getSecurityManager().checkPermission(EasyViewArchived, self) and \
            self.expiration_date <= now and self.expiration_date != NullDate) :
            raise Unauthorized, "This document has been archived."
        editor = getattr(self,EASYEDITORID)
        editor = aq_inner(editor).__of__(self)
        template = editor.getTemplate(self.getTemplateId())

        #Test init tracker, or should it be done explicitlly from the templates?
        template.initTracking()

        browserfolder = editor.getBrowserFolderByHUA(REQUEST.HTTP_USER_AGENT)

        if REQUEST is not None and REQUEST.has_key('epview'):
            viewname = REQUEST.get('epview')
        if viewname is None or not template.hasView(viewname):
            viewname = self.getViewName()
        view =  template.getView(viewname)

        REQUEST.set('browserfolder',browserfolder)
        REQUEST.set('viewid',viewname)
        REQUEST.set('templateid',template.id)
        REQUEST.set('viewmode','view')
        REQUEST.set('view',view)
        REQUEST.set('here',self)
        REQUEST.set('template',template)
        REQUEST.set('editor',editor)
        REQUEST.set('SITEROOTURL',editor.getSiteRootURL())

        if view:
            return view(client=self,REQUEST=REQUEST)
        else:
            raise "No default view", "This document doesn't have a default view associated to it."

    __call__ = index_html

    def __render_with_namespace__(self, namespace):
        # This is supposed to do something else, I just don't know what.
        # Using this solves problem with dtml-with which calls the
        # object and there for calls index_html (which is unappropriate
        # when using dtml-with.
        # Maybe we should try to wrap our selfs in the namespace???
        return self

    # PrincipiaSearchSource Support

    security.declarePublic('PrincipiaSearchSource')
    def PrincipiaSearchSource(self):
        result = ""
        try:
            editor = aq_base(getattr(self,EASYEDITORID)).__of__(self)
            template = editor.getTemplate(self.getTemplateId())
            if template:
                for region in template.regionValues(isSearchable=1):
                    reg = region.manage_load(self, default="")
                    if type(reg) == type(''):
                        txt = reg
                    else:
                        if hasattr(reg,'meta_type') and reg.meta_type=='DTML Method':
                            txt = reg.document_src()
                        else:
                            txt = str(reg)
                    result = result + " " + striphtml(txt)
        except:pass
        return striphtml(result)

    security.declareProtected(EasyTemplateViewChange, 'manage_template')
    manage_template = DTMLFile('dtml/manage_template_EasyEditable',globals())

    security.declareProtected(EasyTemplateViewChange, 'manage_edit_template')
    def manage_edit_template(self, new_template_id, views=[],REQUEST=None):
        """\
        Change Template and Views
        """
        if not self.allow_edit():
            #message=self.getWorkflowStateMessage()
            state_name=self.getWorkflowStateName()
            return MessageDialog(
                title  = "Unsufficent privaledge.",
                message= "Unsufficent privaledge<br>Not allowed to edit documents in the %s state." % state_name,
                action ='manage_workspace')
        if hasattr(self,EASYEDITORID):
            editor = aq_base(getattr(self,EASYEDITORID)).__of__(self)
        else:
            return self.manage_template(client=self,REQUEST=REQUEST
                , manage_tabs_message='manage_template'
                , management_view="Can't find an Editor named: %s" % (EASYEDITORID)
                )

        self.setViews(views)

        self._template = new_template_id

        templateid = self.getTemplateId()
        template = editor.getTemplate(templateid)

        pre_init=getattr(template, 'pre_init', None)
        if callable(pre_init):  pre_init(self, template)

        contentregions= template.regionItems()

        #After Add
        for id, region in contentregions:
            region.manage_initilized(self)

        post_init=getattr(template, 'post_init', None)
        if callable(post_init):  post_init(self, template)

        #self._updateContributers()
        if REQUEST is not None:
            return self.manage_template(self,REQUEST,manage_tabs_message="",management_view='Template')

    security.declarePrivate('setViews')
    def setViews(self,views):
        for record in views:
            setattr(self, record.viewattr, record.viewname)


    security.declarePrivate('snippet_template_SelectTemplate')
    def snippet_template_SelectTemplate(self,REQUEST=None):
        if hasattr(self,EASYEDITORID):
            editor = getattr(self,EASYEDITORID)
            templates= editor.templateMap()
            templateid = self.getTemplateId()
            if templateid <> '' and self.istemplateidAcquired():
                acquire_text = 'Acquired [%s]' % (templateid,)
                activekeys = [ACQUIRETEMPLATE]
            else:
                acquire_text = 'Acquired'
                activekeys = [templateid]
            #templates = [(ACQUIRETEMPLATE, acquire_text)] + templates
            return selectTemplateDTML(self,REQUEST=REQUEST, items = templates, name='new_template_id', activekeys = activekeys)

        return "Can't find an Editor named: %s" % (EASYEDITORID,)

    security.declarePrivate('snippet_template_SelectDefaultView')
    def snippet_template_SelectDefaultView(self, viewattr = DEFAULTVIEWATTR, REQUEST=None):
        if hasattr(self,EASYEDITORID):
            editor = getattr(self,EASYEDITORID)
            templateid = self.getTemplateId()
            try:
                template = editor.getTemplate(templateid)
            except:
                #It should probably return some kind of default view snippet?
                return ""
            if template:
                views = template.viewMap()
                pre, viewname = self.getViewName(viewattr, 1)
                activekeys = [viewname]
                if pre=='**': views = [(ACQUIREVIEW,"Acquired [%s]" % viewname)] + views
                elif pre=='*': views = [(ACQUIREVIEW,"Default [%s]" % viewname)] + views
                #else: views = [(ACQUIREVIEW, 'Acquired')] + views
                hidden = """<input type="hidden" name="views.viewattr:records" value="%s">""" % (viewattr,)
                return hidden + selectTemplateDTML(self,REQUEST=REQUEST, items = views, name="views.viewname:records", activekeys = activekeys)
        #It should probably return some kind of default view snippet?
        return ""

    def getDocument(self):
        return self

    def getDefaultFolderName(self, name, REQUEST=None, default=_marker):
        return self.meta_type

    def addNewDocument(self , id
                            , template = None
                            , title=''
                            , navtitle=''
                            , description=''
                            , language=''
                            , category=()
                            , subject=()
                            , effective_date=NullDate
                            , expiration_date=NullDate
                            , format='text/html'
                            , add_to_navigation=None
                            , REQUEST=None,RESPONSE=None
                        ):
        raise "Not implemented"

    def MKCOL_handler(self,id,REQUEST=None,RESPONSE=None):
        """
            Handle WebDAV MKCOL.
        """
        self.addNewDocument(id, REQUEST=REQUEST,RESPONSE=RESPONSE)

    def PUT_factory( self, name, typ, body ):
        """
            Dispatcher for PUT requests to non-existent IDs.  Returns
            an object of the appropriate type (or None, if we don't
            know what to do).
        """
        try:
            if name and name.endswith('.pt'):
                from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
                ob = ZopePageTemplate(name, body, content_type=typ)
            elif typ in ('text/html', 'text/xml', 'text/plain'):
                from OFS.DTMLMethod import DTMLMethod
                #__init__(self, source_string='', mapping=None, __name__='<string>', **vars):
                ob = DTMLMethod( '', __name__=name )
            elif typ[:6]=='image/':
                from Products.EasyImage.EasyImage import EasyImage
                ob=EasyImage(name, body, '', content_type=typ)
            else:
                from Products.EasyFile.EasyFile import EasyFile
                ob = EasyFile(name, body, '', content_type=typ)
        except:
            import sys, traceback
            raise exc_type, exc_value
        return ob

    # __bobo_traverse__ extracts a content version id by traversing
    # the special name epver_*. The wildcard part equals the name
    # of the content version.
    # The version name can be accessed from the REQUEST from the
    # templates, regions and strorage features to decide which
    # information to return to the user.
    #
    # __bobo_traverse__ looks up the requested attributes/object name by first
    # checking the current object and its acquisition context. If that fails it
    # continues with looking for a view of the current Template in the EasyEditor
    # instance.
    #
    #def __bobo_traverse__(self, REQUEST, name):
    #
    #Has been replaced by:
    #

    ##Skinnable
    #NEW#__of__=EasySkinnable.__of__
    #NEW#__getattr__=EasySkinnable.__getattr__
    #NEW#_checkId=EasySkinnable._checkId
    #NEW#valid_property_id=EasySkinnable.valid_property_id

    ####REPLACES def __bobo_traverse__(self, REQUEST, name):
    #NEW#def __getitem__(self, name):
    #NEW#    #If this is an SubObject handled by the object manager, return it.
    #NEW#    request=self.REQUEST #Obsolete?
    #NEW#    ##request.set('templateid', self.getTemplateId())
    #NEW#    v=self._getOb(name, None)
    #NEW#    if v is not None: return v

    #NEW#    #Check if this is a WebDAV request
    #NEW#    method=request.get('REQUEST_METHOD', 'GET')
    #NEW#    if request.maybe_webdav_client and not method in ('GET', 'POST'):
    #NEW#        return NullResource(self, name, request).__of__(self)
    #NEW#    if name[0:6]=="epver_":
    #NEW#        request['epver'] = name[6:]
    #NEW#        return self
    #NEW#    if name[0:7]=="epview_":
    #NEW#        request['epview'] = name[7:]
    #NEW#        return self

    #NEW#    #for viewattr in self._view_defaults.keys():
    #NEW#    #    if hasattr(self, viewattr) and getattr(self, viewattr)==name:
    #NEW#    #        request['epview'] = name
    #NEW#    #        if hasattr(self, name):
    #NEW#    #            return getattr(self, name)
    #NEW#    #        return self.index_html
    #NEW#    if hasattr(self, name):
    #NEW#        return getattr(self, name)
    #NEW#    raise KeyError, name

    #OLD#
    def __bobo_traverse__(self, REQUEST, name):
        #If this is an SubObject handled by the object manager, return it.
        v=self._getOb(name, None)
        if v is not None: return v
        request=self.REQUEST #Obsolete?
        #Check if this is a WebDAV request
        method=request.get('REQUEST_METHOD', 'GET')
        if request.maybe_webdav_client and not method in ('GET', 'POST'):
            return NullResource(self, name, request).__of__(self)
        if name[0:6]=="epver_":
            REQUEST['epver'] = name[6:]
            return self
        if name[0:7]=="epview_":
            REQUEST['epview'] = name[7:]
            return self

        for viewattr in self._view_defaults.keys():
            if hasattr(self, viewattr) and getattr(self, viewattr)==name:
                REQUEST['epview'] = name
                if hasattr(self, name):
                    return getattr(self, name)
                return self.index_html
        if hasattr(self, name):
            return getattr(self, name)
        editor = getattr(self,EASYEDITORID, None)
        if editor is not None:
            templateid = self.getTemplateId()
            template = editor.getTemplate(templateid)
            #Init Tracker???
            template.initTracking()
            if template and template.hasView(name):
                view =  template.getView(name)
                browserfolder = editor.getBrowserFolderByHUA(REQUEST.HTTP_USER_AGENT)
                REQUEST.set('browserfolder',browserfolder)
                REQUEST.set('viewid',name)
                REQUEST.set('view',view)
                REQUEST.set('templateid',templateid)
                REQUEST.set('viewmode','view')
                REQUEST.set('here', self)
                REQUEST.set('template', template)
                REQUEST.set('editor', editor)
                REQUEST.set('SITEROOTURL',editor.getSiteRootURL())
                return view

    #Event Triggers
    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        EasySyncAware.manage_afterAdd(self, item, container)
        ##DEBUG 1.7.6: EasyCatalogAware.manage_afterAdd(self, item, container)
        ##EasyCatalogAware.manage_afterAdd(self, item, container)

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        EasySyncAware.manage_beforeDelete(self, item, container)
        ##DEBUG 1.7.6: EasyCatalogAware.manage_beforeDelete(self, item, container)
        ##EasyCatalogAware.manage_beforeDelete(self, item, container)

    security.declarePrivate('manage_afterClone')
    def manage_afterClone(self, item):
        EasySyncAware.manage_afterClone(self, item)
        #DEBUG 1.7.6: EasyCatalogAware.manage_afterClone(self, item)
        ##EasyCatalogAware.manage_afterClone(self, item)


    security.declarePublic('modification_date')
    def modification_date(self):
        if hasattr(self,EASYEDITORID):
            editor = aq_base(getattr(self,EASYEDITORID)).__of__(self)
        else:
            return MessageDialog(
                title  ="Can't find Editor",
                message= "Can't find an Editor named: %s" % (EASYEDITORID),
                action ='manage_workspace')
        unabletosave = []
        templateid = self.getTemplateId()
        template = editor.getTemplate(templateid)
        contentregions= template.regionItems()
        last_changed=self.bobobase_modification_time()
        for id, region in contentregions:
            try:
                new_last_changed=region.getStorage().manage_modification_date(id, self)
                if new_last_changed<last_changed: last_changed=new_last_changed
            except:
                pass
        return last_changed



    security.declarePrivate('_updateContributers')
    def _updateContributers(self):
        current_user = getSecurityManager().getUser().getId()
        author = self.author
        self.last_modified_by = current_user
        if author != current_user:
            contributors = self.Contributors()
            if current_user not in contributors:
                contributors = contributors + (current_user,)
            self.setContributors(contributors)



    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        ob_ver=tr(aq_base(self),'_object_version', 1.7)
        try: ob_ver=float(ob_ver)
        except: ob_ver=1.7
        if ob_ver<1.7: self._object_version = '1.8' ; changed=1
        if not hasattr(aq_base(self),'_object_links'):
            self._object_links = {} ; changed=1
        if REQUEST is not None:
            return "<h1>Done</h1><p>Changed: %s</p>" % str(changed)
        return changed


    security.declareProtected('Easy Upgrade Manage', 'upgrade_views')
    def upgrade_views(self, REQUEST=None, changed=0):
        """ """
        #self._view_defaults = {
        #         DEFAULTVIEWATTR: ('default', 'Default', 0)
        #         }
        #
        #try:
        #    getdefaultview = self.EasyEditor.getDefaultView
        #except:
        #    getdefaultview = None
        #for key, value in self._view_defaults.items():
        #    if not hasattr(self, key):
        #        if getdefaultview:
        #            setattr(self, key, getdefaultview(key, value))
        #        else:
        #            setattr(self, key, value[0])
        #

        if REQUEST is not None:
            return """\
                <h1>OK</h1>
                <p>self._view_defaults: %s</p>

                """ % (str(self._view_defaults),)
        return changed


Globals.InitializeClass(EasyEditable)

def initialize(context):
    """Initialize EasyEditable Product"""
    pass







