# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/EasyEditorRegistry.py,v 1.2 2004/06/29 14:14:15 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyEditor Registry"""
__version__='$Revision: 1.2 $'[11:-2]

# Thanks to Ty and Phillip and the PlugIn product from which the following code is borrowed.

# Global PlugIn class registry
# key: kind name; value: lists of meta-type dicts for classes of kind

from App.ProductContext import ProductContext
import Products
from Products.EasyCore.EasyTypeRegistry import EasyTypeRegistry, EasyMetaTypeInformation

EasyStorage = {}
ViewDefaults = {}
ViewDefaultsByMetaType = {}
EasyTemplates = {}
EasyTemplateHelpers = {}

class _ProductContextPatch:


    def registerEasyStorage(self, instance_class=None, storage_type=''):
        """
        Register a plug-in class.  The class registered will not be included in the
        """

        # create and add a singleton
        if instance_class and storage_type:
            storage = instance_class(storage_type)
            EasyStorage[storage_type] = storage
        else:
            LOG('EasyEditor', WARNING, 'registerEasyStorage', "Invalid Instance Class for storage '%s'\n" % (storage_type or "No Name",))


    def registerViewDefault(self, view_defaults, meta_type):
        for viewattr, viewentry in view_defaults.items():
            if ViewDefaults.has_key(viewattr):
                entry = ViewDefaults[viewattr]
                meta_types = entry[2]
                if meta_type in meta_types:
                    from zLOG import LOG, WARNING
                    LOG('EasyEditor', WARNING, 'registerViewDefault', "'%s' has alread the meta_type '%s' registered.\nThis might be a potential conflict.\n" % (viewattr,meta_type))
                else:
                    meta_types = meta_types + (meta_type,)
                    ViewDefaults[viewattr] = ( entry[0], entry[1], meta_types, entry[2])
            else:
                ViewDefaults[viewattr] = (viewentry[0], viewentry[1], (meta_type,), viewentry[2])

            newviews = list(ViewDefaultsByMetaType.get(meta_type, []))
            newviews.append((viewattr, viewentry[0],viewentry[1],viewentry[2]))
            newviews.sort(lambda x,y: cmp(x[3],y[3]))
            ViewDefaultsByMetaType[meta_type] = tuple(newviews)

    def registerEasyTemplateClass(self, instance_class=None, meta_type='',
                              permission=None, constructors=(),
                              icon=None, permissions=None, legacy=(),
                              category='Easy Templates',):
        """
        """

        result = self.registerClass(instance_class,meta_type,
            permission,constructors,icon,permissions,legacy,category=category)

        meta = Products.meta_types[-1]
        Products.meta_types = Products.meta_types[:-1]

        if len(constructors)>1:
            method = constructors[1]
            if type(method) is type(()):
                name,method = method
            else: name = method.__name__
            meta['template_add_method'] = name

        kind = instance_class.meta_type

        l = EasyTemplates.get(kind, [])
        l.append(meta)
        EasyTemplates[kind] = l

        return result

    def registerEasyTemplateHelperClass(self, instance_class=None, meta_type='',
                              permission=None, constructors=(),
                              icon=None, permissions=None, legacy=(),
                              category='Easy Templates',
                              ):
        """
        """

        result = self.registerClass(instance_class,meta_type,
            permission,constructors,icon,permissions,legacy,category=category)

        meta = Products.meta_types[-1]
        Products.meta_types = Products.meta_types[:-1]

        if len(constructors)>1:
            method = constructors[1]
            if type(method) is type(()):
                name,method = method
            else: name = method.__name__
            meta['template_add_method'] = name

        kind = instance_class.meta_type

        l = EasyTemplateHelpers.get(kind, [])
        l.append(meta)
        EasyTemplateHelpers[kind] = l

        return result

    def registerEasyType(self, instance_class):
        mt_information=EasyMetaTypeInformation(instance_class)
        EasyTypeRegistry.registerType(mt_information)

# Patch ourselves in so we look like part of the Zope registration API
ProductContext.__dict__.update(_ProductContextPatch.__dict__)
