# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/EasyRoot.py,v 1.6 2004/07/13 11:05:23 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__="""EasyRoot"""
__version__='$Revision: 1.6 $'[11:-2]

import sys, Globals, DateTime
from Acquisition import aq_base, aq_chain
from Globals import DTMLFile
from string import join
from AccessControl import ClassSecurityInfo
from Products.EasyCore import EasyCorePermissions
from Products.EasyCore.Utils import Tutils
from urllib import quote
from zLOG import LOG, INFO, BLATHER, ERROR
from copy import deepcopy

from Products.EasyCore.UpgradeSupport.Utils import upgradeProperties
from Products.EasyCore.EasyCorePermissions import *
from Products.EasyCore.EasyCorePermissions import setupPermissions
from Products.EasyCore.EasyEditorConf import EASYEDITORID, DEFAULTTEMPLATEID,DEFAULTVIEWNAME, DEFAULTVIEWATTR, ACQUIRETEMPLATE, ACQUIREVIEW
from Products.EasyEditor.EasyEditable import EasyEditable
from Products.EasyCore.ManageOptionBase import ManageOptionBase
from Products.EasyNavigation.EasyNavigatable import Navigatable
from Products.EasyWorkflow.EasyPublishable import Publishable
from Products.EasyWorkflow.EasyWorkflowBase import NullDate
from Products.EasyWorkflow.HistoryTracker import HistoryTracker
from Products.EasyWorkflow.EasyWorkflowBase import wfDRAFT
from Products.EasyDublinCore.EasyDublinCore import EasyDublinCore
from Products.EasyCore.EasyNavigationConf import NAVIGATION_CATALOG_ID

from Products.EasyCore.BodyTabs import InitializeBodyTabs
from Products.EasyCore import MLOG_TAB

from Products.EasyEditor.EasyDocument import EasyDocument
from Products.EasyEditor.EasyTemplates.EasyTemplates import EasyTemplate
from Products.EasyEditor.EasyTemplates.EasyViews import DTMLView, default_dtml
from Products.EasyEditor.EditableRegions.EditableRegions import EditableRegion
from Products.EasyCore.EasyEditorConf import EASYEDITORID
from Products.EasyCore.EasyEditorConf import DEFAULTSTORAGETYPE, DEFAULTVIEWTITLE
from Products.EasyCore.EasyEditorConf import DEFAULTCONTENT, DEFAULTCONTENTSTORAGEID
from Products.EasyCore.EasyEditorConf import DEFAULTSTORAGECONTENT


from Products.CMFCore.PortalObject import PortalObjectBase


# EasyRoot get added to the application object instead of Folder
# in OFS.Application when the Zope instance is started.

class EasyRoot(EasyDocument, PortalObjectBase, EasyDublinCore):
    """A EasyRoot Object """

    meta_type = 'EasyRoot'
    summary_text_length=150

    security = ClassSecurityInfo()

    _workflowable=1

    _view='default'
    _printview='printview'

    _view_defaults = {
             DEFAULTVIEWATTR: ('default', 'Default', 0),
             '_printview': ('printview', 'Print View', 1)
             }

    skey_default='id'

    _properties = Navigatable._properties + EasyDublinCore._properties +  \
        (
            {'id':'possible_languages', 'label':'Possible Languages', 'type': 'lines', 'mode': 'w',},
            {'id':'possible_categories', 'label':'Possible Categories', 'type': 'lines', 'mode': 'w',},
            {'id':'possible_rights', 'label':'Possible Rights', 'type': 'lines', 'mode': 'w',},
            {'id':'charset', 'label':'Charset', 'type': 'string', 'mode': 'w',},
            {'id':'management_page_charset', 'label':'Management Page Charset', 'type': 'string', 'mode': 'w',},
            {'id':'show_publish_icon', 'label':'Show Publish Icon', 'type':'boolean', 'mode':'w',},
            {'id':'show_worklflow_icon', 'label':'Show Worklflow Icon', 'type':'boolean', 'mode':'w',},
            {'id':'summary_text_length', 'label':'Max Summary Text Length', 'type': 'int', 'mode': 'w',},
            {'id':'skey_default', 'label':'Default Object Sorting', 'type':'selection', 'mode':'wd','select_variable':'possible_obj_sort_columns'},
        )

    use_manage_options = EasyEditable.use_manage_options + \
                        Publishable.use_manage_options + \
                        Navigatable.use_manage_options


    _auto_add_to_navigation = 1
    _isEasyRoot = 1
    show_publish_icon=True
    show_worklflow_icon=True
    charset='ISO-8859-1'
    management_page_charset='UTF-8'

    def __init__(self
                    , id
                    , template=None
                    , title=''
                    , navtitle=''
                    , description=''
                    , language=''
                    , category=()
                    , subject=()
                    , effective_date=NullDate
                    , expiration_date=NullDate
                    , format='text/html'
                    , add_to_navigation=1
                ):
        EasyEditable.__init__(self,id, title, template)
        Navigatable.__init__(self, title, navtitle)
        Publishable.__init__(self)
        EasyDublinCore.__init__(self
                                , title=title
                                , description=description
                                , language=language
                                , category=category
                                , subject=subject
                                , effective_date=effective_date
                                , expiration_date=expiration_date
                                , format=format
                                )
        PortalObjectBase.__init__( self, id, title )
        #DefaultDublinCoreImpl.__init__( self )
        self._auto_add_to_navigation = add_to_navigation
        self.language = language
        self.description = description
        if category is not None:
            self.category=category
        if subject is not None:
            self.subject=subject
        self.initialize()

    def getIcon(self):
        return 'misc_/EasyEditor/EasyRoot.gif'

    security.declarePrivate('initialize')
    def initialize(self, navtitle='',template=ACQUIRETEMPLATE, view=ACQUIREVIEW):

        LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Init EasyRoot\n')
        dirty = 0

        # hacking the EasyDocument data
        self.upgradeWorkflow()

        if not hasattr(self,'_object_version'):
                # Assume that this is a new database.
                self._object_version = '1.8'
                get_transaction().note('_object_version added')
                get_transaction().commit()
                # These roles should be deletable,
                # so only create them when the database is created.
                self._addRole('Author')
                self._addRole('Reviewer')
                setupZopeCorePermissions()
                dirty = 1
                LOG('EasyPublisher', INFO, 'Easy Root init', '  ZODB Initialized\n')

        if not hasattr(self,'navigation_title'):
            self.navigation_title = navtitle
            get_transaction().note('Added navigation_title')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added navigation_title\n')

        if not hasattr(self,'_template'):
            self._template = template
            get_transaction().note('Added _template')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added _template\n')

        if not hasattr(self,'_view'):
            self._view = view
            get_transaction().note('Added _view')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added _view\n')

        if not hasattr(self,'effective_date'):
            self.effective_date = NullDate
            get_transaction().note('Added effective_date')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added effective_date\n')

        if not hasattr(self,'expiration_date'):
            self.expiration_date = NullDate
            get_transaction().note('Added expiration_date')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added expiration_date\n')

        owner = self.getOwner()
        if owner:
            ownerid = owner.getId()
        else:
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Root is unowned\n')
            ownerid = ""

        if not hasattr(self,'last_modified_by'):
            self.last_modified_by = ownerid
            get_transaction().note('Added last_modified_by')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added last_modified_by\n')

        if not hasattr(self,'creation_date'):
            self.creation_date = DateTime.now()
            get_transaction().note('Added creation_date')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added creation_date\n')

        if not hasattr(self,'description'):
            self.description = ''
            get_transaction().note('Added description')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added description\n')

        if not hasattr(self,'subject'):
            self.subject = ''
            get_transaction().note('Added subject')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added subject\n')

        if not hasattr(self,'content_type'):
            self.content_type = 'text/html'
            get_transaction().note('Added content_type')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added content_type\n')

        if not hasattr(self,'contributors'):
            self.contributors = []
            get_transaction().note('Added contributors')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added contributors\n')

        if not hasattr(self,'category'):
            self.category = ''
            get_transaction().note('Added category')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added category\n')

        if not hasattr(self,'language'):
            self.language = 'English'
            get_transaction().note('Added language')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added language\n')

        if not hasattr(self,'rights'):
            self.rights = ''
            get_transaction().note('Added rights')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added rights\n')

        if not hasattr(self,'possible_categories'):
            self.possible_categories = ['Sales', 'Support', 'Software', 'Hardware', 'Cats', 'Dogs', 'Elephants']
            get_transaction().note('Added possible categories')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added possible categories\n')

        if not hasattr(self,'possible_languages'):
            self.possible_languages = ['English', 'Swedish', 'German', 'French', 'Dutch', 'Icelandic']
            get_transaction().note('Added possible languages')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added possible languages\n')

        if not hasattr(self,'possible_rights'):
            self.possible_rights = ['Copyrighted material', 'GPL', 'Public Domain']
            get_transaction().note('Added possible rights')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added possible rights\n')

        # hacking the navigation data
        if not hasattr(self,'navigation_menus_data'):
            parentpath = 'root'
            if hasattr(self, self.navigation_catalog):
              highcount = getattr(self, self.navigation_catalog).getHighestCountForMenu(parentpath)
            else:
              highcount = 0
            self.navigation_menus_data =  { parentpath: (highcount + 1,) }
            get_transaction().note('Added navigation_menus_data')
            get_transaction().commit()
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '  Added navigation_menus_data\n')
            dirty = 1

        #publish_history
        if not hasattr(self,'publish_history'):
            self.publish_history = []
            get_transaction().note('publish_history created')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', 'publish_history created\n')

        #_workflow_history
        if not hasattr(self,'_workflow_history'):
            self._workflow_history = HistoryTracker( Comment="Object Created.",
                                                     User='superuser',
                                                     Time=self.creation_date,
                                                     OldState="",
                                                     State=wfDRAFT
                                                   )
            get_transaction().note('_workflow_history created')
            get_transaction().commit()
            dirty = 1
            LOG('EasyPublisher', BLATHER, 'EasyRoot', '_workflow_history created\n')

        if dirty: self.reindex_object()


    def createSite(self, site_template=None, REQUEST=None):
        """\
        Initialize a site structure from a template.
        """
        if site_template is None:
            site_template={

            }
        try:
            addCMFCoreTool = self.manage_addProduct['CMFCore'].manage_addTool
            addCMFCoreTool('CMF Actions Tool', None)
            #addCMFCoreTool('CMF Catalog', None)
            addCMFCoreTool('CMF Member Data Tool', None)
            addCMFCoreTool('CMF Skins Tool', None)
            addCMFCoreTool('CMF Types Tool', None)
            #addCMFCoreTool('CMF Undo Tool', None)
            addCMFCoreTool('CMF URL Tool', None)
            #addCMFCoreTool('CMF Workflow Tool', None)
            #addCMFDefaultTool = self.manage_addProduct['CMFDefault'].manage_addTool
            #addCMFDefaultTool('Default Discussion Tool', None)
            addCMFDefaultTool('Default Membership Tool', None)
            addCMFDefaultTool('Default Registration Tool', None)
            #addCMFDefaultTool('Default Properties Tool', None)
            #addCMFDefaultTool('Default Metadata Tool', None)
            #addCMFDefaultTool('Default Syndication Tool', None)
            addCMFFormControllerTool = self.manage_addProduct['CMFFormController'].manage_addTool
            addCMFFormControllerTool('Form Controller Tool', None)
        except: pass

        addProduct=self.manage_addProduct
        try:
            addProduct['EasyEditor'].manage_addEasyEditor(REQUEST=REQUEST)
            editor = self._getOb('EasyEditor')
            template=EasyTemplate(id=DEFAULTTEMPLATEID,title=DEFAULTTEMPLATEID)
            editor._setObject(DEFAULTTEMPLATEID, template)
            template = editor._getOb(DEFAULTTEMPLATEID)
            default=DTMLView(DEFAULTVIEWNAME, DEFAULTVIEWTITLE, 1, source_string=default_dtml)
            template._setObject(DEFAULTVIEWNAME, default)
            content = EditableRegion(DEFAULTCONTENT
                                    , DEFAULTSTORAGETYPE
                                    , DEFAULTSTORAGECONTENT
                                    , 1
                                    , '#FFFFFF'
                                    , '#FFCCCC'
                                    , DEFAULTCONTENTSTORAGEID
            )
            template._setObject(DEFAULTCONTENT, content)
        except: pass
        try:
            addProduct['EasyLanguageService'].manage_addEasyLanguageService(REQUEST=REQUEST)
        except: pass
        try:
            addProduct['EasyNavigation'].manage_addEasyNavigation(REQUEST=REQUEST)
        except: pass
        try:
            addProduct['EasyMailService'].manage_addEasyMailService(REQUEST=REQUEST)
        except: pass
        try:
            addProduct['EasyNewsService'].manage_addEasyNewsService(REQUEST=REQUEST)
        except: pass
        try:
            addProduct['EasyNotification'].manage_addEasyNotification(REQUEST=REQUEST)
        except: pass
        try:
            addProduct['EasyResources'].manage_addEasyResources(REQUEST=REQUEST)
        except: pass
        try:
            addProduct['EasySearchService'].manage_EasySearchServiceAdd('EasySearch', REQUEST=REQUEST)
        except: pass
        try:
            addProduct['EasyWorkflow'].manage_addEasyWorkflowManager(REQUEST=REQUEST)
        except: pass



        setupPermissions(self)

        manage_tabs_message='A new EasyRoot has been created.'

        if REQUEST is not None:
            return self.manage_main(self,REQUEST,manage_tabs_message="",management_view='Content')


    #Initilized in EasyEditor __init__.py
    def __special__bobo_traverse__(self, REQUEST, name=None):
        #It's for the Application Override... Don't know what it does?
        try:
            return self.__application__bobo_traverse__(REQUEST, name)
        except:
            return EasyEditable.__bobo_traverse__(self,REQUEST, name)

    #EVENT TRIGGERS
    #These are not going to be used right?
    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        Navigatable.manage_afterAdd(self, item, container)
        EasyEditable.manage_afterAdd(self, item, container)

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        Navigatable.manage_beforeDelete(self, item, container)
        EasyEditable.manage_beforeDelete(self, item, container)


    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        #if getattr(aq_base(self),'_object_version', None)!='1.7': self._object_version = '1.7'
        if getattr(aq_base(self),'_object_version', None)!='1.8': self._object_version = '1.8' ; changed=1
        if not getattr(aq_base(self),'_object_links', None): self._object_links = {} ; changed=1
        upgradeProperties(self, {'skey_default':'position',}, ())
        if REQUEST is not None:
            return "<h1>Done</h1><p>Changed: %s</p>" % str(changed)
        return changed

Globals.InitializeClass(EasyRoot)
InitializeBodyTabs(EasyRoot)




manage_addEasyRootForm=DTMLFile('dtml/manage_addEasyRootForm', globals())

def manage_addEasyRoot(self
                    , id
                    , template=None
                    , title=''
                    , navtitle=''
                    , description=''
                    , language=''
                    , category=()
                    , subject=()
                    , effective_date=NullDate
                    , expiration_date=NullDate
                    , format='text/html'
                    , add_to_navigation=1
                    , REQUEST=None
                    ):
    """Add EasyRoot"""
    id=Tutils.xlate2url(id)
    # BUG
    # Something isn't right here, for instance if a EasyEditor
    # has a none default viewname the created EasyRoot
    # would try to render DEFAULTVIEWNAME instead of EasyRoot.default_view
    ob=EasyRoot(id
                    , template=template
                    , title=title
                    , navtitle=navtitle
                    , description=description
                    , language=language
                    , category=category
                    , subject=subject
                    , effective_date=effective_date
                    , expiration_date=expiration_date
                    , format=format
                    , add_to_navigation = add_to_navigation
                    )
    self._setObject(id, ob)
    ob = self._getOb(id) # get object in context
    ob.createSite()
    ob.initcontent()


    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        REQUEST.RESPONSE.redirect(u+'/manage_main')



def initialize(context):
    """Initialize EasyEditable Product"""


    context.registerClass (
            EasyRoot,
            permission=EasyCorePermissions.EasyRootAdd,
            icon='icons/EasyRoot.gif',
            constructors=(manage_addEasyRootForm,
                          manage_addEasyRoot),
            meta_type = EasyRoot.meta_type,
            category='Easy Publisher Management',
            )

    context.registerEasyType(EasyRoot)