# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/EditableRegions/EditableRegions.py,v 1.3 2004/06/30 21:03:40 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__ = """EasyTemplates Editable Regions"""
__version__='$Revision: 1.3 $'[11:-2]



import Globals, sys
from Globals import DTMLFile
from OFS.SimpleItem import SimpleItem
from urllib import quote
from Acquisition import aq_base, aq_chain, ImplicitAcquisitionWrapper
from Products.EasyEditor.EasyEditorRegistry import EasyStorage
from AccessControl import ClassSecurityInfo
from Products.EasyCore import EasyCorePermissions
from string import strip
import DateTime

from OFS.Cache import Cacheable
from webdav.Lockable import ResourceLockedError
from webdav.WriteLockInterface import WriteLockInterface
from OFS.History import html_diff, Historical
from Products.EasyCore.PropertyTextParser import PropertyTextParser, PropertyTextGenerator
import sys, traceback
from types import DictType
from Products.EasyCore import MLOG_TAB


from Products.EasyCore import MLOG_TAB

_marker=[]


class EditableRegion(SimpleItem, Cacheable, Historical):
    """A EditableRegion"""

    security = ClassSecurityInfo()

    __region_kind__ = 'EditableRegion'

    meta_type = "EditableRegion"
    _v_errors=''

    isPrincipiaFolderish = 1
    isSearchable = 0
    acquisition_blocks_on_creation=0
    content_type="text/plain"
    label=None

    __implements__ = (WriteLockInterface,)

    _hidden_properties = (
                            {   'id':'storagecontainerid',
                                'label':'StorageContainerID',
                                'type': 'string',
                            },
                            {   'id':'label',
                                'label':'Label',
                                'type': 'string',
                            },
                            {   'id':'storage',
                                'label':'Storage',
                                'type': 'string',
                            },
                            {   'id':'default',
                                'label':'Default',
                                'multiline':1,
                                'type': 'text',
                            },
                            {   'id':'isSearchable',
                                'label':'Is Searchable',
                                'type': 'boolean',
                            },
                            {   'id':'bgcolor',
                                'label':'Background Color',
                                'type': 'string',
                            },
                            {   'id':'highlight_bgcolor',
                                'label':'Highlight Color',
                                'type': 'string',
                            },
                            {   'id':'acquisition_blocks_on_creation',
                                'label':'Acquisition Blocks on Creation',
                                'type': 'boolean',
                            },
                        )


    regions_manage_options = ( {'label':'Edit',   'action':'manage_main'}, ) + SimpleItem.manage_options

    def __init__(self, id, storage, default, isSearchable, bgcolor, highlight_bgcolor, storagecontainerid=None, acquisition_blocks_on_creation=0):
        self.id = id
        self.label=id
        if storagecontainerid is None:
            self.storagecontainerid = id
        else:
            self.storagecontainerid = storagecontainerid
        self.storage = storage
        self.default = default
        self.isSearchable = isSearchable
        self.bgcolor = bgcolor
        self.highlight_bgcolor = highlight_bgcolor
        self.acquisition_blocks_on_creation=acquisition_blocks_on_creation
        self._last_changed=DateTime.now()

    def initcontent(self, editablecontainer):
        self.getStorage().manage_save(editablecontainer, self)

    def manage_options(self):
        return self.regions_manage_options


    #MANAGE STORAGE

    def hasStorage(self,storageid):
        return storageid in EasyStorage.keys()

    def getStorage(self):
        return ImplicitAcquisitionWrapper(EasyStorage[self.storage], self)

    def storageIds(self,spec=None):
        return EasyStorage.keys()

    def storageValues(self,spec=None):
        return EasyStorage.values()

    def storageItems(self,spec=None):
        return EasyStorage.items()

    def lastChanged(self):
        return self._last_changed

    def setDirty(self):
        self._last_changed=DateTime.now()
        if hasattr(self,'setEditorDirty') and callable(self.setEditorDirty):
            self.setEditorDirty()

    def hasChanged(self, editablecontainer):
        return self.getStorage().hasChanged(editablecontainer, self)

    # MANAGE EDITABLE REGION

    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_main')
    manage_main = DTMLFile('dtml/manage_main_EditableRegion', globals())

    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_edit')
    def manage_edit(self, label, storage, bgcolor, highlight_bgcolor, isSearchable=0, storagecontainerid=None, acquisition_blocks_on_creation=0, REQUEST=None):
        """Change the storage for this Editable Region"""
        self.label=label
        if storage:
            self.storage = storage
        if storagecontainerid and storagecontainerid!=self.storagecontainerid:
            storagecontainerid = self.checkRegionStorageId(storagecontainerid)
            self.storagecontainerid = storagecontainerid
        elif storagecontainerid=='' and self.storagecontainerid!=self.id:
            try:
                storagecontainerid = self.checkRegionStorageId(self.id)
                self.storagecontainerid = storagecontainerid
            except:
                return self.manage_main(self,REQUEST,management_view='Edit',management_message="ERROR")
        self.bgcolor = bgcolor
        self.highlight_bgcolor = highlight_bgcolor
        self.isSearchable=isSearchable
        self.acquisition_blocks_on_creation=acquisition_blocks_on_creation
        self.manage_save_storage_conf(REQUEST)
        self.manage_save_default_conf(REQUEST)
        self.setDirty()
        if REQUEST is not None:
            return self.manage_main(self,REQUEST,management_view='Edit')

    security.declarePrivate('manage_initilized')
    def manage_initilized(self, editablecontainer):
        self.getStorage().manage_initilized(editablecontainer, self)

    security.declarePrivate('manage_load')
    def manage_load(self, editablecontainer, default=_marker):
        if default is _marker:
            default = self.getDefault()
        return self.getStorage().manage_load(editablecontainer, self, default)

    security.declarePrivate('manage_save')
    def manage_save(self, editablecontainer, value=None):
        self.getStorage().manage_save(editablecontainer, self, value)

    security.declarePrivate('manage_compare')
    def manage_compare(self, editablecontainer, obj):
        return self.getStorage().manage_compare(editablecontainer, self, obj)

    security.declarePrivate('manage_copy')
    def manage_copy(self, editablecontainer, parent):
        self.getStorage().manage_copy(editablecontainer, self, parent)

    security.declarePrivate('manage_save_storage_conf')
    def manage_save_storage_conf(self, REQUEST):
        storage=self.getStorage()
        self.setDirty()
        return storage.manage_save_storage_conf(self, REQUEST)

    security.declarePrivate('manage_view_storage_conf')
    def manage_view_storage_conf(self, REQUEST):
        storage=self.getStorage()
        return storage.manage_view_storage_conf(self, REQUEST)

    security.declarePrivate('manage_view_default_conf')
    def manage_view_default_conf(self, REQUEST):
        storage=self.getStorage()
        self.setDirty()
        return storage.manage_view_default_conf(self, REQUEST)

    security.declarePrivate('manage_save_default_conf')
    def manage_save_default_conf(self, REQUEST):
        storage=self.getStorage()
        return storage.manage_save_default_conf(self, REQUEST)

    security.declarePrivate('getDefault')
    def getDefault(self):
        storage=self.getStorage()
        return storage.getDefault(self)

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        EditableRegion.inheritedAttribute('manage_afterAdd')(self, item, container)
        self.setDirty()


    security.declarePrivate('render_html_view')
    def render_html_view(self, editablecontainer, **kw):
        storage=self.getStorage()
        if hasattr(aq_base(storage), 'render_html_view'):
            return apply(storage.render_html_view, (editablecontainer, self), kw)
        return '<p>No Storage HTML View</p>'

    security.declarePrivate('getStorageType')
    def getStorageType(self):
        storage=self.getStorage()
        return storage.getStorageType()


    def read(self):
        error=''
        try:
            generator = PropertyTextGenerator()
            generator.renderFromInstance(self, self._hidden_properties)
            if self._v_errors:
                return "%s\n%s" %(self._v_errors, generator.getCode())
            return generator.getCode()
        except:
            try:
                error="# ERROR: %s: %s\n# \n" % sys.exc_info()[:2]
                generator_errors='\n'.join(map(lambda err: "# " + str(err), generator.getErrors()))
                if self._v_errors:
                    return error + "%s\n%s" % (self._v_errors, generator_errors)
                return local_errors
            except:
                pass
        return error + self._v_errors

    def munge(self, body):
        self._v_errors = ''
        try:
            parser = PropertyTextParser()
            parser.parseToInstance(self, self._hidden_properties, body)
        except:
            error="# ERROR: %s: %s\n# \n" % sys.exc_info()[:2]
            self._v_errors = error + '\n'.join(map(lambda err: "# " + str(err), parser.getErrors()))


    #FTP Support
    security.declareProtected('View management screens', 'document_src')
    def document_src(self, REQUEST=None, RESPONSE=None):
        """Return unprocessed document source."""
        if RESPONSE is not None:
            RESPONSE.setHeader('Content-Type', self.content_type)
        return self.read()

    ## Protocol handlers
    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'PUT')
    def PUT(self, REQUEST, RESPONSE):
        """Handle HTTP PUT requests."""
        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        body=REQUEST.get('BODY', '')
        self.munge(body)
        self.setDirty()
        self.ZCacheable_invalidate()
        RESPONSE.setStatus(204)
        return RESPONSE

    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_FTPget')
    def manage_FTPget(self):
        "Get source for FTP download"
        return self.read()

    def get_size(self):
        return len(self.read())

    def content_type(self):
        return self.content_type

    def getType(self):
        return self.getStorage().getType()


    security.declarePublic('index_html')
    def index_html(self, client=None, REQUEST=None):
        """Render default content."""
        return self.default

    __call__=index_html

    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        if type(getattr(self.aq_base, '__ac_local_roles__'))==type({}):
            del self.__ac_local_roles__
            changed = 1
        if getattr(self,'_object_version', None)!='1.7': self._object_version = '1.7'
        if REQUEST is not None:
            return "<h1>Done</h1>"
        return changed

Globals.InitializeClass(EditableRegion)

manage_add_EditableRegionForm = DTMLFile('dtml/manage_add_EditableRegionForm', globals())

def manage_add_EditableRegion(self, id, storage, bgcolor, highlight_bgcolor, isSearchable=0, submit='', default='', storagecontainerid=None, REQUEST=None):
    """Add a Editable Region"""
    if hasattr(self,'checkRegionStorageId'):
        storagecontainerid = self.checkRegionStorageId(storagecontainerid)
    else:
        raise "Can't checkRegionStorageId"
    id = strip(id)
    ob = EditableRegion(id, storage,default,isSearchable, bgcolor, highlight_bgcolor,storagecontainerid)
    id=self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')


def initialize(context):
    """Initialize EditableRegion Product"""


    context.registerEasyRegionClass(
            EditableRegion,
            meta_type = EditableRegion.meta_type,
            permission = EasyCorePermissions.EasyTemplateAdd,
            constructors = (manage_add_EditableRegionForm, manage_add_EditableRegion),
            icon='icons/EditableRegion.gif',
            category='Easy Templates',
        )


