# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/Storage/MultiSelectionPropertyStorage.py,v 1.2 2004/06/30 21:03:40 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__="""PropertyStorage

$Colliberty: PropertyStorage.py,v 1.4.37 2001/10/12 18:08:00 johanc Exp $"""
__version__='$Revision: 1.2 $'[11:-2]


import Globals
from Globals import HTML
from SelectionPropertyStorage import SelectionPropertyStorage
from ListTypeStorages import ListTypeStorage
from ZPublisher.Converters import type_converters
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from types import StringType, ListType, TupleType
from string import atoi, capitalize

from Products.EasyCore import MLOG_TAB

_marker=[]

POSSIBLE_LINES='possible_'



class MultiSelectionPropertyStorage(ListTypeStorage, SelectionPropertyStorage):
    """A MultiSelectionPropertyStorage Object """

    property_type = 'multiple selection'
    property_max_height=5
    storage_type="MultiSelectionPropertyStorage"
    value_type='list'

    security = ClassSecurityInfo()

    def validityCheck(self, editablecontainer, region):
        prop = self.getPropertyInfoFor(editablecontainer, region)
        value = hasattr(editablecontainer, region.storagecontainerid, _marker)
        return prop and value is not _marker and \
               type(value) in (ListType, TupleType) and \
               prop.has_key('') and hasattr(editablecontainer, prop['select_variable'])



    def render_html_view(self, editablecontainer, region):
        REQUEST=getattr(editablecontainer ,'REQUEST', None)
        select_variable=self.getSelectVariable(editablecontainer, region)
        property_select_list=getattr(editablecontainer, select_variable, None)
        if property_select_list is None:
            property_select_list=getattr(region, 'property_select_list', [])

        value=self.manage_load(editablecontainer, region, region.getDefault())
        if type(value) not in (TupleType, ListType):value=[value]

        return self._default_render_html_view(self, REQUEST=REQUEST
                            , value=value
                            , select_list=property_select_list
                            , size=max(getattr(region, 'property_max_height', self.property_max_height), len(value))
                            , id=region.id
                            )

    _default_render_html_view=HTML("""\
        <div class="form-element">
        <select name="&dtml-id;:list" size="&dtml-size;" multiple>
          <dtml-in select_list prefix="o">
             <option value="&dtml-o_item;" <dtml-var "o_item in value and 'selected' or ''">><dtml-var o_item></option>
          </dtml-in>
        </select>
        </div>
        """, globals())



    _default_property_config=HTML("""\
        <dtml-with "region">
        <table width="100%">
          <tr><td colspan="2" class="form-label">Mode</td></tr>
          <tr>
            <td>
              <input type="hidden" name="deletable:int:missing" value="0">
              <input type="checkbox" name="deletable:int" value="1" <dtml-if "'d' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Deletable</td>
          </tr>
          <tr>
            <td>
              <input type="hidden" name="writtable:int:missing" value="0">
              <input type="checkbox" name="writtable:int" value="1" <dtml-if "'w' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Writable</td>
          </tr>
        </table>
        <table>
          <tr>
            <td class="form-label">Max Height</td>
            <td><input type="text" name="property_max_height:int" value="&dtml.missing-property_max_height;"></td>
          </tr>
          <tr>
            <td class="form-label">Select Variable</td>
            <td><input type="text" name="property_select_variable" value="&dtml.missing-property_select_variable;"></td>
          </tr>
        </table>
        </dtml-with region>
        """, globals())

    def manage_view_storage_conf(self, region, REQUEST):
        return self._default_property_config(self, region=region, REQUEST=REQUEST)

    def manage_save_storage_conf(self, region, REQUEST):

        property_mode=''
        if REQUEST.get('writtable', 0):
            property_mode=property_mode+'w'
        if REQUEST.get('deletable', 0):
            property_mode=property_mode+'d'
        region.property_mode = property_mode


        region.property_max_height = REQUEST.get('property_max_height',getattr(region,'property_max_height', self.property_max_height))

        region.property_select_variable = REQUEST.get('property_select_variable',getattr(region,'property_select_variable', self.property_select_variable))


Globals.InitializeClass(MultiSelectionPropertyStorage)


def initialize(context):
    """Initialize PropertyStorage"""

    context.registerEasyStorage(MultiSelectionPropertyStorage,   'MultiSelectionPropertyStorage')


