# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/Storage/PropertyStorage.py,v 1.4 2004/07/13 11:05:23 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__="""PropertyStorage

$Colliberty: PropertyStorage.py,v 1.8 2001/10/12 18:08:00 johanc Exp $"""
__version__='$Revision: 1.4 $'[11:-2]


import Globals
from Globals import HTML

from Acquisition import aq_parent, aq_base
from ZPublisher.Converters import type_converters
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from types import StringType, ListType, TupleType
from string import atoi, capitalize

from Storage import EasyStorageBase, getAllListValues, getActiveListValues
from Products.EasyCore import MLOG_TAB

_marker=[]

POSSIBLE_LINES='possible_'


class PropertyStorage(EasyStorageBase):
    """A PropertyStorage Object """

    storage_type="PropertyStorage"
    value_type='string'
    property_type = 'None'
    property_mode='w'
    property_select_variable=''

    security = ClassSecurityInfo()

    def _updateProperty(self, container, region, id, value, force_update=0):
        if force_update or not hasattr(aq_base(container), id):
            setattr(container, id, value)

    def valid_property_id(self, container, region, id):
        if not id or id[:1]=='_' or (id[:3]=='aq_') or (' ' in id):
            return 0
        return 1

    def hasPropertyEntry(self, container, region, id):
        for prop in container._properties:
            if prop['id']==id: return 1
        return 0

    def getPropertyInfoFor(self, container, region, id):
        for prop in container._properties:
            if prop['id']==id:
                return prop
        return None

    def getSelectVariable(self, container, region, id=None):
        select_variable=getattr(region, 'property_select_variable', None)
        if not select_variable:
            if id is None: id=region.storagecontainerid
            select_variable=POSSIBLE_LINES+id
        return select_variable

    def _getSelectVariable(self, container, region, id):
        select_variable=self.getSelectVariable(container, region, id)
        if self.property_type in ('selection', 'multiple selection'):
            if not hasattr(container, select_variable):
                property_select_list=getattr(region, 'property_select_list', [])
                container._setProperty(select_variable, property_select_list, 'lines')
        return select_variable

    def _update_property_entry(self, container, region, id):
        select_variable=self._getSelectVariable(container, region, id)
        if self.hasPropertyEntry(container, region, id):
            new_props=()
            for prop in container._properties:
                if prop['id']==id:
                    prop['type']=self.property_type
                    prop['label']=region.label or region.id
                    prop['mode']=getattr(region, 'property_mode', 'w')
                    if select_variable:
                        prop['select_variable']=select_variable
                    new_props=new_props+(prop,)
                else:
                    new_props=new_props+(prop,)
            container._properties=new_props
        else:
            new_prop = {'id':               id,
                        'type':             self.property_type,
                        'label':            region.label or region.id,
                        'mode':             getattr(region, 'property_mode', 'wd'),
                    }
            if select_variable:
                new_prop['select_variable']=select_variable
            container._properties=container._properties + (new_prop,)


    def _setProperty(self, container, region, id, value, force_update=0):
        container._wrapperCheck(value)
        if not self.valid_property_id(container, region, id):
            raise 'Bad Request', 'Invalid or duplicate property id'

        self._update_property_entry(container, region, id)

        if self.property_type in ('multiple selection', 'tokens', 'lines') and type(value) not in (ListType, TupleType):
            value=[value]

        if force_update or not hasattr(aq_base(container), id):
            setattr(container, id, value)


    def compareHistoryItem(self, container, region, item):
        res=[]
        add=res.append
        if item.storage_type != self.storage_type:   add('storage_type')
        if item.property_type != self.property_type: add('property_type')
        if item.property_mode != self.property_mode: add('property_mode')
        if item.property_label != region.label: add('property_label')
        if item.property_select_variable != self.property_select_variable: add('property_select_variable')

        if item.last_changed != region.lastChanged(): add('lastChanged')
        return res

    def validityCheck(self, container, region):
        prop=self.getPropertyInfoFor(container, region, region.storagecontainerid)
        return prop and hasattr(container, region.storagecontainerid)

    security.declarePrivate('do_initilized')
    def do_initilized(self, container, region):
        self.do_add(container, region, region.getDefault())

    security.declarePrivate('do_add')
    def do_add(self, container, region, value):
        self._setProperty(container, region, region.storagecontainerid, value, force_update=1)
        self.setRSHistoryItem(container, region)
        self._setLastChanged(container, region, region.lastChanged())

    security.declarePrivate('do_get')
    def do_get(self, container, region, default=None):
        return getattr(container, region.storagecontainerid, default)

    security.declarePrivate('do_set')
    def do_set(self, container, region, value):
        setattr(container, region.storagecontainerid, value)

    security.declarePrivate('do_transform')
    def do_transform(self, container, region, item):
        old_storage=item.getStorage()
        if old_storage:
            value=old_storage.getValueForType(container, region, self.value_type)
        else:
            value=self.do_get(self, container, region, default=region.getDefault())
        self.do_add(container, region, value)

    security.declarePrivate('do_transform_property')
    def do_transform_property(self, container, region, item):
        self._update_property_entry(container, region, region.storagecontainerid)

    def isInitialized(self, container, region):
        result=hasattr(container, region.storagecontainerid)
        result=result and self.hasPropertyEntry(container, region, region.storagecontainerid)
        if self.property_type in ('selection', 'multiple selection'):
            select_variable=getattr(region, 'property_select_variable', None)
            if not select_variable:
                select_variable=POSSIBLE_LINES+region.storagecontainerid
            result=result and (hasattr(aq_parent(container), select_variable) or (self.hasPropertyEntry(container, region, select_variable) and hasattr(aq_base(container), select_variable)))
        return result

    security.declarePrivate('manage_initilized')
    def manage_initilized(self, container, region):
        item=self.getRSHistoryItem(container, region)
        if item:
            changes=self.compareHistoryItem(container, region, item)
            if 'storage_type' in changes or 'property_type' in changes:
                self.do_transform(container, region, item)
            elif 'property_label' in changes or 'property_mode' in changes or 'property_select_variable' in changes:
                self.do_transform_property(container, region, item)
        if not self.isInitialized(container, region):
            self.do_initilized(container, region)
        self.setRSHistoryItem(container, region)
        self._setLastChanged(container, region, region.lastChanged())

    security.declarePrivate('manage_save')
    def manage_save(self, editablecontainer, region, value=None):
        if self.hasChanged(editablecontainer, region) or self.isInitialized(editablecontainer, region):
            self.manage_initilized(editablecontainer, region)
        if value is None:
            value=self.do_get(editablecontainer, region, region.getDefault())
        self.do_set(editablecontainer, region, value)

    security.declarePrivate('manage_load')
    def manage_load(self, container, region, default=_marker):
        #This caused problems when deleting properties.
        #Because it recreated when indexed, specialy for select_variables.
        #So I have deleted it, and hope nothing relies one it.
        #if self.hasChanged(container, region) or not self.validityCheck(container, region):
        #    self.manage_initilized(container, region)
        value = self.do_get(container, region, default=default)
        if value is _marker: return region.getDefault()
        return value

    security.declarePrivate('manage_delete')
    def manage_delete(self, container, region):
        #Remove a property, if it's allowed
        if self.manage_delete_allowed(container, region):
            #Remove select_Variable properties if the exist
            prop=self.getPropertyInfoFor(container, region, region.storagecontainerid)
            if prop and prop.has_key('select_variable') and container.hasProperty(prop['select_variable']):
                container._delProperty(prop['select_variable'])
            id=region.storagecontainerid
            if hasattr(container, id): delattr(container,id)
            container._properties=tuple(filter(lambda i, n=id: i['id'] != n, container._properties))


    def manage_compare(self, container, region, obj):
        local_value=self.manage_load(container, region, _marker)
        if local_value is _marker: return 0
        return obj==local_value

    def manage_copy(self, container, region, parent):
        local_value=self.manage_load(container, region, _marker)
        if local_value is _marker:
            return
        if hasattr(parent,'_setProperty'):
            if parent.hasProperty(region.storagecontainerid,):
                if parent.getPropertyType(region.storagecontainerid) == self.property_type:
                    parent._updateProperty(region.storagecontainerid, value)
                    return
                else:
                    if 'd' in filter(lambda x,id=region.storagecontainerid: x['id']==id ,parent._properties)[0]['mode']:
                        container._delProperty(region.storagecontainerid)
                    else:
                        raise "DeletePropertyError", "You can't delete or convert property [%s], (type: [%s] to new type: [%s])" % (region.storagecontainerid,container.getPropertyType(region.storagecontainerid) ,self.property_type)
            if type_converters.has_key(self.property_type):
                value=type_converters[self.property_type](value)
            self._setProperty(parent, region, region.storagecontainerid, value)
        else:
            setattr(parent, region.storagecontainerid, local_value)

    def manage_diff(self, container, region, obj):
        raise 'Not implemented'

    def render_html_view(self, container, region):
        return "%s (%s)" % (self.manage_load(container, region, region.getDefault()) or '--', self.property_type)


    _default_property_config=HTML("""\
        <dtml-with "region">
        <table width="100%">
          <tr><td colspan="2" class="form-label">Mode</td></tr>
          <tr>
            <td>
              <input type="hidden" name="deletable:int:missing" value="0">
              <input type="checkbox" name="deletable:int" value="1" <dtml-if "'d' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Deletable</td>
          </tr>
          <tr>
            <td>
              <input type="hidden" name="writtable:int:missing" value="0">
              <input type="checkbox" name="writtable:int" value="1" <dtml-if "'w' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Writable</td>
          </tr>
        </table>
        </dtml-with region>
        """, globals())

    def manage_view_storage_conf(self, region, REQUEST):
        return self._default_property_config(self, region=region, REQUEST=REQUEST)

    def manage_save_storage_conf(self, region, REQUEST):
        property_mode=''
        if REQUEST.get('writtable', 0):
            property_mode=property_mode+'w'
        if REQUEST.get('deletable', 0):
            property_mode=property_mode+'d'
        region.property_mode = property_mode



Globals.InitializeClass(PropertyStorage)



class BooleanPropertyStorage(PropertyStorage):
    """A BooleanPropertyStorage Object """
    property_type = 'boolean'

    security = ClassSecurityInfo()

    def _saveProperty(self, name, value, editablecontainer):
        setattr(aq_base(editablecontainer),'_' + name + '_storage',self.id)
        if editablecontainer.hasProperty(name):
            if editablecontainer.getPropertyType(name) == self.property_type:
                editablecontainer._updateProperty(name, value)
                return
            else:
                if 'd' in filter(lambda x,name=name: x['id']==name ,editablecontainer._properties)[0]['mode']:
                    editablecontainer._delProperty(name)
                else:
                    raise "DeletePropertyError", "You can't delete or convert property [%s], (type: [%s] to new type: [%s])" % (name,editablecontainer.getPropertyType(name) ,self.property_type)
        editablecontainer._setProperty(name, not not value, self.property_type)

    security.declarePrivate('manage_save')
    def manage_save(self, editablecontainer, region, value=None):
        _storage = getattr(aq_base(editablecontainer),'_' + name + '_storage',None)
        if _storage is not None and _storage!=self.id:
            storage = self.getStorage(_storage)
            if storage is not None:
                storage.manage_delete(name, editablecontainer)
            else: raise "StorageException", "Need to have storage [%s] installed to be able to work with this region." % (_storage,)

        self._saveProperty(name,value,editablecontainer)

    def render_html_view(self, name, editablecontainer, region):
        return """<input type="hidden" name="%s:boolean:default" value="0">
               <input type="checkbox" name="%s:boolean" value="1" %s>"""  % (
                      name
                    , name
                    , self.manage_load(name, editablecontainer, region.default) and 'checked' or ''
                    )


Globals.InitializeClass(BooleanPropertyStorage)


class IntPropertyStorage(PropertyStorage):
    """A IntPropertyStorage Object """
    property_type = 'int'

    def render_html_view(self, name, editablecontainer, region):

        return """<input type="text" name="%s:int" value="%s">"""  % (
                      name
                    , self.manage_load(name, editablecontainer, region.default)
                    )

Globals.InitializeClass(IntPropertyStorage)


class FloatPropertyStorage(PropertyStorage):
    """A FloatPropertyStorage Object """
    property_type = 'float'

    def render_html_view(self, name, editablecontainer, region):
        return """<input type="text" name="%s:float" value="%s">"""  % (
                      name
                    , self.manage_load(name, editablecontainer, region.default)
                    )

Globals.InitializeClass(FloatPropertyStorage)


class LongPropertyStorage(PropertyStorage):
    """A LongPropertyStorage Object """
    property_type = 'long'

    def render_html_view(self, name, editablecontainer, region):
        return """<input type="text" name="%s:long" value="%s">"""  % (
                      name
                    , self.manage_load(name, editablecontainer, region.default)
                    )


Globals.InitializeClass(LongPropertyStorage)

class LinesPropertyStorage(PropertyStorage):
    """A LinesPropertyStorage Object """
    property_type = 'lines'

    def render_html_view(self, name, editablecontainer, region):
        req=getattr(editablecontainer ,'REQUEST', None)
        rows=20
        cols=50
        if req is not None and req.has_key('dtpref_cols') and req.has_key('dtpref_rows'):
            rows=max(atoi(req.dtpref_rows),20)
            if dtpref_cols[-1]=='%':
                cols= str(min(100, max(25,int(dtpref_cols[:-1])+dc)))+'%'
            else:
                cols=str(max(35,int(dtpref_cols)+dc))

        lines=self.manage_load(name, editablecontainer, region.default)
        if type(lines)!=StringType:
            lines = '\n'.join(lines)
        return """<textarea name="%s:lines" rows="%s" cols="%s">%s</textarea>"""  % (
                      name
                    , rows
                    , cols
                    , lines)
Globals.InitializeClass(LinesPropertyStorage)


class TokensPropertyStorage(PropertyStorage):
    """A TokensPropertyStorage Object """
    property_type = 'tokens'

    def render_html_view(self, name, editablecontainer, region):
        req=getattr(editablecontainer ,'REQUEST', None)
        cols=50
        if req is not None and req.has_key('dtpref_cols') and req.has_key('dtpref_rows'):
            dtpref_cols=req.dtpref_cols
            if type(dtpref_cols) is StringType and dtpref_cols[-1]=='%': dtpref_cols=dtpref_cols[:-1]
            cols=max(atoi(dtpref_cols)-15,20)

        tokens=self.manage_load(name, editablecontainer, region.default)
        if type(tokens)!=StringType:
            tokens = ' '.join(tokens)
        return """<input type="text" name="%s:tokens" value="%s" size="%s">"""  % (name, tokens, cols)


Globals.InitializeClass(TokensPropertyStorage)





def initialize(context):
    """Initialize PropertyStorage"""

    context.registerEasyStorage(IntPropertyStorage,     'IntPropertyStorage')
    context.registerEasyStorage(FloatPropertyStorage,   'FloatPropertyStorage')
    context.registerEasyStorage(LongPropertyStorage,   'LongPropertyStorage')
    context.registerEasyStorage(BooleanPropertyStorage, 'BooleanPropertyStorage')
    context.registerEasyStorage(LinesPropertyStorage, 'LinesPropertyStorage')
    context.registerEasyStorage(TokensPropertyStorage, 'TokensPropertyStorage')


    #context.registerEasyStorage(KeywordsPropertyStorage,'KeywordsPropertyStorage')


