# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/Storage/StringPropertyStorages.py,v 1.2 2004/06/30 21:03:40 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__="""StringPropertyStorages

$Colliberty: StringPropertyStorages.py,v 1.4.37 2001/10/12 18:08:00 johanc Exp $"""
__version__='$Revision: 1.2 $'[11:-2]


import Globals
from Globals import HTML
from PropertyStorage import PropertyStorage
from ZPublisher.Converters import type_converters
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from types import StringType, ListType, TupleType
from string import atoi, capitalize


from Products.EasyCore import MLOG_TAB



_marker=[]

POSSIBLE_LINES='possible_'


def quotedHTML(text,
               character_entities=(
                   (('&'), '&amp;'),
                   (("<"), '&lt;' ),
                   ((">"), '&gt;' ),
                   (('"'), '&quot;'))): #"
    for re,name in character_entities:
        if text.find(re) >= 0: text=name.join(text.split(re))
    return text


class StringPropertyStorage(PropertyStorage):
    """A StringPropertyStorage Object """

    property_type = 'string'
    storage_type="StringPropertyStorage"
    value_type='string'

    _default_property_config=HTML("""\
        <dtml-with "region">
        <table width="100%">
          <tr><td colspan="2" class="form-label">Mode</td></tr>
          <tr>
            <td>
              <input type="hidden" name="deletable:int:missing" value="0">
              <input type="checkbox" name="deletable:int" value="1" <dtml-if "'d' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Deletable</td>
          </tr>
          <tr>
            <td>
              <input type="hidden" name="writtable:int:missing" value="0">
              <input type="checkbox" name="writtable:int" value="1" <dtml-if "'w' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Writable</td>
          </tr>
        </table>
        </dtml-with region>
        """, globals())

    def render_html_view(self, editablecontainer, region):
        req=getattr(editablecontainer ,'REQUEST', None)
        cols=50
        if req is not None and req.has_key('dtpref_cols') and req.has_key('dtpref_rows'):
            dtpref_cols=req.dtpref_cols
            if type(dtpref_cols) is StringType and dtpref_cols[-1]=='%': dtpref_cols=dtpref_cols[:-1]
            cols=max(atoi(dtpref_cols)-15,20)
        value=self.manage_load(editablecontainer, region, region.getDefault())
        value=quotedHTML(value)
        return """<input type="text" name="%s:string" value="%s" size="%s">"""  % (region.id, value, cols)

Globals.InitializeClass(StringPropertyStorage)




class TextPropertyStorage(StringPropertyStorage):
    """A TextPropertyStorage Object """

    property_type = 'text'
    property_cols=40
    property_rows=10
    storage_type="TextPropertyStorage"
    value_type='text'

    _default_property_config=HTML("""\
        <dtml-with "region">
        <table width="100%">
          <tr><td colspan="2" class="form-label">Mode</td></tr>
          <tr>
            <td>
              <input type="hidden" name="deletable:int:missing" value="0">
              <input type="checkbox" name="deletable:int" value="1" <dtml-if "'d' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Deletable</td>
          </tr>
          <tr>
            <td>
              <input type="hidden" name="writtable:int:missing" value="0">
              <input type="checkbox" name="writtable:int" value="1" <dtml-if "'w' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Writable</td>
          </tr>
        </table>
        <table width="100%">
          <tr><td colspan="2" class="form-label">Size</td></tr>
          <tr>
            <td class="form-label" width="100">Columns</td>
            <td><input type="text" name="property_cols:int" value="&dtml.missing-property_cols;"></td>
          </tr>
          <tr>
            <td class="form-label" width="100">Rows</td>
            <td><input type="text" name="property_rows:int" value="&dtml.missing-property_rows;"></td>
          </tr>
        </dtml-with region>
        """, globals())


    def render_html_view(self, editablecontainer, region):
        req=getattr(editablecontainer ,'REQUEST', None)
        rows=getattr(region,'property_rows', self.property_rows)
        cols=getattr(region,'property_cols', self.property_cols)
        if req is not None and req.has_key('dtpref_cols') and req.has_key('dtpref_rows'):
            dtpref_rows=req.dtpref_rows
            if type(dtpref_rows) is StringType and dtpref_rows[-1]=='%': dtpref_rows=dtpref_rows[:-1]
            rows=max(atoi(dtpref_rows),20)
            dtpref_cols=req.dtpref_cols
            if type(dtpref_cols) is StringType and dtpref_cols[-1]=='%': dtpref_cols=dtpref_cols[:-1]
            cols=max(atoi(dtpref_cols)-15,20)

        value=self.manage_load(editablecontainer, region, region.getDefault())

        return """<textarea name="%s:text" rows="%s" cols="%s">%s</textarea>"""  % (region.id, rows, cols, value)

    def manage_save_storage_conf(self, region, REQUEST):
        property_mode=''
        if REQUEST.get('writtable', 0):
            property_mode=property_mode+'w'
        if REQUEST.get('deletable', 0):
            property_mode=property_mode+'d'
        region.property_mode = property_mode

        region.property_cols = REQUEST.get('property_cols',getattr(region,'property_cols', self.property_cols))
        region.property_rows = REQUEST.get('property_rows',getattr(region,'property_rows', self.property_rows))



Globals.InitializeClass(TextPropertyStorage)

def initialize(context):
    """Initialize PropertyStorage"""

    context.registerEasyStorage(StringPropertyStorage,   'StringPropertyStorage')
    context.registerEasyStorage(TextPropertyStorage,     'TextPropertyStorage')
