# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyFile/EasyFile.py,v 1.1 2004/06/28 10:05:44 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyFile Object

EasyFile.py,v 1.8 2001/09/29 12:00:00 lennart Exp $"""
__version__='$Revision: 1.1 $'[11:-2]


import Globals
from Globals import DTMLFile
from ComputedAttribute import ComputedAttribute
import string
import os
#from App.Common import rfc1123_date
from webdav.common import rfc1123_date
from DateTime import DateTime
from OFS.Image import File
from Products.EasyResources.EasyResource import EasyResource
from Products.EasyDublinCore.EasyDublinCore import EasyDublinCore
from Products.EasyCore.ManageOptionBase import ManageOptionBase
from Products.EasyCore.EasyCorePermissions import *
from Products.EasyCore import getIconpath, renderIconTag
from Products.EasyCore.Utils.Utils import get_filename
from Products.EasyCore.VirtualPath import VirtualPathBase
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_inner, aq_parent, aq_chain
from webdav.WriteLockInterface import WriteLockInterface
from ZPublisher import HTTPRangeSupport
from zLOG import LOG, WARNING
from webdav.Lockable import ResourceLockedError
from OFS.content_types import guess_content_type
from AccessControl.Permissions import access_contents_information, view
from types import TupleType, ListType

AMBIGUOUS_CONTENT_TYPES=(
      'application/octet-stream'
    , 'text/x-unknown-content-type'
    )

class EasyFile(ManageOptionBase, EasyResource, EasyDublinCore, VirtualPathBase, File):
    """A EasyFile Object """

    meta_type = 'EasyFile'
    fixed_content_type=0

    security = ClassSecurityInfo()

    __catalogs__=('EasyResources',)

    __implements__ = (WriteLockInterface, HTTPRangeSupport.HTTPRangeInterface)

    manage_options_left = ( {'label':'Edit',            'action': 'manage_main' },
                          )

    manage_options_mid = File.manage_options[2:3]+(
                           {'label':'View/Download',   'action': ''            },)

    manage_options_right = File.manage_options[3:]

    #manage_options = manage_options_file + manage_options_inherited
    use_manage_options = ((manage_options_left, manage_options_mid, manage_options_right),)

    def __init__(self, id, file
                    , title=''
                    , category=()
                    , subject=()
                    , description=''
                    , language=''
                    , content_type=''
                    , precondition=''
                ):
        EasyDublinCore.__init__(self, category=category
                                    , subject=subject
                                    , description=description
                                    , language=language
                                )
        File.__init__(self, id, title, file, content_type=content_type, precondition=precondition)

    def _isPrincipiaFolderish(self):
        return getattr(self, 'show_files_in_tree', 0)
    isPrincipiaFolderish= ComputedAttribute(_isPrincipiaFolderish,1)

    def getIcon(self):
        if getattr(self, 'disable_dynamic_icons', 0):
            return self.icon
        return getIconpath(self.content_type, self.icon)

    security.declareProtected(access_contents_information, 'renderIconTag')
    def renderIconTag(self, *args, **kw):
        return apply(renderIconTag, (self.content_type, self.icon)+args, kw)

    security.declareProtected(access_contents_information, 'hasOriginal')
    def hasOriginal(self):
        return 1

    security.declareProtected(access_contents_information, 'hasPreview')
    def hasPreview(self):
        return 0

    #MetaDate API

    security.declareProtected(access_contents_information, 'getContentType')
    def getContentType(self):
        return self.content_type


    #
    def canChangeContentType(self):
        return not self.fixed_content_type

    def manage_afterClone(self, item):
        File.manage_afterClone(self, item)

    def manage_afterAdd(self, item, container):
        if self.canChangeContentType():
            self.updateContentType()
        File.manage_afterAdd(self, item, container)
        self.index_object()

    def manage_beforeDelete(self, item, container):
        File.manage_beforeDelete(self, item, container)
        self.unindex_object()

    security.declareProtected(EasyFileChange, 'manage_main')
    manage_main = DTMLFile('dtml/manage_editEasyFileForm', globals())

    def setContentType(self, content_type=None):
        if content_type is not None:
            self.content_type=str(content_type)
            #if content_type in FIXED_CONTENT_TYPES:
            #    self.fixed_content_type=1

    def updateContentType(self):
        others=getattr(self, 'ambiguous_content_types', [])
        if type(others) not in (TupleType, ListType): others=[others]
        if self.content_type in AMBIGUOUS_CONTENT_TYPES or\
                self.content_type in others:
            content_type, content_enc=guess_content_type(self.getId(), self.data[:100], None)
            if content_type is not None:
                self.content_type=content_type

    security.declareProtected(EasyFileChange, 'manage_edit')
    def manage_edit(self, title, content_type=None, precondition='',
                    filedata=None, REQUEST=None):
        """\
        Changes the title and content type attributes of the File or Image.
        """
        if self.wl_isLocked():
            raise ResourceLockedError, "File is locked via WebDAV"

        self.title=str(title)
        self.setContentType(content_type)
        if precondition:
            self.precondition=str(precondition)
        elif self.precondition: del self.precondition
        if filedata is not None:
            self.update_data(filedata, content_type, len(filedata))
        else:
            self.ZCacheable_invalidate()
        if REQUEST:
            message="Saved changes."
            return self.manage_main(self,REQUEST,manage_tabs_message=message)

    security.declareProtected(EasyFileChange, 'manage_upload')
    def manage_upload(self,file='',REQUEST=None):
        """\
        Replaces the current contents of the File or Image object with file.

        The file or images contents are replaced with the contents of 'file'.
        """
        if self.wl_isLocked():
            raise ResourceLockedError, "File is locked via WebDAV"
        data, size = self._read_data(file)
        content_type=REQUEST.get('content-type', None)
        if content_type is None:
            content_type=self._get_content_type(file, data, self.__name__,
                                            'application/octet-stream')
        self.update_data(data, content_type, size)
        self.setContentType(content_type)

        if REQUEST:
            message="Saved changes."
            return self.manage_main(self,REQUEST,manage_tabs_message=message)

    def PUT(self, REQUEST, RESPONSE):
        """\
        Handle HTTP PUT requests
        """
        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        file=REQUEST['BODYFILE']
        data, size = self._read_data(file)
        content_type=REQUEST.get_header('content-type', None)
        if content_type is None:
            content_type=self._get_content_type(file, data, self.__name__, self.content_type)
        self.update_data(data, content_type, size)
        self.setContentType(content_type)
        RESPONSE.setStatus(204)
        return RESPONSE



    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """\

        """
        if hasattr(aq_base(self),'filename'):
            # Upgrading from 1.4's ExtFile based to 1.5's File based EasyImage
            parent = self.aq_parent
            filename = get_filename( ['var','reposit'], base.filename)
            file = open(filename, 'rb')
            data = file.read()
            file.close()
            manage_addEasyFile=parent.manage_addProduct['EasyFile'].manage_addEasyFile
            manage_addEasyFile(parent, 'tmpimg', data, base.title)
            parent._delObject(self.id)
            parent.manage_renameObject('tmpimg',base.id)
            changed=1

        if getattr(self,'_object_version', None)!='1.7': self._object_version = '1.7'
        if REQUEST is not None:
            return "<h1>Done</h1>"
        return changed

Globals.InitializeClass(EasyFile)

