# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyResources/EasyResources.py,v 1.2 2004/06/29 14:14:16 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__='''EasyResources product: EasyResources class.'''
__version__='1.7.5'

import Globals
from string import join, split, lower

from Globals import HTML
from Acquisition import aq_base

from ComputedAttribute import ComputedAttribute

from AccessControl import ClassSecurityInfo
from Products.EasyCore import EasyCorePermissions


from Products.EasyCore.ZCatalog.ProtectedCatalog import ProtectedCatalog

from Products.EasyCore.UpgradeSupport.Utils import upgrade_indexes
from Products.EasyCore.VirtualPath import VirtualPathBase
from Products.EasyCore.VirtualPathHelpers import *
from Products.EasyPresentationHelpers.EasyPresentationHelpers import EasyPresentationHelpers
from Products.EasyEditor.EasyPublisher import EasyPublisherBase

from Products.EasyCore.EasyResourcesConf import RESOURCE_CATALOG_ID, EASYIMAGES, EASYFILES, EASYRESOURCES

EASY_VHM='EasyVHM'


DEFAULT_INDEXES = {}
REQUIRED_INDEXES = {
          'id':'FieldIndex'
        #, 'title':'TextIndex'
        , 'bobobase_modification_time':'FieldIndex'
        #, 'summary':'TextIndex'
        , 'path':'PathIndex'
        , 'meta_type':'FieldIndex'

        , 'getVirtualRootPath':'FieldIndex'
        , 'virtualPath':'FieldIndex'
        , 'physicalPath':'FieldIndex'
        , 'virtualParentPath':'FieldIndex'
        , 'physicalParentPath':'FieldIndex'
        , 'category':'KeywordIndex'
        , 'content_type':'FieldIndex'
        , 'subject':'KeywordIndex'
        , 'version':'FieldIndex'
        , 'release':'FieldIndex'
        , 'virtualPathIndex':'PathIndex'
}
DEPRECATED_INDEXES=('id','title','meta_type','bobobase_modification_time','PrincipiaSearchSource','path')

DEFAULT_COLUMNS = ()
REQUIRED_COLUMNS=(
          'id'
        , 'title'
        , 'meta_type'
        , 'bobobase_modification_time'
        #, 'summary'

        , 'icon'
        , 'thumbnail_url'
        , 'thumbnail_width'
        , 'thumbnail_height'
        , 'thumbnail_size'
        , 'thumbnail_content_type'
        , 'prev_url'
        , 'prev_width'
        , 'prev_height'
        , 'prev_size'
        , 'prev_content_type'
        , 'width'
        , 'height'
        , 'size'
        , 'content_type'
        , 'getVirtualRootPath'
        , 'virtualPath'
        , 'physicalPath'
        , 'virtualParentPath'
        , 'physicalParentPath'
        , 'category'
        , 'subject'
        , 'description'
        , 'version'
        , 'release'
        , 'isAnObjectManager'
        , 'hasSubDocuments'
        #is_webviewable
)
DEPRECATED_COLUMNS=('summary', 'bobobase_modification_time', 'meta_type', 'title', 'id')

class ResourceBrain:

    def _getVirtualURLXD(self):
        """\
        Returns a virtualURL.
        If the domain that is being accessed doesn't match
        the domain that the menu item is in the URL is remapped
        to include the protocol, domain name, and port.

        XD stand for Cross Domain
        """
        vhm=self.EasyVHM
        if vhm:
            try:
                item_domain=vhm.get_domain(self.physicalPath)
                if self.domain!=item_domain:
                    return vhm.get_site_root_url(self.physicalPath,1) + self.virtualURL
            except:
                pass
        return path2url(self.virtualPath)
    virtualURLXD = ComputedAttribute(_getVirtualURLXD, 1)

    def isLocalDomain(self):
        """\
        Test if the domain that is being accessed doesn't match
        the domain that the menu item is in.
        """
        vhm = self.EasyVHM
        if vhm:
            try:
                item_domain=vhm.get_domain(self.physicalPath)
                return self.domain==item_domain
            except:
                pass
        return 1

class EasyResources(ProtectedCatalog, VirtualPathBase, EasyPresentationHelpers, EasyPublisherBase):
    """\
    EasyResources object

    """

    security = ClassSecurityInfo()

    meta_type = RESOURCE_CATALOG_ID


    _properties=({'id':'default_preview_create', 'type': 'int'},
                 {'id':'default_preview_height', 'type': 'int'},
                 {'id':'default_preview_width', 'type': 'int'},
                )


    manage_options = ProtectedCatalog.manage_options


    def __init__(self, id, title='', vocab_id=None, container=None):
        ProtectedCatalog.__init__(self, id, title, vocab_id, container)

        self.default_preview_create = 1
        self.default_preview_height = 100
        self.default_preview_width = 100


        for id in DEPRECATED_INDEXES:
            self.delIndex(id, force=1, dont_raise=1)
        for id, type in REQUIRED_INDEXES.items():
            self.addIndex(id, type, undeletable=1)
        for id, type in DEFAULT_INDEXES.items():
            self.addIndex(id, type)

        for id in DEPRECATED_COLUMNS:
            self.delColumn(id, force=1, dont_raise=1)
        for id in REQUIRED_COLUMNS:
            self.addColumn(id, undeletable=1)
        for id in DEFAULT_COLUMNS:
            self.addColumn(id)



    security.declarePrivate('get_resources')
    #usage: get_resources?path:tuple=&path:tuple=Document&path:tuple=test1&spec:list=EasyImage&spec:list=EasyWord&spec:list=EasyPDF
    def get_resources(self, container_path, spec=None, query=None):
        if type(query)!=type({}): query={}
        if type(container_path) != type([]): container_path = [container_path]
        if spec is not None and type(spec) != type([]): spec = [spec]
        query['physicalPath'] = container_path
        if spec:
            query['meta_type'] = spec
        return self.searchResults(query)

    def extractBrain(self, path, pathTreeNodes):
        for brain in pathTreeNodes:
            if brain.physicalPath==path:
                return brain
        raise "PathError", "Path doesn't exist in pathTree"


    security.declarePrivate('renderXmlTreeNode')
    def renderXmlTreeNode(self,node_path,document_path,current_paths,fullmetadata=0,pathTreeNodes=None,easynavigation=None):
        result = ""
        if easynavigation is None:
            easynavigation = self.getEasyNavigation()
        obj=None
        brain=None

        class LocalResourceBrain(ResourceBrain):  pass
        LocalResourceBrain.EasyVHM=getattr(self, EASY_VHM, None)
        LocalResourceBrain.domain=self.getCurrentDomainName()

        try:
            easynavigation._catalog.useBrains(LocalResourceBrain)
            brain = easynavigation.searchResults(physicalPath=[node_path])[0]
        except:
            obj = self.unrestrictedTraverse(node_path, None)

        if not (brain or obj) or node_path==document_path:
            if not (brain or obj): do_this_path = node_path[:-1]
            else:  do_this_path = node_path
            self._catalog.useBrains(LocalResourceBrain)
            resources = self.searchResults(physicalParentPath=[do_this_path], meta_type="EasyImage")
            if easynavigation is not None:
                easynavigation._catalog.useBrains(LocalResourceBrain)
                resources = easynavigation.searchResults(physicalParentPath=[do_this_path]) + resources
        else: resources = []

        hasSubFolders = node_path!=document_path or len(resources)
        isRoot = node_path==current_paths[0]

        resources = list(resources)
        resources.sort(lambda x,y: cmp(lower(x.id),lower(y.id)))

        if brain is not None:
            result = result + xmlBrainProperties(brain, hasSubFolders = hasSubFolders, current_paths=current_paths, object_path=document_path, fullmetadata=fullmetadata, isRoot=isRoot)
        elif obj is not None:
            result = result + xmlObjectProperties(obj, hasSubFolders = hasSubFolders, current_paths=current_paths, object_path=document_path, fullmetadata=fullmetadata, isRoot=isRoot)

        if (brain or obj) and node_path!=document_path:
            renderedTreeNode = self.renderXmlTreeNode(document_path[0:len(node_path)+1],document_path,current_paths,fullmetadata,pathTreeNodes,easynavigation)
            if renderedTreeNode:
                result = result + xmlObjectsStart
                result = result + xmlObjectStart
                result = result + renderedTreeNode
                result = result + xmlObjectStop
                result = result + xmlObjectsStop
        elif (brain or obj) and len(resources) > 0:
            result = result + xmlObjectsStart
            for resourcebrain in resources:
                if resourcebrain.physicalPath in current_paths or resourcebrain.physicalParentPath == document_path:
                    result = result + xmlObjectStart
                    result = result + xmlResourcesSnippet(resourcebrain, current_paths=current_paths, object_path=document_path, fullmetadata=fullmetadata, EASYIMAGES=EASYIMAGES, EASYRESOURCES=EASYRESOURCES, EASYFILES=EASYFILES)
                    result = result + xmlObjectStop
            result = result + xmlObjectsStop
        else:
            notfirst=0
            for resourcebrain in resources:
                if notfirst:
                    result = result + xmlObjectStop
                    result = result + xmlObjectStart
                result = result + xmlResourcesSnippet(resourcebrain, current_paths=current_paths, object_path=document_path, fullmetadata=fullmetadata, EASYIMAGES=EASYIMAGES, EASYRESOURCES=EASYRESOURCES, EASYFILES=EASYFILES)
                notfirst=1
        return result

    security.declareProtected(EasyCorePermissions.AccessContentsInformation, 'xmlSubObjects')
    def xmlSubObjects(self, document_url, fullmetadata=0, REQUEST=None, RESPONSE=None):
        """Returns objects as XML"""
        physpath = self.getPhysicalParentPath()
        current_paths = [physpath]

        class LocalResourceBrain(ResourceBrain):  pass
        LocalResourceBrain.EasyVHM=getattr(self, EASY_VHM, None)
        LocalResourceBrain.domain=self.getCurrentDomainName()

        document_path = self.virtualURLToPhysicalPath(document_url)
        for id in document_path[len(physpath):]:
            physpath = physpath + (id,)
            current_paths.append(physpath)

        top_document = self.physicalParentPath
        try:
            easynavigation = self.getEasyNavigation()
            easynavigation._catalog.useBrains(LocalResourceBrain)
            pathTreeNodes = easynavigation.searchResults(physicalPath=[top_document])
        except:
            easynavigation = None
            pathTreeNodes = None

        pathTreeNodes = list(pathTreeNodes)
        pathTreeNodes.sort(lambda x,y: cmp(lower(x.id),lower(y.id)))


        result = xmlTreeStart
        renderedTreeNode = self.renderXmlTreeNode(top_document,document_path,current_paths,fullmetadata,pathTreeNodes,easynavigation)
        if renderedTreeNode:
            result = result + xmlObjectsStart
            result = result + xmlObjectStart
            result = result + renderedTreeNode
            result = result + xmlObjectStop
            result = result + xmlObjectsStop
        result = result + xmlTreeStop
        RESPONSE.setHeader('Content-Type', 'text/XML')
        return result

    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        #base=aq_base(self)
        ob_ver=getattr(self,'_object_version', 0.0)
        try: ob_ver=float(ob_ver)
        except: ob_ver=1.7

        changed, messages=upgrade_indexes(self, changed, DEFAULT_INDEXES, DEPRECATED_INDEXES, REQUIRED_INDEXES, DEFAULT_COLUMNS, DEPRECATED_COLUMNS, REQUIRED_COLUMNS)

        self._object_version = 1.75

        if REQUEST is not None:
            return "<h1>%s</h1>%s"%(changed and 'Changed' or 'Not Changed', ''.join(map(lambda x: '<p>%s</p>'%x, messages)),)
        return changed, messages


Globals.InitializeClass(EasyResources)


xmlTreeStart = """<?xml version="1.0" encoding="ISO-8859-1"?><body><tree>"""
xmlTreeStop =  """</tree></body>"""
xmlObjectsStart = """<objects>"""
xmlObjectsStop =  """</objects>"""
xmlObjectStart = """<object>"""
xmlObjectStop =  """</object>"""

xmlPathProperties = HTML( """
<id><dtml-var "physicalPath[-1]"></id>
<display_id><dtml-var "physicalPath[-1]"></display_id>
<url><dtml-var virtualURLXD></url>
<title><dtml-var "virtualPath[-1]"></title>
<icon><dtml-var icon></icon>
<meta_type><dtml-var meta_type></meta_type>
<hasSubDocuments><dtml-var hasSubFolders missing="0"></hasSubDocuments>
<isAnObjectManager><dtml-var isAnObjectManager missing="0"></isAnObjectManager>
<isRoot><dtml-var isRoot missing="0"></isRoot>
<dtml-if fullmetadata>
    <title><![CDATA[<dtml-var "physicalPath[-1]">]]></title>
    <navigation_title><![CDATA[<dtml-var "physicalPath[-1]">]]></navigation_title>
    <parentURL><dtml-var "path2root_safe_url(physicalPath[:-1])"></parentURL >
    <effective_date></effective_date>
    <expiration_date></expiration_date>
    <author>unknown</author>
    <last_modified_by>unknown</last_modified_by>
    <workflowstate><dtml-var getWorkflowState></workflowstate>
    <workflowchangedby><dtml-var getWorkflowStateLastChangedBy></workflowchangedby>
    <workflowchangedate><dtml-var getWorkflowStateLastChangedDate></workflowchangedate>
</dtml-if>
<iscurrent><dtml-if "physicalPath == object_path">1<dtml-else>0</dtml-if></iscurrent>
<inpath><dtml-if "physicalPath in current_paths">1<dtml-else>0</dtml-if></inpath>
""", globals())


xmlBrainProperties = HTML( """
<id><dtml-var id></id>
<display_id><dtml-var "isLocalDomain() and id or virtualURLXD"></display_id>
<url><dtml-var virtualURLXD></url>
<title><![CDATA[<dtml-if title><dtml-var title><dtml-else><dtml-if navigation_title><dtml-var navigation_title><dtml-else><dtml-var id></dtml-if></dtml-if>]]></title>
<icon><dtml-var icon></icon>
<meta_type><dtml-var meta_type></meta_type>
<hasSubDocuments><dtml-var "hasSubDocuments or hasSubFolders" missing="0"></hasSubDocuments>
<isAnObjectManager><dtml-var isAnObjectManager missing="0"></isAnObjectManager>
<isRoot><dtml-var isRoot missing="0"></isRoot>
<dtml-if fullmetadata>
    <navigation_title><![CDATA[<dtml-if navigation_title><dtml-var navigation_title><dtml-else><dtml-var id></dtml-if>]]></navigation_title>
    <parentURL><dtml-if "virtualParentPath is not _.None"><dtml-var "path2url(virtualParentPath)"></dtml-if></parentURL >
    <effective_date><dtml-var effective_date></effective_date>
    <expiration_date><dtml-var expiration_date></expiration_date>
    <author><dtml-var author></author>
    <last_modified_by><dtml-var last_modified_by></last_modified_by>
    <workflowstate><dtml-var getWorkflowState></workflowstate>
    <workflowchangedby><dtml-var getWorkflowStateLastChangedBy></workflowchangedby>
    <workflowchangedate><dtml-var getWorkflowStateLastChangedDate></workflowchangedate>
</dtml-if>
<iscurrent><dtml-if "physicalPath == object_path">1<dtml-else>0</dtml-if></iscurrent>
<inpath><dtml-if "physicalPath in current_paths">1<dtml-else>0</dtml-if></inpath>
""", globals())

xmlObjectProperties = HTML( """
<id><dtml-var id></id>
<display_id><dtml-var "isLocalDomain() and id or virtualURLXD"></display_id>
<url><dtml-var virtualURLXD></url>
<title><![CDATA[<dtml-if title><dtml-var title><dtml-else><dtml-if navigation_title><dtml-var navigation_title><dtml-else><dtml-var id></dtml-if></dtml-if>]]></title>
<icon><dtml-var icon></icon>
<meta_type><dtml-var meta_type></meta_type>
<hasSubDocuments><dtml-var "hasSubDocuments() or hasSubFolders()" missing="0"></hasSubDocuments>
<isAnObjectManager><dtml-var isAnObjectManager missing="0"></isAnObjectManager>
<isRoot><dtml-var isRoot missing="0"></isRoot>
<dtml-if fullmetadata>
    <navigation_title><![CDATA[<dtml-if navigation_title><dtml-var navigation_title><dtml-else><dtml-var id></dtml-if>]]></navigation_title>
    <parentURL><dtml-if "virtualParentPath is not _.None"><dtml-var "path2root_safe_url(virtualParentPath)"></dtml-if></parentURL >
    <effective_date><dtml-var effective_date></effective_date>
    <expiration_date><dtml-var expiration_date></expiration_date>
    <author><dtml-var author></author>
    <last_modified_by><dtml-var last_modified_by></last_modified_by>
    <workflowstate><dtml-var getWorkflowState></workflowstate>
    <workflowchangedby><dtml-var getWorkflowStateLastChangedBy></workflowchangedby>
    <workflowchangedate><dtml-var getWorkflowStateLastChangedDate></workflowchangedate>
</dtml-if>
<iscurrent><dtml-if "physicalPath == object_path">1<dtml-else>0</dtml-if></iscurrent>
<inpath><dtml-if "physicalPath in current_paths">1<dtml-else>0</dtml-if></inpath>
""", globals())

#xmlResourcesSnippet
xmlResourcesSnippet = HTML( """
<id><dtml-var id></id>
<display_id><dtml-var "isLocalDomain() and id or virtualURLXD"></display_id>
<url><dtml-var virtualURLXD></url>
<title><![CDATA[<dtml-var title>]]></title>
<icon><dtml-var icon></icon>
<meta_type><dtml-var meta_type></meta_type>
<hasSubDocuments><dtml-var hasSubDocuments missing="0"></hasSubDocuments>
<isAnObjectManager><dtml-var isAnObjectManager missing="0"></isAnObjectManager>
<isAResource><dtml-if "meta_type in EASYRESOURCES">1<dtml-else>0</dtml-if></isAResource>
<dtml-if "meta_type in EASYIMAGES">
    <width><dtml-var width></width>
    <height><dtml-var height></height>
    <size><dtml-var size></size>
    <content_type><dtml-var content_type></content_type>
    <preview_url><dtml-var prev_url></preview_url>
    <preview_width><dtml-var prev_width></preview_width>
    <preview_height><dtml-var prev_height></preview_height>
    <preview_size><dtml-var prev_size></preview_size>
    <preview_content_type><dtml-var prev_content_type></preview_content_type>
</dtml-if>
<dtml-if fullmetadata>
    <effective_date><dtml-var effective_date></effective_date>
    <expiration_date><dtml-var expiration_date></expiration_date>
    <author><dtml-var author></author>
    <last_modified_by><dtml-var last_modified_by></last_modified_by>
    <workflowstate><dtml-var getWorkflowState></workflowstate>
    <workflowchangedby><dtml-var getWorkflowStateLastChangedBy></workflowchangedby>
    <workflowchangedate><dtml-var getWorkflowStateLastChangedDate></workflowchangedate>
</dtml-if>
<iscurrent><dtml-if "physicalPath == object_path">1<dtml-else>0</dtml-if></iscurrent>
<inpath><dtml-if "physicalPath in current_paths">1<dtml-else>0</dtml-if></inpath>
""", globals())



def manage_addEasyResources(self, REQUEST=None):
    """Add a EasyResources object"""
    c=EasyResources(RESOURCE_CATALOG_ID, '', None, self)
    self._setObject(RESOURCE_CATALOG_ID, c)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)




def initialize(context):
    """Initialize StyleManagers Product"""

    context.registerClass(
        EasyResources,
        permission=EasyCorePermissions.EasyNavigationServiceAdd,
        icon='icons/EasyResources.gif',
        constructors = ( manage_addEasyResources, manage_addEasyResources),
        meta_type = RESOURCE_CATALOG_ID,
        category='Easy Publisher Management',
    )




