# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyWorkflow/EasyDiff.py,v 1.1 2004/06/28 10:05:49 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyPublishSync mixin class"""
__version__='$Revision: 1.1 $'[11:-2]

import Globals
from Acquisition import Implicit, aq_base, aq_get
from types import DictType, TupleType, ListType, StringType
from Products.EasyCore.VirtualPathHelpers import path2url, url2path, comparePaths, absolute2relative
from Products.EasyCore.EasyEditorConf import EASYEDITORID

from Products.EasyCore.Utils.DebugUtils import MLOG_TAB


from EasyObjectDiff import ObjectInfo, ObjectDiff, MISSINGOBJECT
from EasyPropertyDiff import PropertySheetInfo, PropertySheetDiff, MISSINGPROPERTYSHEETINFO
from EasyRegionDiff import RegionInfo, RegionDiff, MISSINGREGIONINFO
from EasyNavigationDiff import NavigationInfo, NavigationDiff, MISSINGNAVIGATIONINFO

_marker=[]

class MasterDataCopy(Globals.Persistent, Implicit):

    _objects_ids={}
    _remote_path=None
    _properysheets={}
    _properties=None
    _navigation=[]
    _regions={}

    def __init__(self):
        self._objects_ids={}
        self._remote_path=None
        self._properysheets={}
        self._properties=None
        self._navigation=[]
        self._regions={}

    #def copyData(self, remote_obj):
    #    self.copyProperties(remote_obj)
    #    self.copyRegions(remote_obj)
    #    self.copyPath(remote_obj)
    #    self.copyNavigation(remote_obj)

    def parsePropertysheets(self, local_obj, exclude=(),diff_sheets=None):
        local_sheets={}
        if type(diff_sheets)!=type({}): diff_sheets={}
        propertysheets = getattr(local_obj, 'propertysheets', None)
        if propertysheets is not None:
            for key, value in propertysheets.items():
                if key not in exclude:
                    local_sheets[key]=PropertySheetInfo(key, value, local_obj)
                    diff_sheets[key]=None
        return local_sheets

    def copyProperties(self
                , remote_obj
                , exclude='webdav'
                , ignore=('creation_date','contributors','language')
            ):
        if type(exclude)==StringType: exclude=(exclude,)
        self._properysheets=self.parsePropertysheets(remote_obj,exclude=exclude)


    def getRegionItems(self, object):
        editor = aq_base(getattr(object,EASYEDITORID))
        templateid = object.getTemplateId()
        template = editor.getTemplate(templateid)
        return template.regionItems()

    def copyRegions(self, remote_obj):
        regions={}
        editor = aq_base(getattr(remote_obj,EASYEDITORID))
        templateid = remote_obj.getTemplateId()
        template = editor.getTemplate(templateid)
        contentregions= template.regionItems()
        ###MLOG_TAB("remote_obj.id",remote_obj.id)
        ###MLOG_TAB("editor.physicalURL",editor.getPhysicalPath())
        ###MLOG_TAB("templateid",templateid)
        ###MLOG_TAB("contentregions",contentregions)
        for id, region in contentregions:
            reg = region.manage_load(remote_obj, default=_marker)
            if reg is not _marker:
                #region.manage_copy(remote_obj, self)
                meta_type = getattr(reg, 'meta_type', None)
                regions[id] = RegionInfo(id, region, remote_obj)
        self._regions = regions

    def copyPath(self, remote_obj):
        self._remote_path=remote_obj.getPhysicalPath()

    def copyNavigation(self, remote_obj):
        navigation=[]
        remote_navigation_brains=remote_obj.getMenuItems()
        for brain in remote_navigation_brains:
            navigation.append(NavigationInfo(brain, remote_obj.physicalPath))
        self._navigation=navigation


    def masterHasMoved(self, remote_obj):
        # Returns 1 if the object has moved, null if not applicable
        if self._remote_path is None: return None
        if comparePaths(self._remote_path,remote_obj.getPhysicalPath()):
            return 0
        return 1

    def hasMasterCopy(self):
        return self._remote_path is not None

    def getDiffObjects(self, local, spec=None):
        remote_obj=local.getMasterDocument()
        local_objects=local.objectItems(spec=spec)
        objects_diff={}
        local_objs={}
        remote_objs={}
        old_remote_objs={}
        hardlinks={}
        #Local
        for id, ob in local_objects:
            local_objs[id] = ObjectInfo(ob)
            objects_diff[id]=None
            ishard = getattr(ob, 'isHardLinked', 0)
        #Remote
        if remote_obj is not None:
            remote_objects=remote_obj.objectItems(spec=spec)
            for id, ob in remote_objects:
                remote_objs[id] = ObjectInfo(ob)
                objects_diff[id]=None
        #Old Remote
        old_remote_objs = self._objects_ids or {}
        for id in old_remote_objs.keys():
            objects_diff[id]=None
        for id in objects_diff.keys():
            objects_diff[id]=ObjectDiff(id
                                , local_objs.get(id, MISSINGOBJECT)
                                , remote_objs.get(id, MISSINGOBJECT)
                                , old_remote_objs.get(id, MISSINGOBJECT)
                              )
        return objects_diff.items()

    def getDiffPropertySheets(self, local_obj, exclude='webdav'):
        if type(exclude)==StringType: exclude=(exclude,)
        diff_sheets={}

        local_sheets=self.parsePropertysheets(local_obj
                            , diff_sheets=diff_sheets
                            , exclude=exclude
                        )
        remote_obj=local_obj.getMasterDocument()
        remote_sheets=self.parsePropertysheets(remote_obj
                            , diff_sheets=diff_sheets
                            , exclude=exclude
                        )
        for id in self._properysheets.keys(): diff_sheets[id]=None

        for id in diff_sheets.keys():
            diff_sheets[id]=PropertySheetDiff(id
                                , local_sheets.get(id, MISSINGPROPERTYSHEETINFO)
                                , remote_sheets.get(id, MISSINGPROPERTYSHEETINFO)
                                , self._properysheets.get(id, MISSINGPROPERTYSHEETINFO)
                              )
        return diff_sheets.items()

    def getDiffRegions(self, local_obj, exclude=()):

        diff_regions={}
        local_regions={}
        remote_regions={}
        #Local
        regions=self.getRegionItems(local_obj)
        if regions is not None:
            for key, value in regions:
                if key not in exclude:
                    local_regions[key]=RegionInfo(key, value, local_obj)
                    diff_regions[key]=None
        #Remote

        remote_obj=local_obj.getMasterDocument()
        regions=self.getRegionItems(remote_obj)
        if regions is not None:
            for key, value in regions:
                if key not in exclude:
                    remote_regions[key]=RegionInfo(key, value, remote_obj)
                    diff_regions[key]=None


        #Old Remote
        old_remote_regions=self._regions or {}
        for id in old_remote_regions.keys(): diff_regions[id]=None

        for id in diff_regions.keys():
            diff_regions[id]=RegionDiff(id
                                , local_regions.get(id, MISSINGREGIONINFO)
                                , remote_regions.get(id, MISSINGREGIONINFO)
                                , old_remote_regions.get(id, MISSINGREGIONINFO)
                                , local_obj=local_obj , remote_obj=remote_obj
                              )
        return diff_regions.items()


    def getDiffNavigation(self, local_obj):
        old_order=list(self._navigation)
        max_menu=len(old_order)
        local_order=[]
        remote_order=[]
        local_navigation_brains=self.getMenuItems()
        remote_obj=local_obj.getMasterDocument()
        remote_navigation_brains=remote_obj.getMenuItems()
        for brain in remote_navigation_brains:
            remote_order.append(brain)
        max_menu=max(max_menu,len(remote_order))
        for brain in local_navigation_brains:
            local_order.append(brain)
        max_menu=max(max_menu,len(local_order))
        nav_diff=[]
        local_base=local_obj.getPhysicalPath()
        remote_base=remote_obj.getPhysicalPath()
        for order in range(max_menu):
            nav_diff.append((order, NavigationDiff(order
                                , NavigationInfo(len(local_order)>order and local_order[order] or None, local_base)
                                , NavigationInfo(len(remote_order)>order and remote_order[order] or None, remote_base)
                                , len(old_order)>order and old_order[order] or MISSINGNAVIGATIONINFO
                           )))
        return nav_diff
