# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyWorkflow/HistoryTracker.py,v 1.1 2004/06/28 10:05:50 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""HistoryTracker Object"""
__version__='$Revision: 1.1 $'[11:-2]


import Globals
from Persistence import Persistent
from Products.EasyCore.Utils.PersistentList import PersistentList
import DateTime

try: from DocumentTemplate.cDocumentTemplate import DictInstance #, InstanceDict
except: from DocumentTemplate.pDocumentTemplate import DictInstance #, InstanceDict


def setDefaultRoles(permission, roles):
    '''
    Sets the defaults roles for a permission.
    '''
    registered = AccessControl.Permission._registeredPermissions
    if not registered.has_key(permission):
        registered[permission] = 1
        Products.__ac_permissions__=(
            Products.__ac_permissions__+((permission,(),roles),))
        mangled = AccessControl.Permission.pname(permission)
        setattr(Globals.ApplicationDefaultPermissions, mangled, roles)

class HistoryException(Exception):
    pass


class HistoryTracker(Persistent):
    """The HistoryTracker is used to persistently hold a history of events"""

    def __init__(self, **kw):
        """The keys you initialize the history tracker with will be required in all other entrys"""
        self._history = PersistentList()
        self._required = ()
        if kw: apply(self.addHistoryEntry, (), kw)


    def addHistoryEntry(self, **kw):
        # First check that all required fields are included:
        if not self._required: self._required=kw.keys()
        for field in self._required:
            if not kw.has_key(field):
                raise HistoryException('Required field was not given')
        self._history.append(kw)

    def getLastEntry(self):
        if len(self._history) == 0:
            return {
                'Comment':'No entry',
                'User':'Unknown',
                'Time':DateTime.now(),
                'OldState':'Draft',
                'State':'Draft',
                'OldAuthor':'Unknown',
                'Author':'Unknown'
            }
        return self._history[-1]

    def getLastEntryField(self, field, default='_fail'):
        item = self.getLastEntry()
        if item.has_key(field):
            return item[field]
        if default == '_fail':
            if field in self._required:
                raise HistoryException('No history was found, or required field was not in last record.')
            return None
        return default

    def getValues(self):
        items=[]
        for item in self._history:
            item=DictInstance(item)
            item.required=self._required
            items.append(item)
        return items

