#####################################################################
#
# Xron -- Zope Scheduled Event Product
# Copyright (C) 2000 Loren Stafford
#
# Derived from the ZScheduler product version 0.0.7 in accord
# with the terms of the ZScheduler's license.
#
# Based on code by Martijn Pieters, (c) 1999 Antraciet B.V.
# Used with explicit permission.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
#         Free Software Foundation, Inc.,
#         59 Temple Place - Suite 330,
#         Boston, MA    02111-1307, USA.
#
#####################################################################
""" Scheduled DTML Methods for Xron """

from Globals import DTMLFile, InitializeClass
from DateTime import DateTime
from AccessControl import ClassSecurityInfo
from urllib import quote
import sys, string

from OFS.Folder import Folder
from Products.Xron.XronBase import XronBase

from Products.EasyCore import MLOG_TAB



def recalcMinutes2Days(minutes):
    return minutes*60*24


class XronPop3Client(XronBase, Folder):

    meta_type = 'Xron POP3 Client'
    security = ClassSecurityInfo()

    def __init__(self, id, title = ''
                , periodyears=0
                , periodmonths=0
                , perioddays=0
                , periodhours=0
                , periodminutes=0
                , periodweeks=0
                , start=None
                , host = 'localhost', port=110
                , user=None, password=None):
        XronBase.__init__(self, id, title
                        , periodyears=periodyears
                        , periodmonths=periodmonths
                        , perioddays=perioddays
                        , periodhours=periodhours
                        , periodminutes=periodminutes
                        , periodweeks=periodweeks
                        , start=start)
        self.host=host
        self.port=port
        self._user=user
        self._password=password


    def do_trigger(self, **kw):
        print "XronPop3Client", self.next_update


    def getUserName(self):
        return self._user

    def getPasswordDummy(self):
        try: size=len(self._password)
        except: size=8
        return '*' * size

    security.declareProtected('Manage Xron Pop3 Client', 'manage_main')
    manage_main=DTMLFile('dtml/manage_pop3_client', globals())

    security.declareProtected('Manage Xron Pop3 Client', 'manage_edit')
    def manage_edit(self, title
                    , host=None, port=None
                    , user=None, password=None
                    ,REQUEST=None):
        """\
        Manage POP3 configuration
        """
        message=''
        self.title=title
        if host: self.host=host
        if port: self.port=port
        if user: self._user=user
        if password: self._password=password

        if REQUEST:
            message="Content has been changed."
            return self.manage_main(self,REQUEST,manage_tabs_message=message)

InitializeClass(XronPop3Client)


manage_addXronPop3ClientForm = DTMLFile('dtml/manage_addXronPop3ClientForm', globals())

def manage_addXronPop3Client(self, id, title = ''
                            , periodyears=0
                            , periodmonths=0
                            , perioddays=0
                            , periodhours=0
                            , periodminutes=0
                            , periodweeks=0
                            , start=None
                            , host = '', port = 110
                            , user = None, password = None
                            , REQUEST=None, submit=None):
    """\
    Add an XronPop3Client object.
    """

    event=XronPop3Client( id, title
                            , periodyears=periodyears
                            , periodmonths=periodmonths
                            , perioddays=perioddays
                            , periodhours=periodhours
                            , periodminutes=periodminutes
                            , periodweeks=periodweeks
                            , start=start
                            , host = host, port=port
                            , user=user, password=password)
    id=self._setObject(id, event)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')

