# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/ZMIPatch/OFS_PropertyManager/__init__.py,v 1.3 2004/07/01 10:46:37 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__ = """OFS_PropertyManager Monkey Patch."""
__version__='$Revision: 1.3 $'[11:-2]

import re
from types import TupleType
from zLOG import LOG, INFO, BLATHER
from App.version_txt import getZopeVersion
from Globals import InitializeClass, DTMLFile
from AccessControl import ClassSecurityInfo
from Products.EasyCore.BodyTabs import InitializeBodyTabs, UpdateTab
from Products.EasyCore import MLOG_TAB
from types import StringType

from AccessControl.Permissions import manage_properties
from OFS.PropertyManager import PropertyManager


#LOG('ZMIPatch', BLATHER, 'ZMI Patch:OFS.PropertyManager', """Hotfixing: OFS PropertyManager: , Enable Labels in the property ManagementView.\n\t\tJohan Carlsson, 2002-09-19, version Easy Publisher 1.8\n""")



PropertyManager.security = ClassSecurityInfo()


def propertyLabel(self, id):
    """Return a label for the given property id
    """
    prop = filter(lambda i, s=self, id=id: i['id']==id, self._properties)
    if len(prop)==0: return id
    return prop[0].get('label', id)

PropertyManager.propertyLabel = propertyLabel


SPLITTER_RE=re.compile(r'''optgroup[ ]*:[ ]*"(.+)"|optgroup[ ]*:[ ]*(.+)|keyvalue[ ]*:[ ]*("[^";]+")[ ]*;[ ]*("[^";]+")|keyvalue[ ]*:[ ]*("[^";]+")[ ]*;[ ]*([^;]+)|keyvalue[ ]*:[ ]*([^;]+)[ ]*;[ ]*("[^";]+")|keyvalue[ ]*:[ ]*([^;]+)[ ]*;[ ]*([^";]+)|(.+)''', re.I)
def parseSelectVariable(self, select_variable=[], splitter=SPLITTER_RE):
    """\
    Parse the select_variable for key:value entries.
    Keys and values can be seperated with : splitted value are striped.
    Keys and values can be escaped with " or '.

    Returns key:value items.
    """
    if type(select_variable) == StringType:
        select_variable=getattr(self, select_variable, [])
    if select_variable is None:
        select_variable=[]
    items=[]
    for value in select_variable:
        if type(value) is TupleType and len(value)>1:
            if len(value)==2:
                items.append({'key':value[0],'value':value[1],'optgroup':''})
            else:
                items.append({'key':value[0],'value':value[1],'optgroup':value[2]})
        else:
            matches = splitter.findall(value)
            if len(matches)>0:
                opt1, opt2, v1,k1, v2,k2, v3,k3, v4,k4, kv5 = matches[0]
                if k1 and v1:
                    items.append({'key':k1,'value':v1,'optgroup':''})
                elif k2 and v2:
                    items.append({'key':k2,'value':v2,'optgroup':''})
                elif k3 and v3:
                    items.append({'key':k3,'value':v3,'optgroup':''})
                elif k4 and v4:
                    items.append({'key':k4,'value':v4,'optgroup':''})
                elif kv5:
                    items.append({'key':kv5,'value':kv5,'optgroup':''})
                elif opt1:
                    items.append({'key':'','value':'','optgroup':opt1})
                elif opt2:
                    items.append({'key':'','value':'','optgroup':opt2})
            else:
                items.append({'key':matches,'value':matches,'optgroup':''})
    return items

PropertyManager.parseSelectVariable = parseSelectVariable
PropertyManager.parseSelectVariable__roles__=None

zope_version=getZopeVersion()

if zope_version[0:3] >= (2,7,0) and zope_version[0:3] < (2,8,0):
    PropertyManager.manage_propertiesForm=DTMLFile('dtml/Zope270/properties', globals()
                            , property_extensible_schema__=1
                            , management_view='Properties'
                            )
    PropertyManager.manage_propertyTypeForm=DTMLFile('dtml/Zope270/propertyType', globals()
                            , management_view='Properties'
                            )

elif zope_version[0:3] >= (2,6,1) and zope_version[0:3] < (2,7,0):

    PropertyManager.manage_propertiesForm=DTMLFile('dtml/Zope261/properties', globals()
                            , property_extensible_schema__=1
                            , management_view='Properties'
                            )
    PropertyManager.manage_propertyTypeForm=DTMLFile('dtml/Zope261/propertyType', globals()
                            , management_view='Properties'
                            )

elif zope_version[0:3] >= (2,5,1) and zope_version[0:3] < (2,6,1):
    PropertyManager.manage_propertiesForm=DTMLFile('dtml/Zope251/properties', globals()
                            , property_extensible_schema__=1
                            , management_view='Properties'
                            )
    PropertyManager.manage_propertyTypeForm=DTMLFile('dtml/Zope251/propertyType', globals()
                            , management_view='Properties'
                            )
else:
    PropertyManager.manage_propertiesForm=DTMLFile('dtml/Zope270/properties', globals()
                            , property_extensible_schema__=1
                            , management_view='Properties'
                            )
    PropertyManager.manage_propertyTypeForm=DTMLFile('dtml/Zope270/propertyType', globals()
                            , management_view='Properties'
                            )


def manage_moveDown(self, property_id, REQUEST=None):
    """Moves a property down in the proprerty sheet"""
    # XXX FIXME : trap bad property_id and meaningful new position
    newprops = []
    i = 0
    for p in self._properties:
        if p['id'] == property_id:
            position = i
            moved = p
        else:
            newprops.append(p)
            i += 1
    newprops.insert(position + 1, moved)
    self._properties = tuple(newprops)
    if REQUEST is not None:
        message = "Property %s moved down" % property_id
        return self.manage_propertiesForm(self,REQUEST, manage_tabs_message=message)
    return
# XXX FIXME: This does not work (Anonymous can use 'manage_moveDown')
PropertyManager.security.declareProtected(manage_properties, 'manage_moveDown')
PropertyManager.manage_moveDown = manage_moveDown

def manage_moveUp(self, property_id, REQUEST=None):
    """Moves a property up in the proprerty sheet"""
    # XXX FIXME : trap bad property_id and meaningful new position
    newprops = []
    for p in self._properties:
        if p['id'] == property_id:
            newprops.insert(len(newprops) - 1, p)
        else:
            newprops.append(p)
    self._properties = tuple(newprops)
    if REQUEST is not None:
        message = "Property %s moved up" % property_id
        return self.manage_propertiesForm(self,REQUEST, manage_tabs_message=message)
    return
PropertyManager.security.declareProtected(manage_properties, 'manage_moveUp')
PropertyManager.manage_moveUp = manage_moveUp

def manage_moveTop(self, property_id, REQUEST=None):
    """Moves a property to the top of a property sheet"""
    # XXX FIXME : trap bad property_id and meaningful new position
    newprops = []
    for p in self._properties:
        if p['id'] == property_id:
            moved = p
        else:
            newprops.append(p)
    newprops.insert(0, moved)
    self._properties = tuple(newprops)
    if REQUEST is not None:
        message = "Property %s moved to the top" % property_id
        return self.manage_propertiesForm(self,REQUEST, manage_tabs_message=message)
    return
PropertyManager.security.declareProtected(manage_properties, 'manage_moveTop')
PropertyManager.manage_moveTop = manage_moveTop

def manage_moveEnd(self, property_id, REQUEST=None):
    """Moves a property to the end of the property sheet"""
    # XXX FIXME : trap bad property_id and meaningful new position
    newprops = []
    for p in self._properties:
        if p['id'] == property_id:
            moved = p
        else:
            newprops.append(p)
    newprops.append(moved)
    self._properties = tuple(newprops)
    if REQUEST is not None:
        message = "Property %s moved to the end" % property_id
        return self.manage_propertiesForm(self,REQUEST, manage_tabs_message=message)
    return
PropertyManager.security.declareProtected(manage_properties, 'manage_moveEnd')
PropertyManager.manage_moveEnd = manage_moveEnd


InitializeClass(PropertyManager)

