# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/ZMIPatch/OFS_Traversable/__init__.py,v 1.2 2004/07/01 10:46:37 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#


__doc__ = """OFS_Traversable."""
__version__='$Revision: 1.2 $'[11:-2]

from zLOG import LOG, INFO, BLATHER


#LOG('ZMIPatch', BLATHER, 'ZMI Patch:OFS.Traversable', 'ZMI Patch: OFS Traversable: Enable VirtualPath extension and xmlSubOnbjects.\n\t\tJohan Carlsson, 2002-11-24, version Easy Publisher 1.8\n')


import Globals
from Globals import HTML
from string import strip
from types import StringType
from Acquisition import aq_base, aq_parent, aq_inner
from AccessControl import ClassSecurityInfo, getSecurityManager
from Products.EasyCore.EasyCorePermissions  import AccessContentsInformation
from OFS.Traversable import Traversable
from Products.EasyCore.VirtualPathHelpers import path2url
from Products.EasyCore.EasyXMLSupport import navigatable_classes


_marker=[]

NEWLINE="\n"

EASY_VHM = 'EasyVHM'


security = ClassSecurityInfo()

security.declareProtected(AccessContentsInformation, 'rdfPhysicalPath')
def rdfPhysicalPath(self):
    return self.getPhysicalPath()
Traversable.rdfPhysicalPath=rdfPhysicalPath

security.declareProtected(AccessContentsInformation, 'rdfVirtualPath')
def rdfVirtualPath(self):
    vhm = getattr(self, EASY_VHM, None)
    if vhm:
        p_path = self.getPhysicalPath()
        return vhm.get_virtual_path(p_path)
    return p_path
Traversable.rdfVirtualPath=rdfVirtualPath

security.declareProtected(AccessContentsInformation, 'rdfPhysicalURL')
def rdfPhysicalURL(self, ending_slash=1):
    ppath = self.getPhysicalPath()
    return path2url(ppath, ending_slash);
Traversable.rdfPhysicalURL=rdfPhysicalURL

security.declareProtected(AccessContentsInformation, 'rdfVirtualURL')
def rdfVirtualURL(self):
    return path2url(self.rdfVirtualPath(), ending_slash=1)
Traversable.rdfVirtualURL=rdfVirtualURL

security.declareProtected(AccessContentsInformation, 'rdfVirtualParentPath')
def rdfVirtualParentPath(self):
    vpath=self.rdfVirtualPath()
    if vpath:
        return self.rdfVirtualPath()[:-1]
    return None
Traversable.rdfVirtualParentPath=rdfVirtualParentPath

security.declareProtected(AccessContentsInformation, 'rdfVirtualParentURL')
def rdfVirtualParentURL(self):
    vppath=self.rdfVirtualParentPath()
    if vppath is None: return ''
    return path2url(vppath)
Traversable.rdfVirtualParentURL=rdfVirtualParentURL

security.declareProtected(AccessContentsInformation, 'rdfDocumentId')
def rdfDocumentId(self):
    self=aq_base(self)
    return self.getId()
Traversable.rdfDocumentId=rdfDocumentId

security.declareProtected(AccessContentsInformation, 'rdfTitle')
def rdfTitle(self):
    self=aq_base(self)
    return self.title or getattr(self,'navigation_title', '') or self.getId()
Traversable.rdfTitle=rdfTitle

security.declareProtected(AccessContentsInformation, 'rdfNavigationTitle')
def rdfNavigationTitle(self):
    self=aq_base(self)
    return getattr(self,'navigation_title', '') or self.getId()
Traversable.rdfNavigationTitle=rdfNavigationTitle

security.declareProtected(AccessContentsInformation, 'rdfIsRoot')
def rdfIsRoot(self):
    isRoot=getattr(self,'isRoot', None)
    if isRoot is not None:
        return isRoot()
    _iseasyroot = getattr(self, '_isEasyRoot', None)
    if _iseasyroot is not None: return _iseasyroot
    id=self.getId()
    return self.getId()=='Zope'
Traversable.rdfIsRoot=rdfIsRoot

security.declareProtected(AccessContentsInformation, 'rdfDate')
def rdfDate(self):
    date = getattr(self, 'effective_date', None )
    if not date:
        date = self.bobobase_modification_time()
    return date.ISO()
Traversable.rdfDate=rdfDate

security.declareProtected(AccessContentsInformation, 'rdfCreationDate')
def rdfCreationDate(self):
    date = getattr(self, 'creation_date', None )
    if date is None:
        date = self.bobobase_modification_time()
    return date.ISO()
Traversable.rdfCreationDate=rdfCreationDate

security.declareProtected(AccessContentsInformation, 'rdfModificationDate')
def rdfModificationDate(self):
    #Should be update to ModificationDate
    return self.bobobase_modification_time().ISO()
Traversable.rdfModificationDate=rdfModificationDate

security.declareProtected(AccessContentsInformation, 'rdfEffectiveDate')
def rdfEffectiveDate(self):
    return getattr(self,'effective_date', '')
Traversable.rdfEffectiveDate=rdfEffectiveDate

security.declareProtected(AccessContentsInformation, 'rdfExpirationDate')
def rdfExpirationDate(self):
    return getattr(self,'expiration_date', '')
Traversable.rdfExpirationDate=rdfExpirationDate

security.declareProtected(AccessContentsInformation, 'rdfAuthor')
def rdfAuthor(self):
    #This mifht be in the wrong order or something?
    owner = self.getOwner()
    if owner:
        return owner.getId()
    return getattr(self, 'author', '')

Traversable.rdfAuthor=rdfAuthor

security.declareProtected(AccessContentsInformation, 'rdfWorkflowState')
def rdfWorkflowState(self):
    wfs=getattr(self,'getWorkflowState', None)
    if wfs: return wfs()
    return ''
Traversable.rdfWorkflowState=rdfWorkflowState

security.declareProtected(AccessContentsInformation, 'rdfIcon')
def rdfIcon(self):
    #We need a default icon
    geticon=getattr(self,'getIcon', None) or getattr(self, 'icon', '')
    if callable(geticon): geticon=geticon()
    return geticon
Traversable.rdfIcon=rdfIcon

security.declareProtected(AccessContentsInformation, 'rdfMetaType')
def rdfMetaType(self):
    self=aq_base(self)
    return getattr(self,'meta_type', 'no meta type')
Traversable.rdfMetaType=rdfMetaType

security.declareProtected(AccessContentsInformation, 'rdfContentType')
def rdfContentType(self):
    self=aq_base(self)
    return getattr(aq_base(self), 'content_type', '') or getattr(self,'meta_type', '') or 'no content type'
Traversable.rdfContentType=rdfContentType

security.declareProtected(AccessContentsInformation, 'rdfHasSubDocuments')
def rdfHasSubDocuments(self, allowed=None):
    self=aq_base(self)
    objectIds=getattr(self,'objectIds', None)
    if objectIds is not None and callable(objectIds):
        objectIds=objectIds(spec=allowed)
    return not not objectIds
Traversable.rdfHasSubDocuments=rdfHasSubDocuments

security.declareProtected(AccessContentsInformation, 'rdfIsAnObjectManager')
def rdfIsAnObjectManager(self):
    self=aq_base(self)
    return getattr(self, 'isAnObjectManager', 0)
Traversable.rdfIsAnObjectManager=rdfIsAnObjectManager



security.declareProtected(AccessContentsInformation, 'rdfIsCurrent')
def rdfIsCurrent(self, current_paths):
     return self.getPhysicalPath() == current_paths[-1]
Traversable.rdfIsCurrent=rdfIsCurrent


security.declareProtected(AccessContentsInformation, 'rdfInPath')
def rdfInPath(self, current_paths):
    return self.getPhysicalPath() in current_paths
Traversable.rdfInPath=rdfInPath


security.declareProtected(AccessContentsInformation, 'rdfVirtualRootPath')
def rdfVirtualRootPath(self, path=None):
    if path is None: path=self.getPhysicalPath()
    vhm = getattr(self, EASY_VHM, None)
    if vhm:
        return vhm.get_site_root_path(path)
    return ('',)

Traversable.rdfVirtualRootPath=rdfVirtualRootPath

security.declareProtected(AccessContentsInformation, 'rdfSiteRoot')
def rdfSiteRoot(self):
    #NEEDS TO BE IMPLEMENTED
    return self.unrestrictedTraverse(self.getVirtualRootPath())
Traversable.rdfSiteRoot=rdfSiteRoot


security.declareProtected(AccessContentsInformation, 'xmlTraverse')
def xmlTraverse(self, path, root, default=_marker, restricted=1):

    get=getattr
    has=hasattr
    N=None
    M=_marker

    if type(path) is StringType: path = path.split('/')
    else: path=list(path)

    REQUEST={'TraversalRequestNameStack': path}
    XMLPARENTS=[]
    path.reverse()
    pop=path.pop

    if len(path) > 1 and not path[0]:
        # Remove trailing slash
        path.pop(0)

    if restricted: securityManager=getSecurityManager()
    else: securityManager=N

    if not path[-1]:
        # If the path starts with an empty string, go to the root first.
        pop()
        self=root
        if (restricted and not securityManager.validateValue(self)):
            raise Unauthorized, name

    object = self
    XMLPARENTS.append(object)

    while path:
        name=pop()
        __traceback_info__ = path, name

        if name[0] == '_':
            # Never allowed in a URL.
            raise 'NotFound', name

        if name=='..':
            o=getattr(object, 'aq_parent', M)
            if o is not M:
                if (restricted and not securityManager.validate(
                    object, object,name, o)):
                    raise Unauthorized, name
                object=o
                continue

        t=get(object, '__bobo_traverse__', N)
        if t is not N:
            o=t(REQUEST, name)

            if restricted:
                container = N
                if aq_base(o) is not o:
                    # The object is wrapped, so the acquisition
                    # context determines the container.
                    container = aq_parent(aq_inner(o))
                elif has(o, 'im_self'):
                    container = o.im_self
                elif (has(get(object, 'aq_base', object), name)
                      and get(object, name) == o):
                    container = object
                if (not securityManager.validate(object,
                                                 container, name, o)):
                    raise Unauthorized, name

        else:
            if restricted:
                o = guarded_getattr(object, name, M)
            else:
                o = get(object, name, M)
            if o is M:
                o=object[name]
                if (restricted and not securityManager.validate(object, object, N, o)):
                    raise Unauthorized, name

        object=o
        XMLPARENTS.append(object)

    return XMLPARENTS
Traversable.xmlTraverse=xmlTraverse

#
security.declareProtected(AccessContentsInformation, 'xmlSubObjects')
def xmlSubObjects(self, document_url=None, allowed=None, RESPONSE=None):
    """xmlSubObjects"""
    if document_url is None:
        document_url = self.rdfVirtualURL()

    if allowed is None:
        allowed = navigatable_classes.keys()
    else:
        if type(allowed) == type('') and allowed:
            allowed = map(strip, allowed.split(','))
        elif type(allowed) == type(''):
            allowed = navigatable_classes.keys()
        if type(allowed) == type(()): allowed = list(filter)
        if 'EasyRoot' not in allowed: allowed.append('EasyRoot')
    root=self.rdfSiteRoot()
    xml_parents = self.xmlTraverse(document_url,root,default=_marker)
    treeinfo=''
    if xml_parents:
        current_paths=map(lambda p: p.getPhysicalPath(), xml_parents)
        xml_parents.reverse()
        xml_object=xml_parents[0]
        treeinfo = treeinfo +xmlObjectsStart+xmlObjectStart+"\n"
        treeinfo = treeinfo + buildTree(xml_parents, current_paths, allowed)
        treeinfo = treeinfo +"\n"+xmlObjectStop+xmlObjectsStop
        objects=renderObjects(xml_object, current_paths, allowed)
    RESPONSE.setHeader('Content-Type', 'text/XML')
    treeinfo = treeinfo % (objects,)
    code, message,traceback='','',''
    result = xmlResult % (code, message,traceback)
    siteinformation = xmlSiteInformation(self)
    bookmarks= xmlBookmarks(self)
    treestart = xmlTreeStart % (result+siteinformation+bookmarks,)
    return treestart + treeinfo + xmlTreeStop

Traversable.xmlSubObjects=xmlSubObjects


def renderObjects(xml_object, current_paths, allowed):
    result=""
    for obj in xml_object.objectValues(spec=allowed):
        props=xmlObjectProperties(obj,current_paths=current_paths, allowed=allowed)
        result=result+xmlObjectStart+"\n"+props+xmlObjectStop
    return result

def buildTree(xml_path, current_paths, allowed):
    xml_object=xml_path.pop()
    result=xmlObjectProperties(xml_object,current_paths=current_paths, allowed=allowed)
    result=result+xmlObjectsStart
    if xml_path:
        subobjects=buildTree(xml_path, current_paths, allowed)
        if subobjects:
            result=result+xmlObjectStart+"\n"+subobjects+xmlObjectStop
    else:
        result=result+r"%s"
    result=result+xmlObjectsStop
    return result


xmlTreeStart = """<?xml version="1.0" encoding="ISO-8859-1"?><body>%s<tree>"""
xmlResult = """\
<result>
  <result_code>%s</result_code>
  <result_message>%s</result_message>
  <result_traceback>%s</result_traceback>
</result>\
"""
xmlSiteInformation = HTML("""\
<site_information>
  <site_url></site_url>
  <site_email></site_email>
  <isEasyVHM></isEasyVHM>
</site_information>\
""", globals())

xmlBookmarks = HTML("""\
<bookmarks>
</bookmarks>\
""", globals())

xmlTreeStop =  """</tree></body>"""
xmlObjectsStart = """<objects>"""
xmlObjectsStop =  """</objects>"""
xmlObjectStart = """<object>"""
xmlObjectStop =  """</object>"""

xmlObjectProperties = HTML( """\
<document_id><dtml-var rdfDocumentId></document_id>
<url><dtml-var rdfVirtualURL></url>
<parentURL><dtml-var rdfVirtualParentURL></parentURL >
<title><![CDATA[<dtml-var rdfTitle>]]></title>
<navigation_title><![CDATA[<dtml-var rdfNavigationTitle>]]></navigation_title>
<icon><dtml-var rdfIcon></icon>
<meta_type><dtml-var rdfMetaType></meta_type>
<content_type><dtml-var rdfContentType></content_type>
<date><dtml-var rdfDate></date>
<creation_date><dtml-var rdfCreationDate></creation_date>
<modification_date><dtml-var rdfModificationDate></modification_date>
<effective_date><dtml-var rdfEffectiveDate></effective_date>
<expiration_date><dtml-var rdfExpirationDate></expiration_date>
<author><dtml-var rdfAuthor></author>
<workflowstate><dtml-var rdfWorkflowState></workflowstate>
<hasSubDocuments><dtml-var "rdfHasSubDocuments(allowed=allowed)"></hasSubDocuments>
<isAnObjectManager><dtml-var rdfIsAnObjectManager></isAnObjectManager>
<isRoot><dtml-var rdfIsRoot></isRoot>
<iscurrent><dtml-var "rdfIsCurrent(current_paths)"></iscurrent>
<inpath><dtml-var "rdfInPath(current_paths)"></inpath>
""", globals())


Traversable.security=security
Globals.InitializeClass(Traversable)
