#!/usr/local/bin/python
#
# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/utils/DBPackScript.py,v 1.6 2003/03/05 16:13:07 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""
Calls Zope to pack the database at regular intevals

Usage:
    DBPackScript.py (-u <username>) (-p <password>) url

    exit(0) -- the packing did 302, script executed successfully
    exit(1) -- the pack wasn't attempted due to invalid configuration of the script
    exit(2) -- the packing didn't return 302 nor 500
    exit(3) -- the packing did 500, that is no packing needed
    exit(4) -- unauthorized 401


"""
__version__='$Revision: 1.6 $'[11:-2]

import sys
from urlparse import urlparse
from urllib import urlencode
from httplib import HTTPConnection, HTTPSConnection
from base64 import encodestring

# http://localhost:8080/Control_Panel/Database/manage_pack
url='http://localhost:8080/Control_Panel/Database/manage_pack'
username='***'
password='***'
days=1 #days:float
verbose=0

default_args={'url':url,'u':username,'p':password,'days':days, 'v':verbose}

def parseArgs(default={}):
    for arg in sys.argv[1:]:
        arg = arg.split('=')
        if len(arg)==2:
            key, value=arg
            key=key.strip().lower()
            value=value.strip()
            default[key]=value
        elif len(arg)==1:
            key=arg[0].strip().lower()
            default[key]=1
        elif len(arg)>2:
            key, value= arg[0], a[1:]
            key=key.strip().lower()
            value='='.join(value)
            default[key]=value
    return default

def get_authtoken(username,password, atype='Basic'):
    if atype=='Basic':
        authorization=(
            "Basic %s" % (encodestring('%s:%s' % (username,password))).replace('\012',''))
        return authorization
    raise ValueError, 'Unknown authentication scheme: %s' % atype


def zopeRequest(url, username=None, password=None, days=0, headers={}):
    """Send a request back to Zope"""

    scheme, host, path = urlparse(url)[:3]
    ssl = scheme == 'https'

    if ssl:
        h = HTTPSConnection(host)
    else:
        h = HTTPConnection(host)

    body=urlencode({'days:float':days})

    h.putrequest('POST', path)
    h.putheader('User-Agent', 'DBPackScript/%s' % __version__)
    h.putheader('Connection', 'close')

    for header, value in headers.items():
        h.putheader(header, value)
    
    h.putheader("Content-Length", str(len(body)))
    

    if username is not None and password is not None:
        h.putheader("Authorization", get_authtoken(username,password))

    h.endheaders()
    h.send(body)
    return h.getresponse()



if __name__=='__main__':
    args=parseArgs(default_args)
    url=args.get('url',None)
    username=args.get('u',None)
    password=args.get('p',None)
    verbose=args.get('v',None)
    days=args.get('days',None)
    #ssl=args.get('s',None)
    if url is not None:
        file=open('DBPack.log','a')
        r=zopeRequest(url,username=username, password=password, days=days)
        
        if verbose:
            print "Status:   %i" % r.status
            print "Url:      %s" % url
            print "Username: %s" % username
            print "Password: %s" % ('*' * len(password),)
            print "Days:     %s" % days
        file.write("=" *55)
        file.write("\n")
        file.write("Status:   %i\n" % r.status)
        file.write("Url:      %s\n" % url)
        file.write("Username: %s\n" % username)
        file.write("Password: %s\n" % ('*' * len(password),))
        file.write("Days:     %s\n" % str(days))
        file.write("-" *55)
        file.write("\n\n")

        if r.status==302:
            if verbose: print "Script executed successfully"
            file.write("Script executed successfully\n")
            file.write("-" *55)
            file.write("\n\n")
            sys.exit(0)
        if r.status==500:
            if verbose: print "No packing needed!"
            file.write("No packing needed!\n")
            file.write("-" *55)
            file.write("\n\n")
            sys.exit(3)
        if r.status==401:
            if verbose: print "Unauthorized!"
            file.write("Unauthorized!\n")
            file.write("-" *55)
            file.write("\n\n")
            sys.exit(4)

        if verbose: print "Something went wrong."
        file.write("Something went wrong.\n")
        file.write("-" *55)
        file.write("\n\n")
        sys.exit(1)
        file.close()
        
    print "Invalid URL"        
    sys.exit(1)
